@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -26,7 +26,7 @@
  * \brief Various types to use for strings and unicode.
  * \author Julien Jorge
  */
-#ifndef __CHARSET_DEF__HPP__
+#ifndef __CHARSET_DEF_HPP__
 #define __CHARSET_DEF_HPP__
 
 #include <string>
