@@ -2,7 +2,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -36,6 +36,8 @@
 #include "engine/level.hpp"
 #include "universe/collision_info.hpp"
 
+#include "visual/scene_element_sequence.hpp"
+
 /*----------------------------------------------------------------------------*/
 bear::engine::base_item::id_type bear::engine::base_item::s_next_id = 1;
 
@@ -79,8 +81,8 @@ void bear::engine::base_item::print_allocated()
  * \brief Contructor.
  */
 bear::engine::base_item::base_item()
-  : m_id( s_next_id ), m_layer(NULL), m_z_position(0), m_z_fixed(true),
-    m_dying(false)
+  : m_id( s_next_id ), m_layer(NULL), m_z_position(0),
+    m_flags(item_flag_z_fixed), m_dying(false)
 {
   ++s_next_id;
 #ifndef NDEBUG
@@ -94,7 +96,7 @@ bear::engine::base_item::base_item()
  */
 bear::engine::base_item::base_item( const base_item& that )
   : level_object(that), physical_item(that), m_id( s_next_id ), m_layer(NULL),
-    m_z_position(that.get_z_position()), m_z_fixed(that.m_z_fixed),
+    m_z_position(that.get_z_position()), m_flags(item_flag_z_fixed),
     m_dying(false)
 {
   ++s_next_id;
@@ -150,7 +152,6 @@ void bear::engine::base_item::build()
  */
 void bear::engine::base_item::destroy()
 {
-  CLAW_PRECOND( m_layer != NULL );
   // nothing to do
 } // base_item::destroy()
 
@@ -166,7 +167,51 @@ void bear::engine::base_item::progress( universe::time_type elapsed_time )
 
 /*----------------------------------------------------------------------------*/
 /**
- * \brief Get the sprite representing the item.
+ * \brief Insert the visual of the item at the end of a given list.
+ * \param visuals The list in which the visual is added.
+ *
+ * This method uses get_visual() to get the visuals and make a
+ * visual::scene_element_sequence of them.
+ */
+void
+bear::engine::base_item::insert_visual( std::list<scene_visual>& visuals ) const
+{
+  std::list<scene_visual> v;
+  get_visual( v );
+
+  if ( v.size() > 1 )
+    {
+      v.sort( scene_visual::z_position_compare() );
+      visual::scene_element_sequence e;
+
+      for ( ; !v.empty() ; v.pop_front() )
+        {
+#ifndef NDEBUG
+          const visual::rectangle_type r
+            ( v.front().scene_element.get_bounding_box() );
+          if ( (r.width() == 0) || (r.height() == 0) )
+            claw::logger << claw::log_warning
+                         << "Empty visual::scene_element is inserted in a "
+                         << "visual::scene_element_sequence by '"
+                         << get_class_name() << "'. This should be avoided."
+                         << std::endl;
+#endif // NDEBUG
+
+          e.push_back(v.front().scene_element);
+        }
+
+      visuals.push_back( scene_visual(e, get_z_position()) );
+    }
+  else if (v.size() == 1)
+    {
+      visuals.push_back( v.front() );
+      visuals.back().z_position = get_z_position();
+    }
+} // base_item::insert_visual()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the visuals representing the item.
  * \param visuals (out) The sprites of the item, and their positions.
  */
 void
@@ -185,10 +230,6 @@ bear::engine::base_item::get_visual( std::list<scene_visual>& visuals ) const
 bool bear::engine::base_item::set_u_integer_field
 ( const std::string& name, unsigned int value )
 {
-  claw::logger << claw::log_warning
-               << "base_item::set_u_integer_field(): '" << name
-               << "' hasn't been set." << std::endl;
-
   return false;
 } // base_item::set_u_integer_field()
 
@@ -207,12 +248,7 @@ bear::engine::base_item::set_integer_field( const std::string& name, int value )
   if (name == "base_item.position.depth")
     m_z_position = value;
   else
-    {
-      claw::logger << claw::log_warning
-                   << "base_item::set_integer_field(): '" << name
-                   << "' hasn't been set." << std::endl;
-      ok = false;
-    }
+    ok = false;
 
   return ok;
 } // base_item::set_integer_field()
@@ -247,13 +283,12 @@ bear::engine::base_item::set_real_field( const std::string& name, double value )
     set_hardness(value);
   else if (name == "base_item.system_angle")
     set_system_angle(value);
+  else if (name == "base_item.speed.x")
+    set_speed( universe::speed_type(value, get_speed().y) );
+  else if (name == "base_item.speed.y")
+    set_speed( universe::speed_type(get_speed().x, value) );
   else
-    {
-      claw::logger << claw::log_warning
-                   << "base_item::set_real_field(): '" << name
-                   << "' hasn't been set." << std::endl;
-      ok = false;
-    }
+    ok = false;
 
   return ok;
 } // base_item::set_real_field()
@@ -280,21 +315,18 @@ bear::engine::base_item::set_bool_field( const std::string& name, bool value )
     set_phantom(value);
   else if (name == "base_item.fixed.x")
     {
-      if ( value ) 
-	add_position_constraint_x();
+      if ( value )
+        add_position_constraint_x();
     }
   else if (name == "base_item.fixed.y")
     {
-      if ( value ) 
-	add_position_constraint_y();
+      if ( value )
+        add_position_constraint_y();
     }
+  else if (name == "base_item.fixed.z")
+    set_z_fixed(value);
   else
-    {
-      claw::logger << claw::log_warning
-                   << "base_item::set_bool_field(): '" << name
-                   << "' hasn't been set." << std::endl;
-      ok = false;
-    }
+    ok = false;
 
   return ok;
 } // base_item::set_bool_field()
@@ -309,10 +341,6 @@ bear::engine::base_item::set_bool_field( const std::string& name, bool value )
 bool bear::engine::base_item::set_string_field
 ( const std::string& name, const std::string& value )
 {
-  claw::logger << claw::log_warning
-               << "base_item::set_string_field(): '" << name
-               << "' hasn't been set." << std::endl;
-
   return false;
 } // base_item::set_string_field()
 
@@ -326,10 +354,6 @@ bool bear::engine::base_item::set_string_field
 bool bear::engine::base_item::set_item_field
 ( const std::string& name, base_item* value )
 {
-  claw::logger << claw::log_warning
-               << "base_item::set_item_field(): '" << name
-               << "' hasn't been set." << std::endl;
-
   return false;
 } // base_item::set_item_field()
 
@@ -343,10 +367,6 @@ bool bear::engine::base_item::set_item_field
 bool bear::engine::base_item::set_sprite_field
 ( const std::string& name, const visual::sprite& value )
 {
-  claw::logger << claw::log_warning
-               << "base_item::set_sprite_field(): '" << name
-               << "' hasn't been set." << std::endl;
-
   return false;
 } // base_item::set_sprite_field()
 
@@ -360,13 +380,25 @@ bool bear::engine::base_item::set_sprite_field
 bool bear::engine::base_item::set_animation_field
 ( const std::string& name, const visual::animation& value )
 {
-  claw::logger << claw::log_warning
-               << "base_item::set_animation_field():  '" << name
-               << "' hasn't been set." << std::endl;
-
   return false;
 } // base_item::set_animation_field()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set a field of type \c audio::sample.
+ * \param name The name of the field.
+ * \param value The new value of the field. The item receiving this value must
+ *        release de memory with a delete at some time.
+ * \return false if the field "name" is unknow, true otherwise.
+ */
+bool bear::engine::base_item::set_sample_field
+( const std::string& name, audio::sample* value )
+{
+  delete value;
+
+  return false;
+} // base_item::set_sample_field()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Set a field of type \c list of unsigned integer.
@@ -377,10 +409,6 @@ bool bear::engine::base_item::set_animation_field
 bool bear::engine::base_item::set_u_integer_list_field
 ( const std::string& name, const std::vector<unsigned int>& value )
 {
-  claw::logger << claw::log_warning
-               << "base_item::set_u_integer_list_field(): '" << name
-               << "' hasn't been set." << std::endl;
-
   return false;
 } // base_item::set_u_integer_list_field()
 
@@ -395,9 +423,6 @@ bool
 bear::engine::base_item::set_integer_list_field
 ( const std::string& name, const std::vector<int>& value )
 {
-  claw::logger << claw::log_warning
-               << "base_item::set_integer_list_field(): '" << name
-               << "' hasn't been set." << std::endl;
   return false;
 } // base_item::set_integer_list_field()
 
@@ -412,10 +437,6 @@ bool
 bear::engine::base_item::set_real_list_field
 ( const std::string& name, const std::vector<double>& value )
 {
-  claw::logger << claw::log_warning
-               << "base_item::set_real_list_field(): '" << name
-               << "' hasn't been set." << std::endl;
-
   return false;
 } // base_item::set_real_list_field()
 
@@ -430,10 +451,6 @@ bool
 bear::engine::base_item::set_bool_list_field
 ( const std::string& name, const std::vector<bool>& value )
 {
-  claw::logger << claw::log_warning
-               << "base_item::set_bool_list_field(): '" << name
-               << "' hasn't been set." << std::endl;
-
   return false;
 } // base_item::set_bool_list_field()
 
@@ -447,10 +464,6 @@ bear::engine::base_item::set_bool_list_field
 bool bear::engine::base_item::set_string_list_field
 ( const std::string& name, const std::vector<std::string>& value )
 {
-  claw::logger << claw::log_warning
-               << "base_item::set_string_list_field(): '" << name
-               << "' hasn't been set." << std::endl;
-
   return false;
 } // base_item::set_string_list_field()
 
@@ -464,10 +477,6 @@ bool bear::engine::base_item::set_string_list_field
 bool bear::engine::base_item::set_item_list_field
 ( const std::string& name, const std::vector<base_item*>& value )
 {
-  claw::logger << claw::log_warning
-               << "base_item::set_item_list_field(): '" << name
-               << "' hasn't been set." << std::endl;
-
   return false;
 } // base_item::set_item_list_field()
 
@@ -481,10 +490,6 @@ bool bear::engine::base_item::set_item_list_field
 bool bear::engine::base_item::set_sprite_list_field
 ( const std::string& name, const std::vector<visual::sprite>& value )
 {
-  claw::logger << claw::log_warning
-               << "base_item::set_sprite_list_field(): '" << name
-               << "' hasn't been set." << std::endl;
-
   return false;
 } // base_item::set_sprite_list_field()
 
@@ -498,13 +503,26 @@ bool bear::engine::base_item::set_sprite_list_field
 bool bear::engine::base_item::set_animation_list_field
 ( const std::string& name, const std::vector<visual::animation>& value )
 {
-  claw::logger << claw::log_warning
-               << "base_item::set_animation_list_field():  '" << name
-               << "' hasn't been set." << std::endl;
-
   return false;
 } // base_item::set_animation_list_field()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set a field of type \c list of audio::sample.
+ * \param name The name of the field.
+ * \param value The new value of the field. The item receiving this value must
+ *        release de memory with a delete at some time.
+ * \return false if the field "name" is unknow, true otherwise.
+ */
+bool bear::engine::base_item::set_sample_list_field
+( const std::string& name, const std::vector<audio::sample*>& value )
+{
+  for (std::size_t i=0; i!=value.size(); ++i)
+    delete value[i];
+
+  return false;
+} // base_item::set_sample_list_field()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell if the item is correctly initialized.
@@ -540,9 +558,10 @@ const char* bear::engine::base_item::get_class_name() const
 void bear::engine::base_item::kill()
 {
   claw::logger << claw::log_verbose
-               << "Killing id #" << m_id << std::endl;
+               << "Killing id #" << m_id << ' ' << this << std::endl;
 
   CLAW_PRECOND( m_layer != NULL );
+  CLAW_PRECOND( !get_insert_as_static() );
 
   if (!m_dying)
     {
@@ -552,6 +571,15 @@ void bear::engine::base_item::kill()
     }
 } // base_item::kill()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Indicates if the fonction "kill" has been called.
+ */
+bool bear::engine::base_item::is_dead() const
+{
+  return m_dying;
+} // base_item::is_dead()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell the environment in which the item is.
@@ -595,7 +623,10 @@ int bear::engine::base_item::get_z_position() const
  */
 void bear::engine::base_item::set_z_fixed( bool fixed )
 {
-  m_z_fixed = fixed;
+  if (fixed)
+    m_flags |= item_flag_z_fixed;
+  else
+    m_flags &= ~item_flag_z_fixed;
 } // base_item::set_z_fixed()
 
 /*----------------------------------------------------------------------------*/
@@ -604,9 +635,28 @@ void bear::engine::base_item::set_z_fixed( bool fixed )
  */
 bool bear::engine::base_item::is_z_fixed() const
 {
-  return m_z_fixed;
+  return m_flags & item_flag_z_fixed;
 } // base_item::is_z_fixed()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Call this method before inserting the item in a level to ensure it
+ *        will be static in the world.
+ */
+void bear::engine::base_item::set_insert_as_static()
+{
+  m_flags |= item_flag_insert_as_static;
+} // base_item::set_insert_as_static()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Tell if this item has to be inserted as a static item in the level.
+ */
+bool bear::engine::base_item::get_insert_as_static() const
+{
+  return m_flags & item_flag_insert_as_static;
+} // base_item::get_insert_as_static()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell that this item is always displayed.
@@ -663,6 +713,18 @@ const bear::engine::layer& bear::engine::base_item::get_layer() const
   return *m_layer;
 } // base_item::get_layer()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the layer in which the item is.
+ * \pre m_layer != NULL
+ */
+bear::engine::layer& bear::engine::base_item::get_layer()
+{
+  CLAW_PRECOND( m_layer != NULL );
+
+  return *m_layer;
+} // base_item::get_layer()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Add an item in the same environment of \a this.
@@ -675,6 +737,37 @@ void bear::engine::base_item::new_item( base_item& item ) const
   m_layer->add_item(item);
 } // base_item::new_item()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Export the methods of the class.
+ */
+void bear::engine::base_item::init_exported_methods()
+{
+  TEXT_INTERFACE_CONNECT_PARENT_METHOD_0
+    ( base_item, universe::physical_item_state,
+      remove_position_constraint_x, void );
+
+  TEXT_INTERFACE_CONNECT_PARENT_METHOD_0
+    ( base_item, universe::physical_item_state,
+      remove_position_constraint_y, void );
+
+  TEXT_INTERFACE_CONNECT_PARENT_METHOD_0
+    ( base_item, universe::physical_item_state,
+      add_position_constraint_x, void );
+
+  TEXT_INTERFACE_CONNECT_PARENT_METHOD_0
+    ( base_item, universe::physical_item_state,
+      add_position_constraint_y, void );
+
+  TEXT_INTERFACE_CONNECT_METHOD_1
+    ( base_item, set_z_position, void, int );
+
+  TEXT_INTERFACE_CONNECT_PARENT_METHOD_2
+    ( base_item, universe::physical_item_state,
+      set_center_of_mass, void,
+      const universe::coordinate_type&, const universe::coordinate_type& );
+} // god::init_exported_methods()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Cast the other item and call collision(base_item, collision_info).
@@ -703,3 +796,7 @@ void bear::engine::base_item::time_step( universe::time_type elapsed_time )
   if (!m_dying)
     progress(elapsed_time);
 } // base_item::time_step()
+
+/*----------------------------------------------------------------------------*/
+TEXT_INTERFACE_IMPLEMENT_METHOD_LIST( bear::engine::base_item )
+
