@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -30,8 +30,12 @@
 
 #include <algorithm>
 #include <claw/functional.hpp>
+
 #include "engine/game.hpp"
-#include "engine/level_loader.hpp"
+#include "engine/level_globals.hpp"
+#include "engine/layer/gui_layer.hpp"
+#include "engine/variable/base_variable.hpp"
+#include "universe/const_item_handle.hpp"
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -83,7 +87,7 @@ void bear::engine::level::start()
  */
 void bear::engine::level::stop()
 {
-  m_level_globals->stop_music(m_music_id);
+  stop_music();
 } // level::stop()
 
 /*----------------------------------------------------------------------------*/
@@ -120,28 +124,8 @@ void bear::engine::level::progress( universe::time_type elapsed_time )
  */
 void bear::engine::level::render( visual::screen& screen ) const
 {
-  for (unsigned int i=0; i!=m_layers.size(); ++i)
-    {
-      // get four times the area covered by the camera
-      region_type r;
-      add_region( r, get_camera_focus(), get_camera_size() );
-
-      universe::rectangle_type active( r.front() );
-      std::list<scene_visual> visuals;
-
-      get_layer_area(i, active); // the active area scaled in the layer
-
-      m_layers[i]->get_visual( visuals, active );
-      visuals.sort( scene_visual::z_position_compare() );
-
-      universe::rectangle_type area( get_camera_focus() );
-      get_layer_area(i, area);   // the camera scaled in the layer
-      render( visuals, area.top_left(), screen,
-              (double)screen.get_size().x / get_camera_size().x,
-              (double)screen.get_size().y / get_camera_size().y );
-    }
-
-  m_gui.render( screen );
+  render_layers(screen);
+  render_gui(screen);
 } // level::render()
 
 /*----------------------------------------------------------------------------*/
@@ -156,11 +140,64 @@ bear::visual::scene_element bear::engine::level::element_to_screen_coordinates
   const universe::rectangle_type area( get_camera_focus() );
 
   return element_to_screen_coordinates
-    ( e, area.top_left(),
+    ( e, area.bottom_left(),
       (double)game::get_instance().get_screen_size().x / get_camera_size().x,
       (double)game::get_instance().get_screen_size().y / get_camera_size().y );
 } // level::element_to_screen_coordinates()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Create a big picture of the level.
+ * \param screen The screen on which we do the captured drawings.
+ * \param img The image in which we save the resulting capture.
+ */
+void bear::engine::level::shot
+( visual::screen& screen, claw::graphic::image& img ) const
+{
+  img.set_size( get_size().x, get_size().y );
+  claw::logger << claw::log_verbose << "Level image allocated." << std::endl;
+
+  claw::graphic::image tmp(screen.get_size().x, screen.get_size().y);
+  const double ratio = 1;
+  const visual::color_type c = screen.get_background_color();
+  screen.set_background_color( claw::graphic::white_pixel );
+
+  for ( unsigned int y=0; y<img.height(); y+=tmp.height() )
+    for ( unsigned int x=0; x<img.width(); x+=tmp.width() )
+      {
+        const universe::rectangle_type area
+          ( x, y, x+screen.get_size().x, y+screen.get_size().y );
+
+        claw::logger << claw::log_verbose << "rendering level x=" << x
+                     << " y=" << y << std::endl;
+
+        screen.begin_render();
+        for (unsigned int i=0; i!=m_layers.size(); ++i)
+          if ( m_layers[i]->get_size() == get_size() )
+            {
+              // get four times the area covered by the camera
+              region_type r;
+              add_region( r, area, area.size() );
+
+              universe::rectangle_type active( r.front() );
+              std::list<scene_visual> visuals;
+
+              m_layers[i]->get_visual( visuals, active );
+              visuals.sort( scene_visual::z_position_compare() );
+
+              render( visuals, area.bottom_left(), screen, ratio, ratio );
+            }
+        screen.end_render();
+
+        screen.shot(tmp);
+        tmp.flip();
+        img.partial_copy(tmp, claw::math::coordinate_2d<int>(x, y));
+      }
+
+  img.flip();
+  screen.set_background_color(c);
+} // level::shot()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the size of the level.
@@ -202,7 +239,7 @@ bear::engine::level_globals& bear::engine::level::get_globals()
  * \brief Set the item to use as the camera.
  * \param cam The camera.
  */
-void bear::engine::level::set_camera( const base_item& cam )
+void bear::engine::level::set_camera( base_item& cam )
 {
   m_camera = cam;
 } // level::set_camera()
@@ -313,6 +350,24 @@ void bear::engine::level::play_music()
     m_music_id = m_level_globals->play_music( m_music );
 } // level::play_music()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Stop the music of the level.
+ */
+void bear::engine::level::stop_music()
+{
+  m_level_globals->stop_music(m_music_id);
+} // level::stop_music()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the current camera.
+ */
+bear::universe::item_handle bear::engine::level::get_camera()
+{
+  return m_camera;
+} // level::get_camera()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the size of the camera.
@@ -353,22 +408,78 @@ bear::universe::position_type bear::engine::level::get_camera_center() const
 
 /*----------------------------------------------------------------------------*/
 /**
- * \brief Get the global variables of the level.
+ * \brief Get the value of a global variable.
+ * \param val (in/out) The variable for which we want the value.
  */
-bear::engine::level::var_map& bear::engine::level::get_level_variables()
+void bear::engine::level::get_level_variable( base_variable& val ) const
 {
-  return m_level_variables;
-} // game::get_level_variables()
+  val.get_value_from(m_level_variables);
+} // level::get_level_variable()
 
 /*----------------------------------------------------------------------------*/
 /**
- * \brief Get the global variables of the level.
+ * \brief Set the value of a global variable.
+ * \param val (in/out) The variable for which we set the value.
  */
-const bear::engine::level::var_map&
-bear::engine::level::get_level_variables() const
+void bear::engine::level::set_level_variable( const base_variable& val )
 {
-  return m_level_variables;
-} // game::get_level_variables()
+  val.assign_value_to(m_level_variables);
+} // level::set_level_variable()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Tell if a given variable exists in the level.
+ * \param val (in/out) The variable to check.
+ */
+bool
+bear::engine::level::level_variable_exists( const base_variable& val ) const
+{
+  return val.exists(m_level_variables);
+} // level::level_variable_exists()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Draw the visible part of the level layers on the screen.
+ * \param screen The screen on which we draw.
+ */
+void bear::engine::level::render_layers( visual::screen& screen ) const
+{
+  const double r_x = (double)screen.get_size().x / get_camera_size().x;
+  const double r_y = (double)screen.get_size().y / get_camera_size().y;
+
+  for (unsigned int i=0; i!=m_layers.size(); ++i)
+    {
+      // get four times the area covered by the camera
+      region_type r;
+      add_region( r, get_camera_focus(), get_camera_size() );
+
+      universe::rectangle_type active( r.front() );
+      std::list<scene_visual> visuals;
+
+      get_layer_area(i, active); // the active area scaled in the layer
+
+      m_layers[i]->get_visual( visuals, active );
+      visuals.sort( scene_visual::z_position_compare() );
+
+      universe::rectangle_type area( get_camera_focus() );
+      get_layer_area(i, area);   // the camera scaled in the layer
+      render( visuals, area.bottom_left(), screen, r_x, r_y );
+    }
+} // level::render_layers()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Draw the visible part of the level layers on the screen.
+ * \param screen The screen on which we draw.
+ */
+void bear::engine::level::render_gui( visual::screen& screen ) const
+{
+  gui_layer_stack::scene_element_list vis;
+  m_gui.render(vis);
+
+  for ( ; !vis.empty(); vis.pop_front() )
+    screen.render( vis.front() );
+} // level::render_gui()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -407,12 +518,12 @@ bear::visual::scene_element bear::engine::level::element_to_screen_coordinates
   visual::scene_element result(e);
 
   // the y-axis of the screen is in reversed direction
-  universe::position_type pos
-    (result.get_position().x - cam_pos.x, cam_pos.y - result.get_position().y);
+  universe::position_type pos( result.get_position() - cam_pos );
 
   result.set_position(pos.x * r_w, pos.y * r_h);
-  result.set_scale_factor(r_w, r_h);
-      
+  result.set_scale_factor
+    ( e.get_scale_factor_x() * r_w, e.get_scale_factor_y() * r_h);
+
   return result;
 } // level::element_to_screen_coordinates()
 
