@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -345,7 +345,18 @@ bear::engine::level_globals::new_sample( const std::string& name )
     load_sound(name);
 
   return m_sound_manager.new_sample(name);
-} // level_globals::get_sound()
+} // level_globals::new_sample()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Create a copy of a sample.
+ * \param s The sample to copy.
+ */
+bear::audio::sample*
+bear::engine::level_globals::new_sample( const audio::sample& s )
+{
+  return m_sound_manager.new_sample(s);
+} // level_globals::new_sample()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -626,7 +637,7 @@ void bear::engine::level_globals::release_item
  * \param msg The message to send to this item.
  */
 bool bear::engine::level_globals::send_message
-( const std::string& target, communication::message& msg )
+( const std::string& target, communication::message& msg ) const
 {
   return m_post_office.send_message( target, msg );
 } // level_globals::send_message()
