@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -23,35 +23,57 @@
 */
 /**
  * \file level_loader.cpp
- * \brief Implementation of the bear::engine::level::loader class.
+ * \brief Implementation of the bear::engine::level_loader class.
  * \author Julien Jorge
  */
 #include "engine/level_loader.hpp"
 
-#include <claw/exception.hpp>
 #include "level_code_value.hpp"
-#include "engine/export.hpp"
+
+#include "engine/base_item.hpp"
+#include "engine/compiled_file.hpp"
+#include "engine/export_function.hpp"
 #include "engine/game.hpp"
+#include "engine/item_factory.hpp"
+#include "engine/level.hpp"
+#include "engine/level_globals.hpp"
+#include "engine/libraries_pool.hpp"
 #include "engine/sprite_loader.hpp"
+#include "engine/layer/layer_creator.hpp"
+#include "engine/layer/layer_factory.hpp"
+
+#include <claw/exception.hpp>
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Constructor.
  * \param f The file from which we load the level.
  */
-bear::engine::level_loader::level_loader
-( compiled_file& f, const std::string& name )
+bear::engine::level_loader::level_loader( compiled_file& f )
   : m_level(NULL), m_layer(NULL), m_file(f), m_current_item(NULL),
     m_items_count(0), m_item_index(0)
 {
-  unsigned int layers_count;
-  std::string level_music;
-  universe::size_box_type level_size;
+  unsigned int maj(0), min(0), rel(0);
 
-  m_file >> level_size.x >> level_size.y >> level_music >> m_items_count
-         >> layers_count >> m_next_code;
+  m_file >> maj >> min >> rel;
 
-  m_level = new level(name, level_size, level_music);
+  if ( (maj != 0) || (min <= 4) )
+    throw CLAW_EXCEPTION( "This version of the level file is not supported." );
+  else
+    {
+      unsigned int layers_count;
+      std::string level_music;
+      universe::size_box_type level_size;
+      std::string name("Anonymous");
+
+      if ( (maj == 0) && (min >= 5) )
+        m_file >> name;
+
+      m_file >> level_size.x >> level_size.y >> level_music >> m_items_count
+             >> layers_count >> m_next_code;
+
+      m_level = new level(name, level_size, level_music);
+    }
 } // level_loader::level_loader()
 
 /*----------------------------------------------------------------------------*/
@@ -153,21 +175,13 @@ bool bear::engine::level_loader::one_step_item()
     case level_code_value::field_sprite    : load_item_field_sprite(); break;
     case level_code_value::field_animation : load_item_field_animation(); break;
     case level_code_value::field_item      : load_item_field_item(); break;
+    case level_code_value::field_sample    : load_item_field_sample(); break;
     default:
       {
         assert( m_current_item != NULL );
 
-        if ( m_fixed )
-          {
-            m_current_item->fix();
-            m_current_item->set_phantom(true);
-          }
-
         if ( !m_current_item->is_valid() )
-          throw CLAW_EXCEPTION( "item is not correctly initialized." );
-
-        claw::logger << claw::log_verbose << "Adding item #"
-                     << m_current_item->get_id() << " in layer." << std::endl;
+          throw claw::exception( "item is not correctly initialized." );
 
         m_layer->add_item( *m_current_item );
 
@@ -231,6 +245,9 @@ void bear::engine::level_loader::load_item_declaration()
   m_file >> n;
   m_referenced.resize(n);
 
+  claw::logger << claw::log_verbose << "Preparing " << n << " items."
+               << std::endl;
+
   for (unsigned int i=0; i!=n; ++i)
     {
       m_file >> class_name;
@@ -249,12 +266,18 @@ void bear::engine::level_loader::load_item_definition()
   CLAW_PRECOND( m_current_item == NULL );
 
   claw::logger << claw::log_verbose << "Defining item #" << m_referenced_index
+               << ' ' << m_referenced[m_referenced_index]->get_class_name()
                << std::endl;
 
+  bool fixed;
+
   m_current_item = m_referenced[m_referenced_index];
-  m_file >> m_fixed >> m_next_code;
+  m_file >> fixed >> m_next_code;
   ++m_referenced_index;
-} // level_loader::load_item_declaration()
+
+  if (fixed)
+    m_current_item->set_insert_as_static();
+} // level_loader::load_item_definition()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -266,10 +289,15 @@ void bear::engine::level_loader::load_item()
 
   std::string class_name;
 
-  m_file >> class_name >> m_fixed >> m_next_code;
+  bool fixed;
+
+  m_file >> class_name >> fixed >> m_next_code;
   ++m_item_index;
 
   m_current_item = create_item_from_string(class_name);
+
+  if (fixed)
+    m_current_item->set_insert_as_static();
 } // level_loader::load_item()
 
 /*----------------------------------------------------------------------------*/
@@ -308,6 +336,9 @@ void bear::engine::level_loader::load_item_field_list()
     case level_code_value::field_item:
       load_item_field_item_list();
       break;
+    case level_code_value::field_sample:
+      load_item_field_sample_list();
+      break;
     }
 } // level_loader::load_item_field_list()
 
@@ -321,7 +352,9 @@ void bear::engine::level_loader::load_item_field_int()
   int val;
   m_file >> field_name >> val >> m_next_code;
 
-  m_current_item->set_integer_field( field_name, val );
+  if ( !m_current_item->set_integer_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_int()
 
 /*----------------------------------------------------------------------------*/
@@ -334,7 +367,9 @@ void bear::engine::level_loader::load_item_field_u_int()
   unsigned int val;
   m_file >> field_name >> val >> m_next_code;
 
-  m_current_item->set_u_integer_field( field_name, val );
+  if ( !m_current_item->set_u_integer_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_u_int()
 
 /*----------------------------------------------------------------------------*/
@@ -347,7 +382,9 @@ void bear::engine::level_loader::load_item_field_real()
   double val;
   m_file >> field_name >> val >> m_next_code;
 
-  m_current_item->set_real_field( field_name, val );
+  if ( !m_current_item->set_real_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_real()
 
 /*----------------------------------------------------------------------------*/
@@ -360,7 +397,9 @@ void bear::engine::level_loader::load_item_field_bool()
   bool val;
   m_file >> field_name >> val >> m_next_code;
 
-  m_current_item->set_bool_field( field_name, val );
+  if ( !m_current_item->set_bool_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_bool()
 
 /*----------------------------------------------------------------------------*/
@@ -375,7 +414,9 @@ void bear::engine::level_loader::load_item_field_string()
   m_file >> field_name >> val >> m_next_code;
   escape(val);
 
-  m_current_item->set_string_field( field_name, val );
+  if ( !m_current_item->set_string_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_string()
 
 /*----------------------------------------------------------------------------*/
@@ -393,7 +434,9 @@ void bear::engine::level_loader::load_item_field_sprite()
 
   m_file >> m_next_code;
 
-  m_current_item->set_sprite_field( field_name, val );
+  if ( !m_current_item->set_sprite_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_sprite()
 
 /*----------------------------------------------------------------------------*/
@@ -404,11 +447,15 @@ void bear::engine::level_loader::load_item_field_animation()
 {
   std::string field_name, path;
 
-  m_file >> field_name >> path >> m_next_code;
+  m_file >> field_name;
 
-  visual::animation val = m_level->get_globals().get_animation(path);
+  visual::animation val =
+    sprite_loader::load_any_animation(m_file, m_level->get_globals());
+  m_file >> m_next_code;
 
-  m_current_item->set_animation_field( field_name, val );
+  if ( !m_current_item->set_animation_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_animation()
 
 /*----------------------------------------------------------------------------*/
@@ -421,9 +468,28 @@ void bear::engine::level_loader::load_item_field_item()
   unsigned int index;
 
   m_file >> field_name >> index >> m_next_code;
-  m_current_item->set_item_field( field_name, m_referenced[index] );
+  if ( !m_current_item->set_item_field( field_name, m_referenced[index] ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_item()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set a field of type sample to an item.
+ */
+void bear::engine::level_loader::load_item_field_sample()
+{
+  std::string field_name;
+
+  m_file >> field_name;
+  audio::sample* s = load_sample_data();
+  m_file >> m_next_code;
+
+  if ( !m_current_item->set_sample_field( field_name, s ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
+} // level_loader::load_item_field_sample()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Set a field of type list of integer to an item.
@@ -433,7 +499,9 @@ void bear::engine::level_loader::load_item_field_int_list()
   std::vector<int> val;
   std::string field_name = load_list<int>(val);
 
-  m_current_item->set_integer_list_field( field_name, val );
+  if ( !m_current_item->set_integer_list_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_int_list()
 
 /*----------------------------------------------------------------------------*/
@@ -445,7 +513,9 @@ void bear::engine::level_loader::load_item_field_u_int_list()
   std::vector<unsigned int> val;
   std::string field_name = load_list<unsigned int>(val);
 
-  m_current_item->set_u_integer_list_field( field_name, val );
+  if ( !m_current_item->set_u_integer_list_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_u_int_list()
 
 /*----------------------------------------------------------------------------*/
@@ -457,7 +527,9 @@ void bear::engine::level_loader::load_item_field_real_list()
   std::vector<double> val;
   std::string field_name = load_list<double>(val);
 
-  m_current_item->set_real_list_field( field_name, val );
+  if ( !m_current_item->set_real_list_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_real_list()
 
 /*----------------------------------------------------------------------------*/
@@ -469,7 +541,9 @@ void bear::engine::level_loader::load_item_field_bool_list()
   std::vector<bool> val;
   std::string field_name = load_list<bool>(val);
 
-  m_current_item->set_bool_list_field( field_name, val );
+  if ( !m_current_item->set_bool_list_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_bool_list()
 
 /*----------------------------------------------------------------------------*/
@@ -495,7 +569,9 @@ void bear::engine::level_loader::load_item_field_string_list()
 
   m_file >> m_next_code;
 
-  m_current_item->set_string_list_field( field_name, val );
+  if ( !m_current_item->set_string_list_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_string_list()
 
 /*----------------------------------------------------------------------------*/
@@ -516,12 +592,14 @@ void bear::engine::level_loader::load_item_field_sprite_list()
 
   m_file >> m_next_code;
 
-  m_current_item->set_sprite_list_field( field_name, val );
+  if ( !m_current_item->set_sprite_list_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_sprite_list()
 
 /*----------------------------------------------------------------------------*/
 /**
- * \brief Set a field of type list of snimation to an item.
+ * \brief Set a field of type list of animation to an item.
  */
 void bear::engine::level_loader::load_item_field_animation_list()
 {
@@ -533,14 +611,13 @@ void bear::engine::level_loader::load_item_field_animation_list()
   std::vector<visual::animation> val(n);
 
   for (unsigned int i=0; i!=n; ++i)
-    {
-      m_file >> path;
-      val[i] = m_level->get_globals().get_animation(path);
-    }
+    val[i] = sprite_loader::load_any_animation(m_file, m_level->get_globals());
 
   m_file >> m_next_code;
 
-  m_current_item->set_animation_list_field( field_name, val );
+  if ( !m_current_item->set_animation_list_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_animation_list()
 
 /*----------------------------------------------------------------------------*/
@@ -567,9 +644,34 @@ void bear::engine::level_loader::load_item_field_item_list()
 
   m_file >> m_next_code;
 
-  m_current_item->set_item_list_field( field_name, val );
+  if ( !m_current_item->set_item_list_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
 } // level_loader::load_item_field_item_list()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set a field of type sample to an item.
+ */
+void bear::engine::level_loader::load_item_field_sample_list()
+{
+  std::string field_name;
+  unsigned int n;
+
+  m_file >> field_name >> n;
+
+  std::vector<audio::sample*> val(n);
+
+  for (unsigned int i=0; i!=n; ++i)
+    val[i] = load_sample_data();
+
+  m_file >> m_next_code;
+
+  if ( !m_current_item->set_sample_list_field( field_name, val ) )
+    claw::logger << claw::log_warning << "field '" << field_name
+                 << "' has not been set." << std::endl;
+} // level_loader::load_item_field_sample_list()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Create an item using the create_* method exported by the item class.
@@ -582,24 +684,19 @@ bear::engine::base_item* bear::engine::level_loader::create_item_from_string
                << std::endl;
 
   base_item* result(NULL);
-  const std::string export_name( export_function_name(name) );
-  libraries_pool& symbols( game::get_instance().get_symbols() );
 
-  if ( symbols.have_symbol( export_name ) )
+  if ( item_factory::get_instance().is_known_type(name) )
     {
-      base_export_function_type func;
-
-      func = symbols.get_symbol<base_export_function_type>( export_name );
-      result = func();
+      result = item_factory::get_instance().create(name);
       result->set_level(*m_level);
       claw::logger << claw::log_verbose << "Item '" << name << "' id="
                    << result->get_id() << std::endl;
     }
   else
     {
-      claw::logger << claw::log_error << "Can't find symbol '" << export_name
+      claw::logger << claw::log_error << "Can't find item class '" << name
                    << "'." << std::endl;
-      throw CLAW_EXCEPTION("Can't find symbol '" + export_name + "'.");
+      throw claw::exception("Can't find item class '" + name + "'.");
     }
 
   return result;
@@ -619,19 +716,19 @@ bear::engine::layer* bear::engine::level_loader::create_layer_from_string
                << std::endl;
 
   layer* result(NULL);
-  const std::string export_name( layer_export_function_name(name) );
-  libraries_pool& symbols( game::get_instance().get_symbols() );
 
-  if ( symbols.have_symbol( export_name ) )
+  if ( layer_factory::get_instance().is_known_type(name) )
     {
-      layer_export_function_type func;
-
-      func = symbols.get_symbol<layer_export_function_type>( export_name );
-      result = func(s);
+      layer_creator* creator = layer_factory::get_instance().create(name);
+      result = creator->create(s);
+      delete creator;
     }
   else
-    claw::logger << claw::log_error << "Can't find symbol '" << export_name
-                 << "'." << std::endl;
+    {
+      claw::logger << claw::log_error << "Can't find layer class '" << name
+                   << "'." << std::endl;
+      throw claw::exception("Can't find layer class '" + name + "'.");
+    }
 
   return result;
 } // level_loader::create_layer_from_string()
@@ -674,3 +771,50 @@ void bear::engine::level_loader::escape( std::string& str ) const
 
   str = result;
 } // level_loader::escape()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Load a sample.
+ */
+bear::audio::sample* bear::engine::level_loader::load_sample_data() const
+{
+  std::string sample_path;
+  int loops;
+  double volume;
+
+  m_file >> sample_path >> loops >> volume;
+  audio::sound_effect e(loops, volume);
+  audio::sample* s = m_level->get_globals().new_sample(sample_path);
+
+  s->set_effect(e);
+
+  return s;
+} // level_loader::load_sample_data()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Load a list of values.
+ * \param v (out) The values read from the file.
+ * \return The name of the corresponding field.
+ */
+template<typename T>
+std::string bear::engine::level_loader::load_list( std::vector<T>& v )
+{
+  std::string field_name;
+  unsigned int n;
+  T val;
+
+  m_file >> field_name >> n;
+
+  v.resize(n);
+
+  for (unsigned int i=0; i!=n; ++i)
+    {
+      m_file >> val;
+      v[i] = val;
+    }
+
+  m_file >> m_next_code;
+
+  return field_name;
+} // level_loader::load_list()
