@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -28,14 +28,14 @@
  */
 #include "engine/game_action/game_action_pop_level.hpp"
 
-#include "engine/game.hpp"
+#include "engine/game_local_client.hpp"
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Apply the action to a game.
  * \param the_game The game to apply to.
  */
-bool bear::engine::game_action_pop_level::apply( game& the_game )
+bool bear::engine::game_action_pop_level::apply( game_local_client& the_game )
 {
   the_game.do_pop_level();
   return true;
