@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -28,7 +28,10 @@
  */
 #include "gui/visual_component.hpp"
 
+#include "input/key_info.hpp"
+
 #include "visual/scene_line.hpp"
+#include "visual/scene_rectangle.hpp"
 
 #include <algorithm>
 #include <claw/assert.hpp>
@@ -44,7 +47,9 @@
  */
 bear::gui::visual_component::visual_component( visual_component* owner )
   : m_box(0, 0, 0, 0), m_owner(owner), m_focused_component(-1),
-    m_visible(true), m_input_priority(false), m_border_color(0, 0, 0, 0)
+    m_visible(true), m_input_priority(false), m_enabled(true),
+    m_top_left_border_color(0, 0, 0, 0),
+    m_bottom_right_border_color(0, 0, 0, 0), m_background_color(0, 0, 0, 0)
 {
   if (m_owner)
     m_owner->add_component(this);
@@ -68,37 +73,56 @@ bear::gui::visual_component::~visual_component()
 void bear::gui::visual_component::render
 ( std::list<visual::scene_element>& e ) const
 {
-  if (m_visible)
+  if (!m_visible)
+    return;
+
+  scene_element_list sub_e;
+
+  // display the current component
+  display( e );
+
+  // display the sub components
+  for ( component_list::const_iterator it=m_components.begin();
+        it!=m_components.end();
+        ++it )
+    (*it)->render( sub_e );
+
+  // the scene elements are placed in the current component, so we adjust
+  // their position according to our position.
+  for ( scene_element_list::iterator it=sub_e.begin(); it!=sub_e.end(); ++it )
+    it->set_position( it->get_position() + m_box.bottom_left() );
+
+  e.insert( e.end(), sub_e.begin(), sub_e.end() );
+
+  // and we draw the borders and the background of the current component
+  visual::rectangle_type box
+    (visual::position_type(0, 0), visual::position_type( width(), height() ));
+
+  if ( get_border_size() != 0 )
     {
-      scene_element_list sub_e;
+      std::vector<visual::position_type> line(3);
+      line[0] = box.bottom_left();
+      line[1] = box.bottom_right();
+      line[2] = box.top_right();
 
-      // display the current component
-      display( e );
-      
-      // display the sub components
-      for ( component_list::const_iterator it=m_components.begin();
-            it!=m_components.end();
-            ++it )
-        (*it)->render( sub_e );
+      if ( m_bottom_right_border_color.components.alpha != 0 )
+        e.push_front
+          ( visual::scene_line
+            (left(), bottom(), m_bottom_right_border_color, line,
+             get_border_size()) );
 
-      // the scene elements are placed in the current component, so we adjust
-      // thein position according to our position.
-      for ( scene_element_list::iterator it=sub_e.begin(); it!=sub_e.end();
-            ++it )
-        it->set_position( it->get_position() + m_box.position );
+      line[1] = box.top_left();
 
-      e.insert( e.end(), sub_e.begin(), sub_e.end() );
-
-      // and we draw the border of the current component
-      std::vector<visual::position_type> p(5, visual::position_type(0, 0));
-
-      p[1].x += width();
-      p[2].x += width();
-      p[2].y += height();
-      p[3].y += height();
-
-      e.push_back( visual::scene_line(left(), top(), m_border_color, p, 1) );
+      if ( m_top_left_border_color.components.alpha != 0 )
+        e.push_front
+          ( visual::scene_line
+            (left(), bottom(), m_top_left_border_color, line,
+             get_border_size()) );
     }
+
+  if ( m_background_color.components.alpha != 0 )
+    e.push_front
+      ( visual::scene_rectangle(left(), bottom(), m_background_color, box) );
 } // visual_component::render()
 
 /*----------------------------------------------------------------------------*/
@@ -108,6 +132,9 @@ void bear::gui::visual_component::render
  */
 bool bear::gui::visual_component::key_pressed( const input::key_info& key )
 {
+  if ( !is_enabled() )
+    return false;
+
   bool result;
 
   if (m_input_priority)
@@ -115,10 +142,10 @@ bool bear::gui::visual_component::key_pressed( const input::key_info& key )
       result = on_key_press(key);
 
       if ( !result )
-        if ( m_focused_component >= 0)
+        if ( m_focused_component >= 0 )
           result = m_components[m_focused_component]->key_pressed(key);
     }
-  else if ( m_focused_component >= 0)
+  else if ( m_focused_component >= 0 )
     {
       result = m_components[m_focused_component]->key_pressed(key);
 
@@ -138,6 +165,9 @@ bool bear::gui::visual_component::key_pressed( const input::key_info& key )
  */
 bool bear::gui::visual_component::char_pressed( const input::key_info& key )
 {
+  if ( !is_enabled() )
+    return false;
+
   bool result;
 
   if (m_input_priority)
@@ -145,10 +175,10 @@ bool bear::gui::visual_component::char_pressed( const input::key_info& key )
       result = on_char_pressed(key);
 
       if ( !result )
-        if ( m_focused_component >= 0)
+        if ( m_focused_component >= 0 )
           result = m_components[m_focused_component]->char_pressed(key);
     }
-  else if ( m_focused_component >= 0)
+  else if ( m_focused_component >= 0 )
     {
       result = m_components[m_focused_component]->char_pressed(key);
 
@@ -170,6 +200,9 @@ bool bear::gui::visual_component::char_pressed( const input::key_info& key )
 bool bear::gui::visual_component::button_pressed
 ( input::joystick::joy_code button, unsigned int joy_index )
 {
+  if ( !is_enabled() )
+    return false;
+
   bool result;
 
   if (m_input_priority)
@@ -205,6 +238,9 @@ bool bear::gui::visual_component::mouse_pressed
 ( input::mouse::mouse_code key,
   const claw::math::coordinate_2d<unsigned int>& pos )
 {
+  if ( !is_enabled() )
+    return false;
+
   bool result;
 
   if (m_input_priority)
@@ -225,6 +261,72 @@ bool bear::gui::visual_component::mouse_pressed
   return result;
 } // visual_component::mouse_pressed()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Inform the component that a mouse button had been released.
+ * \param key The value of the released button.
+ * \param pos The current position of the cursor.
+ */
+bool bear::gui::visual_component::mouse_released
+( input::mouse::mouse_code key,
+  const claw::math::coordinate_2d<unsigned int>& pos )
+{
+  if ( !is_enabled() )
+    return false;
+
+  bool result;
+
+  if (m_input_priority)
+    {
+      result = on_mouse_released(key, pos);
+
+      if ( !result )
+        result = broadcast_mouse_released(key, pos);
+    }
+  else
+    {
+      result = broadcast_mouse_released(key, pos);
+
+      if ( !result )
+        result = on_mouse_released(key, pos);
+    }
+
+  return result;
+} // visual_component::mouse_released()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Inform the component that a mouse button had been maintained.
+ * \param key The value of the maintained button.
+ * \param pos The current position of the cursor.
+ */
+bool bear::gui::visual_component::mouse_maintained
+( input::mouse::mouse_code key,
+  const claw::math::coordinate_2d<unsigned int>& pos )
+{
+  if ( !is_enabled() )
+    return false;
+
+  bool result;
+
+  if (m_input_priority)
+    {
+      result = on_mouse_maintained(key, pos);
+
+      if ( !result )
+        result = broadcast_mouse_maintained(key, pos);
+    }
+  else
+    {
+      result = broadcast_mouse_maintained(key, pos);
+
+      if ( !result )
+        result = on_mouse_maintained(key, pos);
+    }
+
+  return result;
+} // visual_component::mouse_maintained()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Inform the component that the mouse has been moved.
@@ -233,6 +335,9 @@ bool bear::gui::visual_component::mouse_pressed
 bool bear::gui::visual_component::mouse_move
 ( const claw::math::coordinate_2d<unsigned int>& pos )
 {
+  if ( !is_enabled() )
+    return false;
+
   bool result;
 
   if (m_input_priority)
@@ -253,14 +358,45 @@ bool bear::gui::visual_component::mouse_move
   return result;
 } // visual_component::mouse_move()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Adjust the size of the component to fit its sub components.
+ * \param margin The margin around the children.
+ */
+void bear::gui::visual_component::fit( visual::size_type margin )
+{
+  size_type min_x( std::numeric_limits<size_type>::max() );
+  size_type min_y( std::numeric_limits<size_type>::max() );
+  size_type max_x( std::numeric_limits<size_type>::min() );
+  size_type max_y( std::numeric_limits<size_type>::min() );
+
+  component_list::const_iterator it;
+  for ( it=m_components.begin(); it!=m_components.end(); ++it )
+    {
+      min_x = std::min( min_x, (*it)->left() );
+      min_y = std::min( min_y, (*it)->bottom() );
+      max_x = std::max( max_x, (*it)->right() );
+      max_y = std::max( max_y, (*it)->top() );
+    }
+
+  const size_type delta_x = min_x - margin;
+  const size_type delta_y = min_y - margin;
+
+  for ( it=m_components.begin(); it!=m_components.end(); ++it )
+    (*it)->set_bottom_left
+      ( (*it)->left() - delta_x, (*it)->bottom() - delta_y );
+
+  set_size( max_x - delta_x + margin, max_y - delta_y + margin );
+} // visual_component::fit()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Set the size of the component to its maximum size.
  */
 void bear::gui::visual_component::set_size_maximum()
 {
-  set_size( std::numeric_limits<unsigned int>::max(),
-            std::numeric_limits<unsigned int>::max() );
+  set_size( std::numeric_limits<size_type>::max(),
+            std::numeric_limits<size_type>::max() );
 } // visual_component::set_size()
 
 /*----------------------------------------------------------------------------*/
@@ -268,8 +404,7 @@ void bear::gui::visual_component::set_size_maximum()
  * \brief Set the size of the component.
  * \param size The new size.
  */
-void bear::gui::visual_component::set_size
-( const claw::math::coordinate_2d<unsigned int>& size )
+void bear::gui::visual_component::set_size( const size_box_type& size )
 {
   set_size( size.x, size.y );
 } // visual_component::set_size()
@@ -280,17 +415,16 @@ void bear::gui::visual_component::set_size
  * \param w The new width.
  * \param h The new height.
  */
-void bear::gui::visual_component::set_size( unsigned int w, unsigned int h )
+void bear::gui::visual_component::set_size( size_type w, size_type h )
 {
-  const unsigned int old_w = m_box.width;
-  const unsigned int old_h = m_box.height;
+  const size_type old_w = m_box.width();
+  const size_type old_h = m_box.height();
 
-  m_box.width = w;
-  m_box.height = h;
+  m_box.set( left(), bottom(), m_box.left() + w, m_box.bottom() + h );
 
   stay_in_owner();
 
-  if ( (old_w != m_box.width) || (old_h != m_box.height) )
+  if ( (old_w != m_box.width()) || (old_h != m_box.height()) )
     on_resized();
 } // visual_component::set_size()
 
@@ -299,7 +433,7 @@ void bear::gui::visual_component::set_size( unsigned int w, unsigned int h )
  * \brief Set the width of the component.
  * \param w The new width.
  */
-void bear::gui::visual_component::set_width( unsigned int w )
+void bear::gui::visual_component::set_width( size_type w )
 {
   set_size( w, height() );
 } // visual_component::set_width()
@@ -309,7 +443,7 @@ void bear::gui::visual_component::set_width( unsigned int w )
  * \brief Set the height of the component.
  * \param h The new height.
  */
-void bear::gui::visual_component::set_height( unsigned int h )
+void bear::gui::visual_component::set_height( size_type h )
 {
   set_size( width(), h );
 } // visual_component::set_height()
@@ -320,7 +454,7 @@ void bear::gui::visual_component::set_height( unsigned int h )
  * \param pos The new position.
  */
 void bear::gui::visual_component::set_position
-( const claw::math::coordinate_2d<unsigned int>& pos )
+( const position_type& pos )
 {
   set_position( pos.x, pos.y );
 } // visual_component::set_position()
@@ -331,18 +465,11 @@ void bear::gui::visual_component::set_position
  * \param x The new x-coordinate.
  * \param y The new y-coordinate.
  */
-void bear::gui::visual_component::set_position( unsigned int x, unsigned int y )
+void  bear::gui::visual_component::set_position
+( coordinate_type x, coordinate_type y )
 {
-  const unsigned int w = m_box.width;
-  const unsigned int h = m_box.height;
-
-  m_box.position.x = x;
-  m_box.position.y = y;
-  stay_in_owner();
-
-  if ( (w != m_box.width) || (h != m_box.height) )
-    on_resized();
-} // visual_component::set_size()
+  set_bottom_left(x, y);
+} // visual_component::set_position()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -376,29 +503,125 @@ void bear::gui::visual_component::set_input_priority( bool this_first )
   m_input_priority = this_first;
 } // visual_component::set_input_priority()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the left position of the component in its owner.
+ * \param x The new position.
+ */
+void bear::gui::visual_component::set_left( coordinate_type x )
+{
+  set_bottom_left( x, bottom() );
+} // visual_component::set_left()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the right position of the component in its owner.
+ * \param x The new position.
+ */
+void bear::gui::visual_component::set_right( coordinate_type x )
+{
+  set_bottom_left( x - width(), bottom() );
+} // visual_component::set_right()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the top position of the component in its owner.
+ * \param y The new position.
+ */
+void bear::gui::visual_component::set_top( coordinate_type y )
+{
+  set_bottom_left( left(), y - height() );
+} // visual_component::set_top()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the bottom position of the component in its owner.
+ * \param x The new position.
+ */
+void bear::gui::visual_component::set_bottom( coordinate_type y )
+{
+  set_bottom_left( left(), y );
+} // visual_component::set_bottom()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the top-left position of the component in its owner.
+ * \param x The new x-position.
+ * \param y The new y-position.
+ */
+void bear::gui::visual_component::set_top_left
+( coordinate_type x, coordinate_type y )
+{
+  set_bottom_left( x, y - height() );
+} // visual_component::set_top_left()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the bottom-left position of the component in its owner.
+ * \param x The new x-position.
+ * \param y The new y-position.
+ */
+void bear::gui::visual_component::set_bottom_left
+( coordinate_type x, coordinate_type y )
+{
+  const coordinate_type w = m_box.width();
+  const coordinate_type h = m_box.height();
+
+  m_box.left(x);
+  m_box.bottom(y);
+  stay_in_owner();
+
+  if ( (w != m_box.width()) || (h != m_box.height()) )
+    on_resized();
+} // visual_component::set_bottom_left()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the top-right position of the component in its owner.
+ * \param x The new x-position.
+ * \param y The new y-position.
+ */
+void bear::gui::visual_component::set_top_right
+( coordinate_type x, coordinate_type y )
+{
+  set_bottom_left( x - width(), y - height() );
+} // visual_component::set_top_right()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the bottom-right position of the component in its owner.
+ * \param x The new x-position.
+ * \param y The new y-position.
+ */
+void bear::gui::visual_component::set_bottom_right
+( coordinate_type x, coordinate_type y )
+{
+  set_bottom_left( x - width(), y );
+} // visual_component::set_bottom_right()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the width of the component.
  */
-unsigned int bear::gui::visual_component::width() const
+bear::gui::size_type bear::gui::visual_component::width() const
 {
-  return m_box.width;
+  return m_box.width();
 } // visual_component::width()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the height of the component.
  */
-unsigned int bear::gui::visual_component::height() const
+bear::gui::size_type bear::gui::visual_component::height() const
 {
-  return m_box.height;
+  return m_box.height();
 } // visual_component::height()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the x-coordinate of the component's left edge.
  */
-unsigned int bear::gui::visual_component::left() const
+bear::gui::coordinate_type bear::gui::visual_component::left() const
 {
   return m_box.left();
 } // visual_component::left()
@@ -407,7 +630,7 @@ unsigned int bear::gui::visual_component::left() const
 /**
  * \brief Get the x-coordinate of the component's right edge.
  */
-unsigned int bear::gui::visual_component::right() const
+bear::gui::coordinate_type bear::gui::visual_component::right() const
 {
   return m_box.right();
 } // visual_component::right()
@@ -416,7 +639,7 @@ unsigned int bear::gui::visual_component::right() const
 /**
  * \brief Get the y-coordinate of the component's bottom edge.
  */
-unsigned int bear::gui::visual_component::bottom() const
+bear::gui::coordinate_type bear::gui::visual_component::bottom() const
 {
   return m_box.bottom();
 } // visual_component::bottom()
@@ -425,7 +648,7 @@ unsigned int bear::gui::visual_component::bottom() const
 /**
  * \brief Get the y-coordinate of the component's top edge.
  */
-unsigned int bear::gui::visual_component::top() const
+bear::gui::coordinate_type bear::gui::visual_component::top() const
 {
   return m_box.top();
 } // visual_component::top()
@@ -434,67 +657,61 @@ unsigned int bear::gui::visual_component::top() const
 /**
  * \brief Get the coordinates of the component's top left corner.
  */
-claw::math::coordinate_2d<unsigned int>
-bear::gui::visual_component::top_left() const
+bear::gui::position_type bear::gui::visual_component::top_left() const
 {
-  return claw::math::coordinate_2d<unsigned int>( left(), top() );
+  return m_box.top_left();
 } // visual_component::top_left()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the coordinates of the component's top right corner.
  */
-claw::math::coordinate_2d<unsigned int>
-bear::gui::visual_component::top_right() const
+bear::gui::position_type bear::gui::visual_component::top_right() const
 {
-  return claw::math::coordinate_2d<unsigned int>( right(), top() );
+  return m_box.top_right();
 } // visual_component::top_right()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the coordinates of the component's bottom left corner.
  */
-claw::math::coordinate_2d<unsigned int>
-bear::gui::visual_component::bottom_left() const
+bear::gui::position_type bear::gui::visual_component::bottom_left() const
 {
-  return claw::math::coordinate_2d<unsigned int>( left(), bottom() );
+  return m_box.bottom_left();
 } // visual_component::bottom_left()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the coordinates of the component's bottom right corner.
  */
-claw::math::coordinate_2d<unsigned int>
-bear::gui::visual_component::bottom_right() const
+bear::gui::position_type bear::gui::visual_component::bottom_right() const
 {
-  return claw::math::coordinate_2d<unsigned int>( right(), bottom() );
+  return m_box.bottom_right();
 } // visual_component::bottom_right()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the size of the component.
  */
-claw::math::coordinate_2d<unsigned int>
-bear::gui::visual_component::get_size() const
+bear::gui::size_box_type bear::gui::visual_component::get_size() const
 {
-  return claw::math::coordinate_2d<unsigned int>( m_box.width, m_box.height );
+  return m_box.size();
 } // visual_component::get_size()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the position of the component in its owner.
  */
-const claw::math::coordinate_2d<unsigned int>&
-bear::gui::visual_component::get_position() const
+bear::gui::position_type bear::gui::visual_component::get_position() const
 {
-  return m_box.position;
+  return m_box.bottom_left();
 } // visual_component::get_position()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the rectangle bounding this component.
  */
-const claw::math::rectangle<unsigned int>&
+const bear::gui::rectangle_type&
 bear::gui::visual_component::get_rectangle() const
 {
   return m_box;
@@ -509,17 +726,152 @@ bool bear::gui::visual_component::get_visible() const
   return m_visible;
 } // visual_component::get_visible()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the size of the borders.
+ */
+bear::gui::size_type bear::gui::visual_component::get_border_size() const
+{
+  return 1;
+} // visual_component::get_border_size()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Set the color of the border.
  * \param clr The new color.
  */
-void bear::gui::visual_component::set_border_color
-( const claw::graphic::rgba_pixel& clr )
+void bear::gui::visual_component::set_border_color( const color_type& clr )
 {
-  m_border_color = clr;
+  set_top_left_border_color( clr );
+  set_bottom_right_border_color( clr );
 } // visual_component::set_border_color()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the color of the top and left borders.
+ * \param clr The new color.
+ */
+void
+bear::gui::visual_component::set_top_left_border_color( const color_type& clr )
+{
+  m_top_left_border_color = clr;
+} // visual_component::set_top_left_border_color()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the color of the top and left borders.
+ */
+const bear:: gui::color_type&
+bear::gui::visual_component::get_top_left_border_color() const
+{
+  return m_top_left_border_color;
+} // visual_component::get_top_left_border_color()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the color of the bottom and right borders.
+ * \param clr The new color.
+ */
+void bear::gui::visual_component::set_bottom_right_border_color
+( const color_type& clr )
+{
+  m_bottom_right_border_color = clr;
+} // visual_component::set_bottom_right_border_color()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the color of the bottom and right borders.
+ */
+const bear::gui::color_type&
+bear::gui::visual_component::get_bottom_right_border_color() const
+{
+  return m_bottom_right_border_color;
+} // visual_component::get_bottom_right_border_color()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the color of the background.
+ * \param clr The new color.
+ */
+void bear::gui::visual_component::set_background_color( const color_type& clr )
+{
+  m_background_color = clr;
+} // visual_component::set_background_color()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the color of the background.
+ * \param clr The new color.
+ */
+const bear::gui::color_type&
+bear::gui::visual_component::get_background_color() const
+{
+  return m_background_color;
+} // visual_component::get_background_color()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Disable this component.
+ */
+void bear::gui::visual_component::disable()
+{
+  m_enabled = false;
+} // visual_component::disable()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Enable this component.
+ */
+void bear::gui::visual_component::enable()
+{
+  m_enabled = true;
+} // visual_component::enable()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Tell if this compontent is enabled.
+ */
+bool bear::gui::visual_component::is_enabled() const
+{
+  return m_enabled;
+} // visual_component::is_enabled()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the focus on this component.
+ */
+void bear::gui::visual_component::set_focus()
+{
+  // all parent controls up to the top component
+  std::list<visual_component*> s;
+  visual_component* c;
+
+  for ( c = this; c!=NULL; c=c->m_owner )
+    s.push_front(c);
+
+  std::list<visual_component*>::const_iterator it_parent(s.begin());
+  std::list<visual_component*>::const_iterator it_child(it_parent);
+  ++it_child;
+
+  for ( ; it_child!=s.end(); ++it_parent, ++it_child )
+    (*it_parent)->set_focus(*it_child);
+
+  for ( it_parent=s.begin(); it_parent!=s.end(); ++it_parent )
+    (*it_parent)->on_focused();
+} // visual_component::set_focus()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the child component having the focus.
+ */
+bear::gui::visual_component* bear::gui::visual_component::get_focus() const
+{
+  if ( m_focused_component < 0 )
+    return NULL;
+  else
+    return m_components[m_focused_component];
+} // visual_component::get_focus()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Add a component in this component.
@@ -636,6 +988,34 @@ bool bear::gui::visual_component::on_mouse_press
   return false;
 } // visual_component::on_mouse_press()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Tell the focused component that a mouse button has been released.
+ * \param key The pressed key.
+ * \param pos The current position of the cursor.
+ * \return true if the key has been processed.
+ */
+bool bear::gui::visual_component::on_mouse_released
+( input::mouse::mouse_code key,
+  const claw::math::coordinate_2d<unsigned int>& pos )
+{
+  return false;
+} // visual_component::on_mouse_released()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Tell the focused component that a mouse button has been maintained.
+ * \param key The pressed key.
+ * \param pos The current position of the cursor.
+ * \return true if the key has been processed.
+ */
+bool bear::gui::visual_component::on_mouse_maintained
+( input::mouse::mouse_code key,
+  const claw::math::coordinate_2d<unsigned int>& pos )
+{
+  return false;
+} // visual_component::on_mouse_maintained()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell the component that the mouse has been moved.
@@ -648,6 +1028,15 @@ bool bear::gui::visual_component::on_mouse_move
   return false;
 } // visual_component::on_mouse_move()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Tell the component he received the focus.
+ */
+void bear::gui::visual_component::on_focused()
+{
+
+} // visual_component::on_focused()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Inform the sub components that the mouse has been moved.
@@ -668,7 +1057,7 @@ bool bear::gui::visual_component::broadcast_mouse_move
 
 /*----------------------------------------------------------------------------*/
 /**
- * \brief Inform the sub components that the mouse has been moved.
+ * \brief Inform the sub components that a mouse button has been pressed.
  * \param button The pressed key.
  * \param pos The current position of the cursor.
  * \return true if the key has been processed.
@@ -687,6 +1076,48 @@ bool bear::gui::visual_component::broadcast_mouse_press
   return result;
 } // visual_component::broadcast_mouse_press()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Inform the sub components that a mouse button has been released.
+ * \param button The pressed key.
+ * \param pos The current position of the cursor.
+ * \return true if the key has been processed.
+ */
+bool bear::gui::visual_component::broadcast_mouse_released
+( input::mouse::mouse_code button,
+  const claw::math::coordinate_2d<unsigned int>& pos )
+{
+  bool result(false);
+  component_list::iterator it;
+
+  for (it=m_components.begin(); !result && (it!=m_components.end()); ++it)
+    if ( (*it)->m_box.includes(pos) )
+      result = (*it)->mouse_released( button, pos - (*it)->get_position() );
+
+  return result;
+} // visual_component::broadcast_mouse_released()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Inform the sub components that a mouse button has been maintained.
+ * \param button The pressed key.
+ * \param pos The current position of the cursor.
+ * \return true if the key has been processed.
+ */
+bool bear::gui::visual_component::broadcast_mouse_maintained
+( input::mouse::mouse_code button,
+  const claw::math::coordinate_2d<unsigned int>& pos )
+{
+  bool result(false);
+  component_list::iterator it;
+
+  for (it=m_components.begin(); !result && (it!=m_components.end()); ++it)
+    if ( (*it)->m_box.includes(pos) )
+      result = (*it)->mouse_maintained( button, pos - (*it)->get_position() );
+
+  return result;
+} // visual_component::broadcast_mouse_maintained()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Adjust the width and the height of the component to stay in the owner.
@@ -695,16 +1126,43 @@ void bear::gui::visual_component::stay_in_owner()
 {
   if (m_owner)
     {
-      if ( m_box.position.x >= m_owner->width() )
-        m_box.position.x = m_owner->width() - 1;
+      coordinate_type n_left(m_box.left());
+      coordinate_type n_right(m_box.right());
+      coordinate_type n_bottom(m_box.bottom());
+      coordinate_type n_top(m_box.top());
 
-      if ( m_box.position.y >= m_owner->height() )
-        m_box.position.y = m_owner->height() - 1;
+      if ( n_left >= m_owner->width() )
+        n_left = m_owner->width();
 
-      if ( m_box.right() >= m_owner->width() )
-        m_box.width = m_owner->width() - m_box.position.x;
+      if ( n_bottom >= m_owner->height() )
+        n_bottom = m_owner->height();
 
-      if ( m_box.bottom() >= m_owner->height() )
-        m_box.height = m_owner->height() - m_box.position.y;
+      if ( n_right >= m_owner->width() )
+        n_right = m_owner->width();
+
+      if ( n_top >= m_owner->height() )
+        n_top = m_owner->height();
+
+      m_box.set( n_left, n_bottom, n_right, n_top );
     }
 } // visual_component::stay_in_owner()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the focus on a given component.
+ * \param c The component on which the focus is set.
+ */
+void bear::gui::visual_component::set_focus( visual_component* c )
+{
+  int i(0);
+  bool stop(false);
+  m_focused_component = -1;
+
+  for ( component_list::const_iterator it=m_components.begin();
+        !stop && (it != m_components.end()); ++it, ++i )
+    if ( *it == c )
+      {
+        m_focused_component = i;
+        stop = true;
+      }
+} // visual_component::set_focus()
