@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -27,47 +27,226 @@
  * \author Julien Jorge
  */
 #include "input/keyboard.hpp"
+
+#include "bear_gettext.hpp"
+
 #include <SDL/SDL.h>
 #include <algorithm>
 #include <claw/assert.hpp>
-
-/*----------------------------------------------------------------------------*/
-std::vector<std::string> bear::input::keyboard::s_key_strings;
-
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Constructor.
- */
-bear::input::keyboard::keyboard()
-{
-  if ( s_key_strings.empty() )
-    default_key_strings();
-} // keyboard::keyboard()
+#include <claw/logger.hpp>
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Convert a key_code to a human-readable string.
  * \param k The key to convert.
  */
-const std::string& bear::input::keyboard::get_name_of( key_code k )
+std::string bear::input::keyboard::get_name_of( key_code k )
 {
-  CLAW_PRECOND( k < s_key_strings.size() );
+  switch(k)
+    {
+    case kc_backspace: return "backspace"; break;
+    case kc_tab: return "tab"; break;
+    case kc_new_line: return "new line"; break;
+    case kc_pause: return "pause"; break;
 
-  return s_key_strings[k];
+    case kc_escape: return "escape"; break;
+    case kc_space: return "space"; break;
+    case kc_exclamation_mark: return "!"; break;
+    case kc_double_quotes: return "\""; break;
+    case kc_hash: return "#"; break;
+    case kc_dollar: return "$"; break;
+    case kc_percent: return "%"; break;
+    case kc_ampersand: return "&"; break;
+    case kc_quote: return "'"; break;
+    case kc_left_parenthesis: return "("; break;
+    case kc_right_parenthesis: return ")"; break;
+    case kc_asterisk: return "*"; break;
+    case kc_plus: return "+"; break;
+    case kc_comma: return ","; break;
+    case kc_minus: return "-"; break;
+    case kc_period: return "."; break;
+    case kc_slash: return "/"; break;
+
+    case kc_0: return "0"; break;
+    case kc_1: return "1"; break;
+    case kc_2: return "2"; break;
+    case kc_3: return "3"; break;
+    case kc_4: return "4"; break;
+    case kc_5: return "5"; break;
+    case kc_6: return "6"; break;
+    case kc_7: return "7"; break;
+    case kc_8: return "8"; break;
+    case kc_9: return "9"; break;
+
+    case kc_colon: return ":"; break;
+    case kc_semicolon: return "; break;"; break;
+    case kc_less: return "<"; break;
+    case kc_equal: return "="; break;
+    case kc_greater: return ">"; break;
+    case kc_question: return "?"; break;
+    case kc_at: return "@"; break;
+
+    case kc_A: return "A"; break;
+    case kc_B: return "B"; break;
+    case kc_C: return "C"; break;
+    case kc_D: return "D"; break;
+    case kc_E: return "E"; break;
+    case kc_F: return "F"; break;
+    case kc_G: return "G"; break;
+    case kc_H: return "H"; break;
+    case kc_I: return "I"; break;
+    case kc_J: return "J"; break;
+    case kc_K: return "K"; break;
+    case kc_L: return "L"; break;
+    case kc_M: return "M"; break;
+    case kc_N: return "N"; break;
+    case kc_O: return "O"; break;
+    case kc_P: return "P"; break;
+    case kc_Q: return "Q"; break;
+    case kc_R: return "R"; break;
+    case kc_S: return "S"; break;
+    case kc_T: return "T"; break;
+    case kc_U: return "U"; break;
+    case kc_V: return "V"; break;
+    case kc_W: return "W"; break;
+    case kc_X: return "X"; break;
+    case kc_Y: return "Y"; break;
+    case kc_Z: return "Z"; break;
+
+    case kc_left_bracket: return "["; break;
+    case kc_backslash: return "\\"; break;
+    case kc_right_bracket: return "]"; break;
+    case kc_caret: return "^"; break;
+    case kc_underscore: return "_"; break;
+    case kc_backquote: return "`"; break;
+
+    case kc_a: return "a"; break;
+    case kc_b: return "b"; break;
+    case kc_c: return "c"; break;
+    case kc_d: return "d"; break;
+    case kc_e: return "e"; break;
+    case kc_f: return "f"; break;
+    case kc_g: return "g"; break;
+    case kc_h: return "h"; break;
+    case kc_i: return "i"; break;
+    case kc_j: return "j"; break;
+    case kc_k: return "k"; break;
+    case kc_l: return "l"; break;
+    case kc_m: return "m"; break;
+    case kc_n: return "n"; break;
+    case kc_o: return "o"; break;
+    case kc_p: return "p"; break;
+    case kc_q: return "q"; break;
+    case kc_r: return "r"; break;
+    case kc_s: return "s"; break;
+    case kc_t: return "t"; break;
+    case kc_u: return "u"; break;
+    case kc_v: return "v"; break;
+    case kc_w: return "w"; break;
+    case kc_x: return "x"; break;
+    case kc_y: return "y"; break;
+    case kc_z: return "z"; break;
+
+    case kc_left_brace: return "{"; break;
+    case kc_vertical_line: return "|"; break;
+    case kc_right_brace: return "}"; break;
+    case kc_tilde: return "~"; break;
+    case kc_delete: return "delete"; break;
+
+    case kc_keypad_0: return "keypad 0"; break;
+    case kc_keypad_1: return "keypad 1"; break;
+    case kc_keypad_2: return "keypad 2"; break;
+    case kc_keypad_3: return "keypad 3"; break;
+    case kc_keypad_4: return "keypad 4"; break;
+    case kc_keypad_5: return "keypad 5"; break;
+    case kc_keypad_6: return "keypad 6"; break;
+    case kc_keypad_7: return "keypad 7"; break;
+    case kc_keypad_8: return "keypad 8"; break;
+    case kc_keypad_9: return "keypad 9"; break;
+
+    case kc_keypad_period: return "keypad ."; break;
+    case kc_keypad_divide: return "keypad /"; break;
+    case kc_keypad_multiply: return "keypad *"; break;
+    case kc_keypad_minus: return "keypad -"; break;
+    case kc_keypad_plus: return "keypad +"; break;
+    case kc_keypad_enter: return "keypad enter"; break;
+    case kc_keypad_equals: return "keypad ="; break;
+
+    case kc_up: return "up"; break;
+    case kc_down: return "down"; break;
+    case kc_right: return "right"; break;
+    case kc_left: return "left"; break;
+
+    case kc_insert: return "insert"; break;
+    case kc_home: return "home"; break;
+    case kc_end: return "end"; break;
+    case kc_page_up: return "page up"; break;
+    case kc_page_down: return "page down"; break;
+
+    case kc_F1: return "F1"; break;
+    case kc_F2: return "F2"; break;
+    case kc_F3: return "F3"; break;
+    case kc_F4: return "F4"; break;
+    case kc_F5: return "F5"; break;
+    case kc_F6: return "F6"; break;
+    case kc_F7: return "F7"; break;
+    case kc_F8: return "F8"; break;
+    case kc_F9: return "F9"; break;
+    case kc_F10: return "F10"; break;
+    case kc_F11: return "F11"; break;
+    case kc_F12: return "F12"; break;
+    case kc_F13: return "F13"; break;
+    case kc_F14: return "F14"; break;
+    case kc_F15: return "F15"; break;
+
+    case kc_num_lock: return "numlock"; break;
+    case kc_caps_lock: return "capslock"; break;
+    case kc_scroll_lock: return "scrolllock"; break;
+    case kc_right_shift: return "right shift"; break;
+    case kc_left_shift: return "left shift"; break;
+    case kc_right_control: return "right control"; break;
+    case kc_left_control: return "left control"; break;
+    case kc_right_alt: return "right alt"; break;
+    case kc_left_alt: return "left alt"; break;
+    case kc_right_super: return "right super"; break;
+    case kc_left_super: return "left super"; break;
+    case kc_print_screen: return "print screen"; break;
+    case kc_system: return "system"; break;
+    case kc_break: return "break"; break;
+    case kc_menu: return "menu"; break;
+    default:
+      {
+        claw::logger << claw::log_warning << "Unknown key #" << k << '.'
+                     << std::endl;
+        return "Unknown key";
+      }
+    }
 } // keyboard::get_name_of()
 
 /*----------------------------------------------------------------------------*/
 /**
- * \brief Set the name of a key.
- * \param k The key to name.
- * \param s The string describing this key.
+ * \brief Convert a key_code to a human-readable string translated with gettext.
+ * \param k The key to convert.
  */
-void bear::input::keyboard::set_name_of( key_code k, const std::string& s )
+std::string bear::input::keyboard::get_translated_name_of( key_code k )
 {
-  CLAW_PRECOND( k < s_key_strings.size() );
+  return bear_gettext( get_name_of(k).c_str() );
+} // keyboard::get_translated_name_of()
 
-  s_key_strings[k] = s;
-} // keyboard::set_name_of()
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get a key_code from its human-readable name.
+ * \param n The name of the code.
+ */
+bear::input::key_code
+bear::input::keyboard::get_key_named( const std::string& n )
+{
+  for (key_code i=kc_range_min; i<=kc_range_max; ++i)
+    if ( get_name_of(i) == n )
+      return i;
+
+  return kc_not_a_key;
+} // keyboard::get_key_named()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -123,24 +302,17 @@ void bear::input::keyboard::refresh()
  */
 void bear::input::keyboard::refresh_events()
 {
-  std::list<SDL_Event> not_mine;
   SDL_Event e;
   m_key_events.clear();
 
-  while ( SDL_PeepEvents(&e, 1, SDL_GETEVENT, SDL_ALLEVENTS) == 1 )
-    if ( e.type == SDL_KEYDOWN )
-      {
-        SDL_KeyboardEvent* evt = reinterpret_cast<SDL_KeyboardEvent*>(&e);
-        m_key_events.push_back
-          ( key_event
-            ( key_event::key_event_character,
-              key_info(evt->keysym.sym, evt->keysym.unicode) ) );
-      }
-    else
-      not_mine.push_back(e);
+  while ( SDL_PeepEvents(&e, 1, SDL_GETEVENT, SDL_KEYDOWNMASK) == 1 ) {
+    SDL_KeyboardEvent* evt = reinterpret_cast<SDL_KeyboardEvent*>(&e);
+    m_key_events.push_back
+      ( key_event
+        ( key_event::key_event_character,
+          key_info(evt->keysym.sym, evt->keysym.unicode) ) );
+  }
 
-  for ( ; !not_mine.empty(); not_mine.pop_front() )
-    SDL_PushEvent( &not_mine.front() );
 } // keyboard::refresh_events()
 
 /*----------------------------------------------------------------------------*/
@@ -324,182 +496,3 @@ bear::input::key_code bear::input::keyboard::sdl_key_to_local
     }
 
 } // keyboard::sdl_key_to_local()
-
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Fill the m_key_string table with default strings.
- */
-void bear::input::keyboard::default_key_strings()
-{
-  s_key_strings.resize(c_key_codes_count);
-  std::fill( s_key_strings.begin(), s_key_strings.end(), "Undefined" );
-
-  s_key_strings[kc_backspace] = "backspace";
-  s_key_strings[kc_tab] = "tab";
-  s_key_strings[kc_new_line] = "new line";
-
-  s_key_strings[kc_escape] = "escape";
-  s_key_strings[kc_space] = "space";
-  s_key_strings[kc_exclamation_mark] = "!";
-  s_key_strings[kc_double_quotes] = "\"";
-  s_key_strings[kc_hash] = "#";
-  s_key_strings[kc_dollar] = "$";
-  s_key_strings[kc_percent] = "%";
-  s_key_strings[kc_ampersand] = "&";
-  s_key_strings[kc_quote] = "'";
-  s_key_strings[kc_left_parenthesis] = "(";
-  s_key_strings[kc_right_parenthesis] = ")";
-  s_key_strings[kc_asterisk] = "*";
-  s_key_strings[kc_plus] = "+";
-  s_key_strings[kc_comma] = ",";
-  s_key_strings[kc_minus] = "-";
-  s_key_strings[kc_period] = ".";
-  s_key_strings[kc_slash] = "/";
-
-  s_key_strings[kc_0] = "0";
-  s_key_strings[kc_1] = "1";
-  s_key_strings[kc_2] = "2";
-  s_key_strings[kc_3] = "3";
-  s_key_strings[kc_4] = "4";
-  s_key_strings[kc_5] = "5";
-  s_key_strings[kc_6] = "6";
-  s_key_strings[kc_7] = "7";
-  s_key_strings[kc_8] = "8";
-  s_key_strings[kc_9] = "9";
-
-  s_key_strings[kc_colon] = ":";
-  s_key_strings[kc_semicolon] = ";";
-  s_key_strings[kc_less] = "<";
-  s_key_strings[kc_equal] = "=";
-  s_key_strings[kc_greater] = ">";
-  s_key_strings[kc_question] = "?";
-  s_key_strings[kc_at] = "@";
-
-  s_key_strings[kc_A] = "A";
-  s_key_strings[kc_B] = "B";
-  s_key_strings[kc_C] = "C";
-  s_key_strings[kc_D] = "D";
-  s_key_strings[kc_E] = "E";
-  s_key_strings[kc_F] = "F";
-  s_key_strings[kc_G] = "G";
-  s_key_strings[kc_H] = "H";
-  s_key_strings[kc_I] = "I";
-  s_key_strings[kc_J] = "J";
-  s_key_strings[kc_K] = "K";
-  s_key_strings[kc_L] = "L";
-  s_key_strings[kc_M] = "M";
-  s_key_strings[kc_N] = "N";
-  s_key_strings[kc_O] = "O";
-  s_key_strings[kc_P] = "P";
-  s_key_strings[kc_Q] = "Q";
-  s_key_strings[kc_R] = "R";
-  s_key_strings[kc_S] = "S";
-  s_key_strings[kc_T] = "T";
-  s_key_strings[kc_U] = "U";
-  s_key_strings[kc_V] = "V";
-  s_key_strings[kc_W] = "W";
-  s_key_strings[kc_X] = "X";
-  s_key_strings[kc_Y] = "Y";
-  s_key_strings[kc_Z] = "Z";
-
-  s_key_strings[kc_left_bracket] = "[";
-  s_key_strings[kc_backslash] = "\\";
-  s_key_strings[kc_right_bracket] = "]";
-  s_key_strings[kc_caret] = "^";
-  s_key_strings[kc_underscore] = "_";
-  s_key_strings[kc_backquote] = "`";
-
-  s_key_strings[kc_a] = "a";
-  s_key_strings[kc_b] = "b";
-  s_key_strings[kc_c] = "c";
-  s_key_strings[kc_d] = "d";
-  s_key_strings[kc_e] = "e";
-  s_key_strings[kc_f] = "f";
-  s_key_strings[kc_g] = "g";
-  s_key_strings[kc_h] = "h";
-  s_key_strings[kc_i] = "i";
-  s_key_strings[kc_j] = "j";
-  s_key_strings[kc_k] = "k";
-  s_key_strings[kc_l] = "l";
-  s_key_strings[kc_m] = "m";
-  s_key_strings[kc_n] = "n";
-  s_key_strings[kc_o] = "o";
-  s_key_strings[kc_p] = "p";
-  s_key_strings[kc_q] = "q";
-  s_key_strings[kc_r] = "r";
-  s_key_strings[kc_s] = "s";
-  s_key_strings[kc_t] = "t";
-  s_key_strings[kc_u] = "u";
-  s_key_strings[kc_v] = "v";
-  s_key_strings[kc_w] = "w";
-  s_key_strings[kc_x] = "x";
-  s_key_strings[kc_y] = "y";
-  s_key_strings[kc_z] = "z";
-
-  s_key_strings[kc_left_brace] = "{";
-  s_key_strings[kc_vertical_line] = "|";
-  s_key_strings[kc_right_brace] = "}";
-  s_key_strings[kc_tilde] = "~";
-
-  s_key_strings[kc_keypad_0] = "keypad 0";
-  s_key_strings[kc_keypad_1] = "keypad 1";
-  s_key_strings[kc_keypad_2] = "keypad 2";
-  s_key_strings[kc_keypad_3] = "keypad 3";
-  s_key_strings[kc_keypad_4] = "keypad 4";
-  s_key_strings[kc_keypad_5] = "keypad 5";
-  s_key_strings[kc_keypad_6] = "keypad 6";
-  s_key_strings[kc_keypad_7] = "keypad 7";
-  s_key_strings[kc_keypad_8] = "keypad 8";
-  s_key_strings[kc_keypad_9] = "keypad 9";
-
-  s_key_strings[kc_keypad_period] = "keypad .";
-  s_key_strings[kc_keypad_divide] = "keypad /";
-  s_key_strings[kc_keypad_multiply] = "keypad *";
-  s_key_strings[kc_keypad_minus] = "keypad -";
-  s_key_strings[kc_keypad_plus] = "keypad +";
-  s_key_strings[kc_keypad_enter] = "keypad enter";
-  s_key_strings[kc_keypad_equals] = "keypad =";
-
-  s_key_strings[kc_up] = "up";
-  s_key_strings[kc_down] = "down";
-  s_key_strings[kc_right] = "right";
-  s_key_strings[kc_left] = "left";
-
-  s_key_strings[kc_insert] = "insert";
-  s_key_strings[kc_home] = "home";
-  s_key_strings[kc_end] = "end";
-  s_key_strings[kc_page_up] = "page up";
-  s_key_strings[kc_page_down] = "page down";
-
-  s_key_strings[kc_F1] = "F1";
-  s_key_strings[kc_F2] = "F2";
-  s_key_strings[kc_F3] = "F3";
-  s_key_strings[kc_F4] = "F4";
-  s_key_strings[kc_F5] = "F5";
-  s_key_strings[kc_F6] = "F6";
-  s_key_strings[kc_F7] = "F7";
-  s_key_strings[kc_F8] = "F8";
-  s_key_strings[kc_F9] = "F9";
-  s_key_strings[kc_F10] = "F10";
-  s_key_strings[kc_F11] = "F11";
-  s_key_strings[kc_F12] = "F12";
-  s_key_strings[kc_F13] = "F13";
-  s_key_strings[kc_F14] = "F14";
-  s_key_strings[kc_F15] = "F15";
-
-  s_key_strings[kc_num_lock] = "numlock";
-  s_key_strings[kc_caps_lock] = "capslock";
-  s_key_strings[kc_scroll_lock] = "scrolllock";
-  s_key_strings[kc_right_shift] = "right shift";
-  s_key_strings[kc_left_shift] = "left shift";
-  s_key_strings[kc_right_control] = "right control";
-  s_key_strings[kc_left_control] = "left control";
-  s_key_strings[kc_right_alt] = "right alt";
-  s_key_strings[kc_left_alt] = "left alt";
-  s_key_strings[kc_right_super] = "right super";
-  s_key_strings[kc_left_super] = "left super";
-  s_key_strings[kc_print_screen] = "print screen";
-  s_key_strings[kc_system] = "system";
-  s_key_strings[kc_break] = "break";
-  s_key_strings[kc_menu] = "menu";
-} // keyboard::default_key_strings()
