@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -31,6 +31,8 @@
 
 #include "time/class_export.hpp"
 
+#include <string>
+
 namespace bear
 {
   namespace systime
@@ -40,6 +42,9 @@ namespace bear
     void TIME_EXPORT sleep( milliseconds_type ms );
     milliseconds_type TIME_EXPORT get_date_ms();
 
+    std::string TIME_EXPORT format_time_s
+    ( unsigned long t, const std::string& format );
+
   } // namespace systime
 } // namespace bear
 
