@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -32,7 +32,9 @@
 
 #include "universe/collision_info.hpp"
 #include "universe/collision_repair.hpp"
+#include "universe/const_item_handle.hpp"
 #include "universe/world.hpp"
+#include "universe/zone.hpp"
 #include "universe/link/base_link.hpp"
 
 #include <claw/assert.hpp>
@@ -236,7 +238,6 @@ void bear::universe::physical_item::time_step( time_type elapsed_time )
  */
 void bear::universe::physical_item::enters_active_region()
 {
-  // nothing to do
 } // physical_item::enters_active_region()
 
 /*----------------------------------------------------------------------------*/
@@ -245,7 +246,7 @@ void bear::universe::physical_item::enters_active_region()
  */
 void bear::universe::physical_item::leaves_active_region()
 {
-  // nothing to do
+  get_world_progress_structure().unselect();
 } // physical_item::leaves_active_region()
 
 /*----------------------------------------------------------------------------*/
@@ -268,6 +269,25 @@ void bear::universe::physical_item::collision( collision_info& info )
 
 } // physical_item::collision()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Check if this item is in collision with an other item.
+ * \param that The item to check the collision with.
+ */
+bool
+bear::universe::physical_item::collides_with( const physical_item& that ) const
+{
+  if ( get_bounding_box().intersects( that.get_bounding_box() ) )
+    {
+      const rectangle_type inter =
+        get_bounding_box().intersection( that.get_bounding_box() );
+
+      return (inter.width() != 0) && (inter.height() != 0);
+    }
+  else
+    return false;
+} // physical_item::collides_with()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell this item which world owns him.
@@ -359,8 +379,7 @@ void bear::universe::physical_item::remove_all_links()
   // we don't have to remove it from here.
   while ( !m_links.empty() )
     {
-      base_link* link = m_links.front();
-      delete link;
+      delete m_links.front();
     }
 } // physical_item::remove_all_links()
 
@@ -447,9 +466,10 @@ void bear::universe::physical_item::adjust_cinetic()
         s.y = 0;
 
       set_speed(s);
-    }
 
-  set_speed( s );
+      if ( (get_angular_speed() < eps.x) && (get_angular_speed() > -eps.x) )
+        set_angular_speed(0);
+    }
 } // physical_item::adjust_cinetic()
 
 /*----------------------------------------------------------------------------*/
@@ -466,7 +486,7 @@ bool bear::universe::physical_item::has_owner() const
  * \brief Get the world that owns this item.
  * \pre has_owner() == true
  */
-bear::universe::world& bear::universe::physical_item::get_owner() const
+const bear::universe::world& bear::universe::physical_item::get_owner() const
 {
   CLAW_PRECOND( has_owner() );
   return *m_owner;
@@ -810,6 +830,9 @@ void bear::universe::physical_item::update_position( time_type elapsed_time )
   speed *= f;
   set_speed( speed );
 
+  set_system_angle( get_system_angle() + get_angular_speed() );
+  set_angular_speed( get_angular_speed() * f );
+
   pos += speed * elapsed_time;
 
   set_bottom_left(pos);
