@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -52,7 +52,7 @@ bear::universe::physical_item_state::physical_item_state()
 bear::universe::physical_item_state::physical_item_state
 ( const physical_item_state& that )
   : physical_item_attributes(that), m_fixed(false), m_x_fixed(that.m_x_fixed),
-    m_y_fixed(that.m_y_fixed), m_global(false)
+    m_y_fixed(that.m_y_fixed), m_global(that.m_global)
 {
 
 } // physical_item_state::physical_item_state()
@@ -95,6 +95,18 @@ bear::universe::physical_item_state::get_height() const
   return m_size.y;
 } // physical_item_state::get_height()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the bouding box of this item.
+ * \param r the new bounding box of the item.
+ */
+void bear::universe::physical_item_state::set_bounding_box
+( const bear::universe::rectangle_type& r )
+{
+  set_bottom_left(r.bottom_left());
+  set_size( r.size() );
+} // physical_item_state::set_bounding_box()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the bouding box of this item.
@@ -278,6 +290,24 @@ void bear::universe::physical_item_state::set_speed( const speed_type& speed )
     }
 } // physical_item_state::set_speed()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the current angular speed of the item.
+ */
+double bear::universe::physical_item_state::get_angular_speed() const
+{
+  return m_angular_speed;
+} // physical_item_state::get_angular_speed()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set the current angular speed of the item.
+ */
+void bear::universe::physical_item_state::set_angular_speed( double speed )
+{
+  m_angular_speed = speed;
+} // physical_item_state::set_angular_speed()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the friction applied to this item.
@@ -696,6 +726,7 @@ bear::universe::physical_item_state::get_center_of_mass() const
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Set (force) the position of the center of the item.
+ * \param pos The new center of mass.
  */
 void bear::universe::physical_item_state::set_center_of_mass
 ( const position_type& pos )
@@ -703,6 +734,18 @@ void bear::universe::physical_item_state::set_center_of_mass
   set_bottom_left( pos - m_size / 2 );
 } // physical_item_state::set_center_of_mass()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Set (force) the position of the center of the item.
+ * \param x The new x_coordinate of the center of mass.
+ * \param y The new y-coordinate of the center of mass.
+ */
+void bear::universe::physical_item_state::set_center_of_mass
+( const coordinate_type& x, const coordinate_type& y )
+{
+  set_center_of_mass( position_type(x,y) );
+} // physical_item_state::set_center_of_mass()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell if the item is fixed.
@@ -897,46 +940,51 @@ bool bear::universe::physical_item_state::has_contact() const
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell that the item has a contact on its left.
+ * \param contact The new contact status.
  */
-void bear::universe::physical_item_state::set_left_contact()
+void bear::universe::physical_item_state::set_left_contact(bool contact)
 {
-  m_left_contact = true;
+  m_left_contact = contact;
 } // physical_item_state::set_left_contact()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell that the item has a contact on its right.
+ * \param contact The new contact status.
  */
-void bear::universe::physical_item_state::set_right_contact()
+void bear::universe::physical_item_state::set_right_contact(bool contact)
 {
-  m_right_contact = true;
+  m_right_contact = contact;
 } // physical_item_state::set_right_contact()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell that the item has a contact on its top.
+ * \param contact The new contact status.
  */
-void bear::universe::physical_item_state::set_top_contact()
+void bear::universe::physical_item_state::set_top_contact(bool contact)
 {
-  m_top_contact = true;
+  m_top_contact = contact;
 } // physical_item_state::set_top_contact()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell that the item has a contact on its bottom.
+ * \param contact The new contact status.
  */
-void bear::universe::physical_item_state::set_bottom_contact()
+void bear::universe::physical_item_state::set_bottom_contact(bool contact)
 {
-  m_bottom_contact = true;
+  m_bottom_contact = contact;
 } // physical_item_state::set_bottom_contact()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell that the item has a contact in middle zone.
+ * \param contact The new contact status.
  */
-void bear::universe::physical_item_state::set_middle_contact()
+void bear::universe::physical_item_state::set_middle_contact(bool contact)
 {
-  m_middle_contact = true;
+  m_middle_contact = contact;
 } // physical_item_state::set_middle_contact()
 
 /*----------------------------------------------------------------------------*/
@@ -989,6 +1037,26 @@ bool bear::universe::physical_item_state::is_artificial() const
   return m_is_artificial;
 } // physical_item_state::is_artificial()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Indicate if the item is considered for weak or strong collisions.
+ * \param w The new status.
+ */
+void bear::universe::physical_item_state::set_weak_collisions( bool w )
+{
+  m_weak_collisions = w;
+} // physical_item_state::set_weak_collisions()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Tell if the item has to be considered as weak collisions.
+ * \param w The new status.
+ */
+bool bear::universe::physical_item_state::has_weak_collisions() const
+{
+  return m_weak_collisions;
+} // physical_item_state::has_weak_collisions()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Set the size of the object.
@@ -1018,7 +1086,8 @@ void bear::universe::physical_item_state::set_size
  */
 void bear::universe::physical_item_state::set_width( size_type width )
 {
-  m_size.x = width;
+  if (!m_fixed && (m_x_fixed == 0))
+    m_size.x = width;
 } // physical_item_state::set_width()
 
 /*----------------------------------------------------------------------------*/
@@ -1028,7 +1097,8 @@ void bear::universe::physical_item_state::set_width( size_type width )
  */
 void bear::universe::physical_item_state::set_height( size_type height )
 {
-  m_size.y = height;
+  if (!m_fixed && (m_y_fixed == 0))
+    m_size.y = height;
 } // physical_item_state::set_height()
 
 /*----------------------------------------------------------------------------*/
@@ -1073,7 +1143,8 @@ void bear::universe::physical_item_state::to_string( std::string& str ) const
   oss << "\ndensity: " << m_density;
   oss << "\nangle: " << m_system_angle;
   oss << "\nfixed: " << m_fixed << ' ' << m_x_fixed << ' ' << m_y_fixed;
-  oss << "\ncan move items: " << m_can_move_items;
+  oss << "\nphantom/c.m.i./art.: " << m_is_phantom << ' ' << m_can_move_items
+      << ' ' << m_is_artificial;
   oss << "\ncontact after collision: " << m_contact_after_collision;
   oss << "\ncontact: { ";
 
