@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -30,7 +30,10 @@
 
 #include "universe/collision_info.hpp"
 #include "universe/collision_repair.hpp"
+#include "universe/density_rectangle.hpp"
 #include "universe/environment_rectangle.hpp"
+#include "universe/force_rectangle.hpp"
+#include "universe/friction_rectangle.hpp"
 #include "universe/link/base_link.hpp"
 
 #include <algorithm>
@@ -94,7 +97,8 @@ void bear::universe::world::progress_entities
   // search each item in the active zone and global item
   search_interesting_items(regions, items);
   assert
-    ( item_set(items.begin(), items.end()).size() == items.size() );
+    ( std::set<physical_item*>(items.begin(), items.end()).size()
+      == items.size() );
 
   // call progress for each interesting item
   progress_items(items, elapsed_time);
@@ -317,7 +321,6 @@ bear::universe::world::get_average_force( const rectangle_type& r ) const
   if (r_area != 0)
     {
       std::list<force_rectangle*>::const_iterator it;
-      double sum_area(0);
 
       for ( it=m_force_rectangle.begin(); it!=m_force_rectangle.end();
             ++it )
@@ -325,7 +328,6 @@ bear::universe::world::get_average_force( const rectangle_type& r ) const
           {
             const double area = r.intersection( (*it)->rectangle ).area();
             result += area / r_area * (*it)->force;
-            sum_area += area;
           }
     }
 
@@ -452,14 +454,14 @@ bool bear::universe::world::is_in_environment
   bool result = false;
 
   std::list<environment_rectangle*>::const_iterator it;
-  
+
   for ( it=m_environment_rectangle.begin();
-	(it!=m_environment_rectangle.end()) && !result;
-	++it )
-    if ( ( (*it)->environment== environment ) && 
-	 (*it)->rectangle.includes( pos ) )
+        (it!=m_environment_rectangle.end()) && !result;
+        ++it )
+    if ( ( (*it)->environment== environment ) &&
+         (*it)->rectangle.includes( pos ) )
       result = true;
-  
+
   return result;
 } // world::is_in_environment()
 
@@ -534,12 +536,7 @@ void bear::universe::world::pick_items_in_rectangle
 
   for ( it=candidates.begin(); it!=candidates.end(); ++it )
     if ( (*it)->get_bounding_box().intersects(r) )
-      {
-        const rectangle_type inter( (*it)->get_bounding_box().intersection(r) );
-
-        if ( (inter.width() != 0) && (inter.height() != 0) )
-          items.push_back(*it);
-      }
+      items.push_back(*it);
 } // world::pick_items_in_rectangle()
 
 /*----------------------------------------------------------------------------*/
@@ -665,15 +662,15 @@ void bear::universe::world::list_active_items
 ( item_list& items, const region_type& regions,
   const item_picking_filter& filter ) const
 {
-  item_set static_items;
+  item_list static_items;
 
   list_static_items( regions, static_items );
-  item_set::const_iterator its;
-  for (its=static_items.begin(); its!=static_items.end(); ++its)
-    if ( filter.satisfies_condition(**its) )
-      items.push_back(*its);
-
   item_list::const_iterator it;
+
+  for (it=static_items.begin(); it!=static_items.end(); ++it)
+    if ( filter.satisfies_condition(**it) )
+      items.push_back(*it);
+
   for ( it=m_entities.begin(); it!=m_entities.end(); ++it )
     if ( item_in_regions(**it, regions) && filter.satisfies_condition(**it) )
       items.push_back( *it );
@@ -687,9 +684,9 @@ void bear::universe::world::list_active_items
 void bear::universe::world::detect_collision_all( item_list& items )
 {
   item_list pending;
-  
+
   for (item_list::iterator it=items.begin(); it!=items.end(); ++it)
-    if ( !(*it)->is_artificial() && !(*it)->is_fixed())
+    if ( !(*it)->is_fixed() )
       add_to_collision_queue(pending, *it);
 
   while ( !pending.empty() )
@@ -702,12 +699,9 @@ void bear::universe::world::detect_collision_all( item_list& items )
 
 /*----------------------------------------------------------------------------*/
 /**
- * \brief Detect and correct the collisions of an item.
- * \param item The item for which we search the collisions.
- * \param coll_detection (in/out) The instance of collision detection.
- * \param pending (out) A list in which are added the items in collision.
- * \param all_items (out) The set of all items processed in the iteration of the
- *        current world::progress() call.
+ * \brief Find the next item to process in the collision detection.
+ * \param pending The list of items waiting for the collision detection.
+ * \return The first item with the greatest mass and the largest collision area.
  */
 bear::universe::physical_item*
 bear::universe::world::pick_next_collision( item_list& pending ) const
@@ -755,6 +749,8 @@ bear::universe::world::pick_next_collision( item_list& pending ) const
 void bear::universe::world::detect_collision
 ( physical_item* item, item_list& pending, item_list& all_items ) const
 {
+  CLAW_PRECOND( !item->has_weak_collisions() );
+
   physical_item* it = item->get_world_progress_structure().pick_next_neighbor();
 
   if (it != NULL)
@@ -794,33 +790,27 @@ bool bear::universe::world::process_collision
 {
   bool result(false);
 
-  if ( self.get_bounding_box().intersects( that.get_bounding_box() ) )
+  if ( self.collides_with(that) )
     {
-      const rectangle_type inter =
-        self.get_bounding_box().intersection( that.get_bounding_box() );
+      result = true;
+      collision_repair repair(self, that);
 
-      if( (inter.width() != 0) && (inter.height() != 0) )
-        {
-          result = true;
-          collision_repair repair(self, that);
+      collision_info info_ab
+        ( self.get_world_progress_structure().get_initial_state(),
+          that.get_world_progress_structure().get_initial_state(),
+          self, that, repair );
+      collision_info info_ba
+        ( that.get_world_progress_structure().get_initial_state(),
+          self.get_world_progress_structure().get_initial_state(),
+          that, self, repair );
 
-          collision_info info_ab
-            ( self.get_world_progress_structure().get_initial_state(),
-              that.get_world_progress_structure().get_initial_state(),
-              self, that, repair );
-          collision_info info_ba
-            ( that.get_world_progress_structure().get_initial_state(),
-              self.get_world_progress_structure().get_initial_state(),
-              that, self, repair );
+      self.collision(info_ab);
+      that.collision(info_ba);
 
-          self.collision(info_ab);
-          that.collision(info_ba);
+      repair.apply();
 
-          repair.apply();
-
-          self.adjust_cinetic();
-          that.adjust_cinetic();
-        }
+      self.adjust_cinetic();
+      that.adjust_cinetic();
     }
 
   return result;
@@ -842,9 +832,9 @@ void bear::universe::world::search_items_for_collision
   const rectangle_type r(item.get_bounding_box());
 
   // add static items
-  item_set static_items;
-  item_set::const_iterator its;
-  m_static_surfaces.get_area( r, static_items );
+  item_list static_items;
+  item_list::const_iterator its;
+  m_static_surfaces.get_area_unique( r, static_items );
 
   for( its=static_items.begin(); its!=static_items.end(); ++its)
     if ( !(*its)->is_artificial()
@@ -873,8 +863,8 @@ void bear::universe::world::search_items_for_collision
  *        \a mass.
  */
 void bear::universe::world::item_found_in_collision
-( const physical_item& item, physical_item* it, item_list& colliding, double& mass,
-  double& area ) const
+( const physical_item& item, physical_item* it, item_list& colliding,
+  double& mass, double& area ) const
 {
   const double a =
     it->get_bounding_box().intersection(item.get_bounding_box()).area();
@@ -933,17 +923,13 @@ void bear::universe::world::search_interesting_items
 ( const region_type& regions, item_list& items ) const
 {
   item_list::const_iterator it;
-  region_type::const_iterator itr;
 
   // add static items of the active zone
-  for (itr=regions.begin(); itr!=regions.end(); ++itr)
-    {
-      item_list static_items;
-      m_static_surfaces.get_area( *itr, static_items );
-      
-      for( it=static_items.begin(); it!=static_items.end(); ++it)
-        select_item(items, *it);
-    }
+  item_list static_items;
+  m_static_surfaces.get_areas( regions.begin(), regions.end(), static_items );
+
+  for( it=static_items.begin(); it!=static_items.end(); ++it)
+    select_item(items, *it);
 
   // add global static items
   for (it=m_global_static_items.begin(); it!=m_global_static_items.end(); ++it)
@@ -978,7 +964,7 @@ void bear::universe::world::stabilize_dependent_items( item_list& items ) const
       pending.pop_front();
       g.add_vertex(src);
 
-      // get the item relative to which I move
+      // get the item relatively to which I move
       physical_item* const ref( src->get_movement_reference() );
 
       if ( ref != NULL )
@@ -1061,22 +1047,20 @@ void bear::universe::world::progress_physic_move_item
       bool move_it(true);
 
       if ( ref != NULL )
-        {
-          if ( !ref->get_world_progress_structure().move_is_done() )
-            {
-              claw::logger << claw::log_warning
-                           << "The reference item for the movement has not "
-                           << "been moved. The current item will not be moved. "
-                           << "The reference item is ";
+        if ( !ref->get_world_progress_structure().move_is_done() )
+          {
+            claw::logger << claw::log_warning
+                         << "The reference item for the movement has not "
+                         << "been moved yet, so the current item will not be "
+                         << "moved. The reference item is ";
 
-              if ( !ref->get_world_progress_structure().is_selected() )
-                claw::logger << claw::log_warning << "not ";
+            if ( !ref->get_world_progress_structure().is_selected() )
+              claw::logger << "not ";
 
-              claw::logger << claw::log_warning << "selected." << std::endl;
-                
-              move_it = false;
-            }
-        }
+            claw::logger << "selected for movement." << std::endl;
+
+            move_it = false;
+          }
 
       if ( move_it )
         {
@@ -1136,12 +1120,9 @@ void bear::universe::world::active_region_traffic
  * \param regions A list of regions in which we take the items.
  */
 void bear::universe::world::list_static_items
-( const region_type& regions, item_set& items ) const
+( const region_type& regions, item_list& items ) const
 {
-  region_type::const_iterator it;
-
-  for (it=regions.begin(); it!=regions.end(); ++it)
-    m_static_surfaces.get_area( *it, items);
+  m_static_surfaces.get_areas_unique( regions.begin(), regions.end(), items );
 } // world::list_static_items()
 
 /*----------------------------------------------------------------------------*/
@@ -1234,12 +1215,13 @@ void bear::universe::world::unselect_item
 void bear::universe::world::add_to_collision_queue
 ( item_list& pending, physical_item* item ) const
 {
-  if ( create_neighborhood(*item) )
-    if ( !item->get_world_progress_structure().is_waiting_for_collision() )
-      {
-        item->get_world_progress_structure().set_waiting_for_collision();
-        pending.push_back(item);
-      }
+  if ( !item->has_weak_collisions() && !item->is_artificial() )
+    if ( create_neighborhood(*item) )
+      if ( !item->get_world_progress_structure().is_waiting_for_collision() )
+        {
+          item->get_world_progress_structure().set_waiting_for_collision();
+          pending.push_back(item);
+        }
 } // world::add_to_collision_queue()
 
 /*----------------------------------------------------------------------------*/
