@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -30,3 +30,14 @@
 #include "engine/export.hpp"
 
 BASE_ITEM_EXPORT( reference_item, bear )
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Constructor.
+ */
+bear::reference_item::reference_item()
+{
+  set_phantom(true);
+  set_artificial(true);
+  set_can_move_items(false);
+} // reference_item::reference_item()
