@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -33,3 +33,8 @@ BASE_ITEM_EXPORT( boolean_equality_creator, bear )
 BASE_ITEM_EXPORT( boolean_disequality_creator, bear )
 BASE_ITEM_EXPORT( logical_and_creator, bear )
 BASE_ITEM_EXPORT( logical_or_creator, bear )
+
+BASE_ITEM_EXPORT( linear_plus_creator, bear )
+BASE_ITEM_EXPORT( linear_minus_creator, bear )
+BASE_ITEM_EXPORT( linear_multiplies_creator, bear )
+BASE_ITEM_EXPORT( linear_divides_creator, bear )
