@@ -1,7 +1,7 @@
 /*
   Bear Engine
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -44,8 +44,10 @@ namespace bear
   class GENERIC_ITEMS_EXPORT reference_item:
     public engine::base_item
   {
-    DECLARE_BASE_ITEM(reference_item, bear);
+    DECLARE_BASE_ITEM(reference_item);
 
+  public:
+    reference_item();
 
   }; // class reference_item
 } // namespace bear
