@@ -1,7 +1,7 @@
 /*
     Bear Engine - Editor library
 
-    Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+    Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
     This program is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by the
@@ -36,7 +36,7 @@
  */
 bf::font_file_edit::font_file_edit
 ( wxWindow& parent, const font_file_type& v )
-  : super(parent, _("font (*.fnt)|*.fnt"), v)
+  : super(parent, _("Font files (*.fnt)|*.fnt"), v)
 {
 
 } // font_file_edit::font_file_edit()
