@@ -1,7 +1,7 @@
 /*
   Bear Engine - Level editor
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -32,16 +32,108 @@
 #include "bf/custom_type.hpp"
 #include "bf/free_edit.hpp"
 #include "bf/item_class_pool.hpp"
+#include "bf/item_reference_edit.hpp"
 #include "bf/set_edit.hpp"
 #include "bf/wx_facilities.hpp"
-#include "bf/animation_file_edit.hpp"
+#include "bf/any_animation_edit.hpp"
 #include "bf/font_file_edit.hpp"
-#include "bf/sample_file_edit.hpp"
+#include "bf/sample_edit.hpp"
 #include "bf/sprite_edit.hpp"
 
+#include "bf/call_by_field_type.hpp"
+
 #include <list>
 #include <claw/assert.hpp>
 
+/*----------------------------------------------------------------------------*/
+const wxEventType
+bf::delete_item_field_event::delete_field_event_type = wxNewEventType();
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Constructor.
+ * \param field_name The name of the cleared field.
+ * \param t The type of the event.
+ * \param id The id of the window that generates the event.
+ */
+bf::delete_item_field_event::delete_item_field_event
+( const std::string& field_name, wxEventType t, wxWindowID id )
+  : wxNotifyEvent(t, id), m_field_name(field_name)
+{
+
+} // delete_item_field_event::delete_item_field_event()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Allocate a copy of this instance.
+ */
+wxEvent* bf::delete_item_field_event::Clone() const
+{
+  return new delete_item_field_event(*this);
+} // delete_item_field_event::Clone()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the name of the field to clear.
+ */
+const std::string& bf::delete_item_field_event::get_field_name() const
+{
+  return m_field_name;
+} // delete_item_field_event::get_field_name()
+
+
+
+
+/*----------------------------------------------------------------------------*/
+const wxEventType
+bf::request_item_id_event::request_item_id_event_type = wxNewEventType();
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Constructor.
+ * \param field The field to which the id will be given.
+ * \param val The array in which the identifiers will be stored.
+ * \param t The type of the event.
+ * \param id The id of the window that generates the event.
+ */
+bf::request_item_id_event::request_item_id_event
+( const type_field& field, wxArrayString& val, wxEventType t, wxWindowID id )
+  : wxNotifyEvent(t, id), m_field(field), m_values(val)
+{
+
+} // request_item_id_event::request_item_id_event()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Allocate a copy of this instance.
+ */
+wxEvent* bf::request_item_id_event::Clone() const
+{
+  return new request_item_id_event(*this);
+} // request_item_id_event::Clone()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the field to which the id will be given.
+ */
+const bf::type_field& bf::request_item_id_event::get_type_field() const
+{
+  return m_field;
+} // request_item_id_event::get_type_field()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Add an id in the list of valid identifiers.
+ * \param id The identifier to add.
+ */
+void bf::request_item_id_event::add_id( const wxString& id ) const
+{
+  m_values.Add(id);
+} // request_item_id_event::add_id()
+
+
+
+
 /*----------------------------------------------------------------------------*/
 const wxColour bf::item_field_edit::s_field_prefix_colour(*wxLIGHT_GREY);
 const std::string bf::item_field_edit::s_no_prefix("- no class -");
@@ -49,15 +141,13 @@ const std::string bf::item_field_edit::s_no_prefix("- no class -");
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Constructor.
- * \param p An object that will do the modifications on the item.
  * \param parent Pointer to the owner.
  * \param id The identifier of this window.
  */
-bf::item_field_edit::item_field_edit
-( proxy& p, wxWindow* parent, wxWindowID id )
+bf::item_field_edit::item_field_edit( wxWindow* parent, wxWindowID id )
   : wxListView( parent, id, wxDefaultPosition, wxDefaultSize,
-                wxLC_REPORT | wxLC_VRULES | wxLC_SINGLE_SEL ), m_proxy(p),
-    m_item(NULL)
+                wxLC_REPORT | wxLC_VRULES | wxLC_SINGLE_SEL ),
+    m_last_selected_field(wxNOT_FOUND)
 {
   InsertColumn(0, _("Property"));
   InsertColumn(1, _("Value"));
@@ -72,63 +162,188 @@ bf::item_field_edit::item_field_edit
            wxListEventHandler(item_field_edit::on_item_activated) );
   Connect( wxEVT_KEY_UP,
            wxKeyEventHandler(item_field_edit::on_key_up) );
-
 } // item_field_edit::item_field_edit()
 
 /*----------------------------------------------------------------------------*/
 /**
- * \brief Set the item for which we want the properties.
- * \param item The item instance concerned by this window.
+ * \brief Refresh the values of the fields.
  */
-void bf::item_field_edit::set_item( item_instance* item )
+void bf::item_field_edit::refresh()
 {
+  update_values();
+} // item_field_edit::refresh()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Add an item for which we want the properties.
+ * \param item The item instance concerned by this window.
+ * \return true if something has changed.
+ */
+bool bf::item_field_edit::add_item( item_instance* item )
+{
+  if ( item == NULL )
+    return clear();
+
+  if ( m_group.insert(item).second )
+    {
+      fill_fields();
+      return true;
+    }
+  else
+    return false;
+} // item_field_edit::add_item()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Stop editing the properties of an item.
+ * \param item The item to remove.
+ * \return true if something has changed.
+ */
+bool bf::item_field_edit::remove_item( item_instance* item )
+{
+  if ( m_group.erase(item) == 0 )
+    return false;
+
+  fill_fields();
+  return true;
+} // item_field_edit::remove_item()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Remove all items.
+ * \return true if something has changed.
+ */
+bool bf::item_field_edit::clear()
+{
+  bool result = !empty();
+
+  m_group.clear();
   int index = GetFirstSelected();
-  m_item = item;
+
+  if (index != wxNOT_FOUND)
+    m_last_selected_field = index;
+
   DeleteAllItems();
 
-  if ( m_item != NULL )
+  return result;
+} // item_field_edit::clear()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Tell if the control contains no items.
+ */
+bool bf::item_field_edit::empty() const
+{
+  return m_group.empty();
+} // item_field_edit::empty()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Tell if there is only one edited item.
+ */
+bool bf::item_field_edit::has_single_item() const
+{
+  if ( empty() )
+    return false;
+  else
+    return ++begin() == end();
+} // item_field_edit::has_single_item()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the single edited item.
+ */
+bf::item_instance& bf::item_field_edit::get_single_item() const
+{
+  CLAW_PRECOND(has_single_item());
+
+  return *begin();
+} // item_field_edit::get_single_item()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get an iterator on the first edited item.
+ */
+bf::item_field_edit::item_iterator bf::item_field_edit::begin() const
+{
+  return m_group.begin();
+} // item_field_edit::begin()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get an iterator just after the last edited item.
+ */
+bf::item_field_edit::item_iterator bf::item_field_edit::end() const
+{
+  return m_group.end();
+} // item_field_edit::end()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get a pointer to the class of all items if they all have the same
+ *        class.
+ */
+const bf::item_class* bf::item_field_edit::get_common_class() const
+{
+  if ( empty() )
+    return NULL;
+
+  const item_class* result(NULL);
+
+  item_iterator it=begin();
+  result = it->get_class_ptr();
+
+  for (++it; (result != NULL) && (it!=end()); ++it)
+    if ( result != it->get_class_ptr() )
+      result = NULL;
+
+  return result;
+} // item_field_edit::get_common_class()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the field with a given name, common to all items.
+ * \param name The name of the field.
+ */
+const bf::type_field&
+bf::item_field_edit::get_common_field( const std::string& name ) const
+{
+  CLAW_PRECOND( !empty() );
+
+#ifndef NDEBUG
+  for (item_iterator it=begin(); it!=end(); ++it)
     {
-      m_item_class_name = m_item->get_class().get_class_name();
-      enumerate_properties();
-
-      if (index != wxNOT_FOUND )
-        {
-          Select(index);
-          EnsureVisible(index);
-        }
-
-      update_values();
+      CLAW_ASSERT( it->get_class().has_field(name),
+                   "Class '" + it->get_class().get_class_name() +
+                   "' has no field named '" + name + "'." );
     }
-} // item_field_edit::set_item()
 
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Tell if there is an item edited.
- */
-bool bf::item_field_edit::has_item() const
-{
-  return m_item != NULL;
-} // item_field_edit::has_item()
+  const type_field::field_type check_type =
+    begin()->get_class().get_field(name).get_field_type();
 
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Get the item currently edited.
- */
-bf::item_instance& bf::item_field_edit::get_item()
-{
-  CLAW_PRECOND( has_item() );
-  return *m_item;
-} // item_field_edit::get_item()
+  for (item_iterator it=begin(); it!=end(); ++it)
+    {
+      CLAW_ASSERT( it->get_class().get_field(name).get_field_type()
+                   == check_type,
+                   "Class '" + it->get_class().get_class_name() +
+                   "' has a field named '" + name
+                   + "' but with a different type." );
+    }
 
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Get the item currently edited.
- */
-const bf::item_instance& bf::item_field_edit::get_item() const
-{
-  CLAW_PRECOND( has_item() );
-  return *m_item;
-} // item_field_edit::get_item()
+  const bool check_list =
+    begin()->get_class().get_field(name).is_list();
+
+  for (item_iterator it=begin(); it!=end(); ++it)
+    {
+      CLAW_ASSERT( it->get_class().get_field(name).is_list() == check_list,
+                   "Class '" + it->get_class().get_class_name() +
+                   "' has a field named '" + name
+                   + "' but with a different list status." );
+    }
+#endif
+
+  return (*m_group.begin())->get_class().get_field(name);
+} // item_field_edit::get_common_field()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -161,7 +376,15 @@ bf::item_field_edit::get_field_name( unsigned int i, std::string& name ) const
         name = field_prefix + '.' + name;
     }
 
-  CLAW_POSTCOND( !result || m_item->get_class().has_field(name) );
+#ifndef NDEBUG
+  bool check_all_items_have_field(true);
+
+  for ( item_iterator it=begin(); check_all_items_have_field && (it!=end());
+        ++it )
+    check_all_items_have_field = it->get_class().has_field(name);
+
+  CLAW_POSTCOND( !result || check_all_items_have_field );
+#endif
 
   return result;
 } // item_field_edit::get_field_name()
@@ -172,21 +395,25 @@ bf::item_field_edit::get_field_name( unsigned int i, std::string& name ) const
  */
 void bf::item_field_edit::enumerate_properties()
 {
-  std::list<item_class const*> hierarchy;
-  std::list<item_class const*>::const_iterator it;
+  if (empty())
+    return;
 
-  m_item->get_class().find_hierarchy( hierarchy );
+  std::list<std::string> fields;
+  item_iterator it=begin();
+  it->get_class().get_field_names_in_hierarchy(fields);
 
-  std::vector<std::string> fields;
+  for (++it; it!=end(); ++it)
+    {
+      std::list<std::string> f;
+      it->get_class().get_field_names_in_hierarchy(f);
 
-  for ( it=hierarchy.begin(); it!=hierarchy.end(); ++it )
-    get_fields_of(fields, **it);
+      std::list<std::string> old_f;
+      old_f.swap(fields);
 
-  std::sort( fields.begin(), fields.end() );
-  const std::vector<std::string>::iterator eit =
-    std::unique(fields.begin(), fields.end());
-
-  fields.resize( eit - fields.begin() );
+      std::set_intersection
+        ( old_f.begin(), old_f.end(), f.begin(), f.end(),
+          std::back_inserter(fields) );
+    }
 
   show_fields(fields);
 } // item_field_edit::enumerate_properties()
@@ -211,15 +438,20 @@ void bf::item_field_edit::get_fields_of
  * \brief Add the fields in the view of the properties.
  * \param fields The names of the fields to add.
  */
-void bf::item_field_edit::show_fields( const std::vector<std::string>& fields )
+void bf::item_field_edit::show_fields( const std::list<std::string>& fields )
 {
   wxString prefix;
   wxString last_prefix;
   std::size_t index = 0;
+  std::list<std::string>::const_iterator it;
+  int last_edited_index(wxNOT_FOUND);
 
-  for ( std::size_t i=0; i!=fields.size(); ++i )
+  for ( it=fields.begin(); it!=fields.end(); ++it )
     {
-      wxString f(std_to_wx_string(fields[i]));
+      if ( *it == m_last_edited_field )
+        last_edited_index = index;
+
+      wxString f(std_to_wx_string(*it));
       prefix = f.BeforeFirst(wxT('.'));
 
       if (prefix == f)
@@ -257,6 +489,12 @@ void bf::item_field_edit::show_fields( const std::vector<std::string>& fields )
 
   SetColumnWidth(0, wxLIST_AUTOSIZE);
   adjust_last_column_size();
+
+  if (last_edited_index != wxNOT_FOUND )
+    {
+      Select(last_edited_index);
+      EnsureVisible(last_edited_index);
+    }
 } // item_field_edit::show_fields()
 
 /*----------------------------------------------------------------------------*/
@@ -266,34 +504,37 @@ void bf::item_field_edit::show_fields( const std::vector<std::string>& fields )
 void bf::item_field_edit::update_values()
 {
   for ( long i=0; i!=GetItemCount(); ++i )
+    update_value(i);
+} // item_field_edit::update_values()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Update the displayed value of a field.
+ * \pararm index The index of the field in the view.
+ */
+void bf::item_field_edit::update_value( long index )
+{
+  std::string name;
+
+  if( get_field_name(index, name) )
     {
       wxListItem prop;
-      prop.SetId(i);
+      prop.SetId(index);
       GetItem(prop);
 
-      std::string name;
+      const type_field& f = get_common_field(name);
+      const bool has_val = group_has_value(f);
+      prop.SetText( get_common_value_as_text(f) );
 
-      if( get_field_name(i, name) )
-        {
-          const item_class& item(m_item->get_class());
-          const type_field& f = item.get_field(name);
+      prop.SetColumn(1);
+      SetItem(prop);
 
-          if ( m_item->has_value(f) )
-            prop.SetText( convert_value_to_text(f) );
-          else
-            prop.SetText
-              ( std_to_wx_string(item.get_default_value(f.get_name())) );
-
-          prop.SetColumn(1);
-          SetItem(prop);
-
-          if ( f.get_required() )
-            set_required_color( i, m_item->has_value(f) );
-          else
-            set_default_value_color(i, m_item->has_value(f));
-        }
+      if ( f.get_required() )
+        set_required_color(index, has_val);
+      else
+        set_default_value_color(index, has_val);
     }
-} // item_field_edit::update_values()
+} // item_field_edit::update_value()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -352,87 +593,19 @@ void bf::item_field_edit::set_default_value_color( unsigned int i, bool b )
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Convert the value of a field in a string.
+ * \param item The item in which the value is taken.
  * \param f The field for which we want the value.
  */
-wxString bf::item_field_edit::convert_value_to_text( const type_field& f ) const
+wxString bf::item_field_edit::convert_value_to_text
+( const item_instance& item, const type_field& f ) const
 {
-  wxString result;
-
-  if ( f.is_list() )
-    switch ( f.get_field_type() )
-      {
-      case type_field::integer_field_type:
-        result = convert_value_to_text< std::list<integer_type> >(f.get_name());
-        break;
-      case type_field::u_integer_field_type:
-        result =
-          convert_value_to_text< std::list<u_integer_type> >(f.get_name());
-        break;
-      case type_field::real_field_type:
-        result = convert_value_to_text< std::list<real_type> >(f.get_name());
-        break;
-      case type_field::boolean_field_type:
-        result = convert_value_to_text< std::list<bool_type> >(f.get_name());
-        break;
-      case type_field::string_field_type:
-        result = convert_value_to_text< std::list<string_type> >(f.get_name());
-        break;
-      case type_field::sprite_field_type:
-        result = convert_value_to_text< std::list<sprite> >(f.get_name());
-        break;
-      case type_field::animation_field_type:
-        result =
-          convert_value_to_text< std::list<animation_file_type> >(f.get_name());
-        break;
-      case type_field::item_reference_field_type:
-        result =
-          convert_value_to_text< std::list<item_reference_type> >(f.get_name());
-        break;
-      case type_field::font_field_type:
-        result =
-          convert_value_to_text< std::list<font_file_type> >(f.get_name());
-        break;
-      case type_field::sample_field_type:
-	result =
-          convert_value_to_text< std::list<sample_file_type> >(f.get_name());
-        break;
-      }
+  if ( !item.has_value(f) )
+    return std_to_wx_string( item.get_class().get_default_value(f.get_name()) );
   else
-    switch ( f.get_field_type() )
-      {
-      case type_field::integer_field_type:
-        result = convert_value_to_text<integer_type>(f.get_name());
-        break;
-      case type_field::u_integer_field_type:
-        result = convert_value_to_text<u_integer_type>(f.get_name());
-        break;
-      case type_field::real_field_type:
-        result = convert_value_to_text<real_type>(f.get_name());
-        break;
-      case type_field::boolean_field_type:
-        result = convert_value_to_text<bool_type>(f.get_name());
-        break;
-      case type_field::string_field_type:
-        result = convert_value_to_text<string_type>(f.get_name());
-        break;
-      case type_field::sprite_field_type:
-        result = convert_value_to_text<sprite>(f.get_name());
-        break;
-      case type_field::animation_field_type:
-        result = convert_value_to_text<animation_file_type>(f.get_name());
-        break;
-      case type_field::item_reference_field_type:
-        result = convert_value_to_text<item_reference_type>(f.get_name());
-        break;
-      case type_field::font_field_type:
-        result = convert_value_to_text<font_file_type>(f.get_name());
-        break;
-      case type_field::sample_field_type:
-        result = convert_value_to_text<sample_file_type>(f.get_name());
-        break;
-      }
-
-  return result;
+    {
+      call_by_field_type<value_to_text_converter, wxString> call;
+      return call(f, f.get_name(), item);
+    }
 } // item_field_edit::convert_value_to_text()
 
 /*----------------------------------------------------------------------------*/
@@ -459,8 +632,13 @@ void bf::item_field_edit::delete_selected_field()
 
       if ( get_field_name(index, name) )
         {
-          m_proxy.delete_field(*m_item, name);
-          update_values();
+          m_last_edited_field = name;
+          delete_item_field_event event
+            ( name, delete_item_field_event::delete_field_event_type, GetId() );
+          event.SetEventObject(this);
+
+          if ( ProcessEvent(event) )
+            update_value(index);
         }
     }
 } // item_field_edit::delete_selected_field()
@@ -472,40 +650,43 @@ void bf::item_field_edit::delete_selected_field()
  */
 void bf::item_field_edit::create_field_editor( const std::string& name )
 {
-  const item_class& item(m_item->get_class());
-  const type_field& f = item.get_field(name);
+  const type_field& f = get_common_field(name);
+  m_last_edited_field = name;
 
   switch ( f.get_field_type() )
     {
     case type_field::integer_field_type:
-      show_simple_property_dialog<integer_type>(f, _("integer"));
+      show_simple_property_dialog<integer_type>(f, _("Integer"));
       break;
     case type_field::u_integer_field_type:
-      show_simple_property_dialog<u_integer_type>(f, _("unsigned integer"));
+      show_simple_property_dialog<u_integer_type>(f, _("Unsigned integer"));
       break;
     case type_field::real_field_type:
-      show_simple_property_dialog<real_type>(f, _("real"));
+      show_simple_property_dialog<real_type>(f, _("Real number"));
       break;
     case type_field::boolean_field_type:
-      show_property_dialog<bool_edit>(f, _("boolean"));
+      if ( !f.is_list() )
+        toggle_boolean_field_value(f);
+      else
+        show_property_dialog<bool_edit>(f, _("Boolean value"));
       break;
     case type_field::string_field_type:
       show_string_property_dialog(f);
       break;
     case type_field::sprite_field_type:
-      show_property_dialog<sprite_edit>(f, _("sprite"));
+      show_property_dialog<sprite_edit>(f, _("Sprite"));
       break;
     case type_field::animation_field_type:
-      show_property_dialog<animation_file_edit>(f, _("animation"));
+      show_property_dialog<any_animation_edit>(f, _("Animation"));
       break;
     case type_field::item_reference_field_type:
       show_item_reference_property_dialog(f);
       break;
     case type_field::font_field_type:
-      show_property_dialog<font_file_edit>(f, _("font"));
+      show_property_dialog<font_file_edit>(f, _("Font"));
       break;
     case type_field::sample_field_type:
-      show_property_dialog<sample_file_edit>(f, _("sample"));
+      show_property_dialog<sample_edit>(f, _("Sound sample"));
       break;
     }
 } // item_field_edit::create_field_editor()
@@ -542,21 +723,112 @@ bf::item_field_edit::show_item_reference_property_dialog( const type_field& f )
 {
   wxArrayString values;
 
-  m_proxy.get_item_identifiers(values, f);
-  values.Sort();
+  request_item_id_event event
+    ( f, values, request_item_id_event::request_item_id_event_type, GetId() );
+  event.SetEventObject(this);
 
-  if ( f.is_list() )
-    edit_item_reference_field< std::list<item_reference_type> >(f, values);
-  else
-    edit_item_reference_field<item_reference_type>(f, values);
+  if ( ProcessEvent(event) )
+    {
+      values.Sort();
+
+      if ( f.is_list() )
+        edit_field< item_reference_edit, std::list<item_reference_type> >
+          (f, _("Item"), values);
+      else
+        edit_field<item_reference_edit, item_reference_type>
+          (f, _("List of items"), values);
+    }
 } // item_field_edit::show_item_reference_property_dialog()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Toggle the value of a boolean field.
+ * \param f The type of the field we are editing.
+ */
+void bf::item_field_edit::toggle_boolean_field_value( const type_field& f )
+{
+  CLAW_PRECOND(!f.is_list());
+  CLAW_PRECOND(f.get_field_type() == type_field::boolean_field_type);
+
+  bool_type v;
+  if ( !get_common_value(f, v) )
+    v.set_value(true);
+  else
+    v.set_value(!v.get_value());
+
+  set_field_value_event<bool_type> event
+    ( f.get_name(), v,
+      set_field_value_event<bool_type>::set_field_value_event_type, GetId() );
+  event.SetEventObject(this);
+
+  if( ProcessEvent(event) )
+    update_values();
+} // item_field_edit::toggle_boolean_field_value()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Fill the list of fields.
+ */
+void bf::item_field_edit::fill_fields()
+{
+  DeleteAllItems();
+
+  if ( empty() )
+    return;
+
+  enumerate_properties();
+  update_values();
+
+  if ( m_last_selected_field != wxNOT_FOUND )
+    if ( m_last_selected_field < GetItemCount() )
+      {
+        Select(m_last_selected_field);
+        EnsureVisible(m_last_selected_field);
+      }
+} // item_field_edit::fill_fields()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Tell if all items in the edited group has a value for a given field.
+ * \param f The field to test.
+ */
+bool bf::item_field_edit::group_has_value( const type_field& f ) const
+{
+  bool result( !empty() );
+
+  for (item_iterator it=begin(); result && (it!=end()); ++it)
+    result = it->has_value(f);
+
+  return result;
+} // item_field_edit::group_has_value()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the value common to all edited items for a given field, or an
+ *        empty string if two items have a different value for this field.
+ * \param f The field to test.
+ */
+wxString
+bf::item_field_edit::get_common_value_as_text( const type_field& f ) const
+{
+  CLAW_PRECOND(!empty());
+
+  item_iterator it=begin();
+  wxString result( convert_value_to_text(*it, f) );
+
+  for ( ++it; !result.empty() && (it!=end()); ++it )
+    if ( convert_value_to_text(*it, f) != result )
+      result.clear();
+
+  return result;
+} // item_field_edit::get_common_value_as_text()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Event sent to a resized window.
  * \param event The event.
  */
-void bf::item_field_edit::on_size(wxSizeEvent& event)
+void bf::item_field_edit::on_size( wxSizeEvent& event )
 {
   adjust_last_column_size();
   event.Skip();
@@ -567,7 +839,7 @@ void bf::item_field_edit::on_size(wxSizeEvent& event)
  * \brief Event sent when the user starts to resize a column.
  * \param event The event.
  */
-void bf::item_field_edit::on_column_begin_drag(wxListEvent& event)
+void bf::item_field_edit::on_column_begin_drag( wxListEvent& event )
 {
   if ( event.GetColumn() + 1 == GetColumnCount() )
     event.Veto();
@@ -580,7 +852,7 @@ void bf::item_field_edit::on_column_begin_drag(wxListEvent& event)
  * \brief Event sent when the user ends resizing a column.
  * \param event The event.
  */
-void bf::item_field_edit::on_column_end_drag(wxListEvent& event)
+void bf::item_field_edit::on_column_end_drag( wxListEvent& WXUNUSED(event) )
 {
   adjust_last_column_size();
 } // item_field_edit::on_column_begin_drag()
@@ -590,7 +862,7 @@ void bf::item_field_edit::on_column_end_drag(wxListEvent& event)
  * \brief Event sent when the user activates an item.
  * \param event The event.
  */
-void bf::item_field_edit::on_item_activated(wxListEvent& event)
+void bf::item_field_edit::on_item_activated( wxListEvent& event )
 {
   std::string name;
 
@@ -617,7 +889,7 @@ void bf::item_field_edit::on_item_activated(wxListEvent& event)
  * \brief Event sent when the user presses a key.
  * \param event The keyboard event that occured.
  */
-void bf::item_field_edit::on_key_up(wxKeyEvent& event)
+void bf::item_field_edit::on_key_up( wxKeyEvent& event )
 {
   switch( event.GetKeyCode() )
     {
