@@ -1,7 +1,7 @@
 /*
   Bear Engine - Editor library
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -28,12 +28,92 @@
  */
 #include "bf/item_instance.hpp"
 
-#include "bf/animation_file_xml_reader.hpp"
-#include "bf/path_configuration.hpp"
-#include "bf/wx_facilities.hpp"
+#include "bf/call_by_field_type.hpp"
+#include "bf/item_class.hpp"
 
 #include <claw/assert.hpp>
-#include <sstream>
+
+#define SPECIALISE_FIELD_TYPE( type, name )                             \
+  template<>                                                            \
+  struct item_instance::field_map_by_type<type>                         \
+  {                                                                     \
+    static std::map<std::string, type>& get( item_instance& item )      \
+    {                                                                   \
+      return item.m_ ## name;                                           \
+    }                                                                   \
+                                                                        \
+    static const std::map<std::string, type>&                           \
+    get( const item_instance& item )                                    \
+    {                                                                   \
+      return item.m_ ## name;                                           \
+    }                                                                   \
+  };                                                                    \
+                                                                        \
+  template<>                                                            \
+  struct item_instance::field_map_by_type< std::list<type> >            \
+  {                                                                     \
+    static std::map< std::string, std::list<type> >&                    \
+    get( item_instance& item )                                          \
+    {                                                                   \
+      return item.m_ ## name ## _list;                                  \
+    }                                                                   \
+                                                                        \
+    static const std::map< std::string, std::list<type> >&              \
+    get( const item_instance& item )                                    \
+    {                                                                   \
+      return item.m_ ## name ## _list;                                  \
+    }                                                                   \
+  }
+
+namespace bf
+{
+  SPECIALISE_FIELD_TYPE( integer_type, int );
+  SPECIALISE_FIELD_TYPE( u_integer_type, u_int );
+  SPECIALISE_FIELD_TYPE( real_type, real );
+  SPECIALISE_FIELD_TYPE( bool_type, bool );
+  SPECIALISE_FIELD_TYPE( string_type, string );
+  SPECIALISE_FIELD_TYPE( sprite, sprite );
+  SPECIALISE_FIELD_TYPE( any_animation, animation );
+  SPECIALISE_FIELD_TYPE( item_reference_type, item_reference );
+  SPECIALISE_FIELD_TYPE( font_file_type, font );
+  SPECIALISE_FIELD_TYPE( sample, sample );
+} // namespace bf
+
+
+
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Tell if a value has been given to a field.
+ * \param item The item instance in which the field is searched.
+ * \param name The name of the searched field.
+ */
+template<typename T>
+bool bf::item_instance::field_has_value<T>::operator()
+  ( const item_instance& item, const std::string& field_name ) const
+{
+  return field_map_by_type<T>::get(item).find(field_name)
+    != field_map_by_type<T>::get(item).end();
+} // item_instance::field_has_value::operator()()
+
+
+
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Erase the value of a field.
+ * \param item The item instance in which the field is erased.
+ * \param name The name of the erased field.
+ */
+template<typename T>
+void bf::item_instance::erase_field_value<T>::operator()
+  ( item_instance& item, const std::string& field_name ) const
+{
+  field_map_by_type<T>::get(item).erase(field_name);
+} // item_instance::erase_field_value::operator()()
+
+
+
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -160,6 +240,62 @@ void bf::item_instance::set_id( const std::string& id )
   m_id = id;
 } // item_instance::set_id()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Rename the identifiers in fields of type item_reference.
+ * \param map_id The map of idendifier's modification.
+ */
+void bf::item_instance::rename_item_reference_fields
+( const std::map<std::string, std::string>& map_id)
+{
+  std::list<std::string> fields;
+  std::list<std::string>::const_iterator it;
+
+  m_class->get_field_names_in_hierarchy(fields);
+
+  for ( it=fields.begin(); it!=fields.end(); ++it )
+    {
+      const type_field& f = m_class->get_field(*it);
+
+      if( f.get_field_type() == type_field::item_reference_field_type )
+        {
+          if( has_value(f) )
+            {
+              if( f.is_list() )
+                {
+                  std::list<item_reference_type>::iterator bit =
+                    m_item_reference_list.find(f.get_name())->second.begin();
+                  const std::list<item_reference_type>::iterator eit =
+                    m_item_reference_list.find(f.get_name())->second.end();
+
+                  for ( ; bit!=eit; ++bit)
+                    if ( map_id.find(bit->get_value()) != map_id.end() )
+                      bit->set_value(map_id.find(bit->get_value())->second);
+                }
+              else
+                {
+                  std::string id;
+                  id = m_item_reference.find(f.get_name())->second.get_value();
+
+                  if ( map_id.find(id) != map_id.end() )
+                    m_item_reference.find
+                      (f.get_name())->second.set_value(map_id.find(id)->second);
+                }
+            }
+        }
+    }
+} // item_instance::rename_item_reference_fields()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Tell if a field has a value.
+ * \param field_name The name of the field to check.
+ */
+bool bf::item_instance::has_value( const std::string& field_name ) const
+{
+  return has_value( get_class().get_field(field_name) );
+} // item_instance::has_value()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell if a field has a value.
@@ -167,79 +303,8 @@ void bf::item_instance::set_id( const std::string& id )
  */
 bool bf::item_instance::has_value( const type_field& f ) const
 {
-  bool result = false;
-
-  if ( f.is_list() )
-    switch ( f.get_field_type() )
-      {
-      case type_field::integer_field_type:
-        result = m_int_list.find(f.get_name()) != m_int_list.end();
-        break;
-      case type_field::u_integer_field_type:
-        result = m_u_int_list.find(f.get_name()) != m_u_int_list.end();
-        break;
-      case type_field::real_field_type:
-        result = m_real_list.find(f.get_name()) != m_real_list.end();
-        break;
-      case type_field::boolean_field_type:
-        result = m_bool_list.find(f.get_name()) != m_bool_list.end();
-        break;
-      case type_field::string_field_type:
-        result = m_string_list.find(f.get_name()) != m_string_list.end();
-        break;
-      case type_field::sprite_field_type:
-        result = m_sprite_list.find(f.get_name()) != m_sprite_list.end();
-        break;
-      case type_field::animation_field_type:
-        result = m_animation_list.find(f.get_name()) != m_animation_list.end();
-        break;
-      case type_field::item_reference_field_type:
-        result = ( m_item_reference_list.find(f.get_name())
-                   != m_item_reference_list.end() );
-        break;
-      case type_field::font_field_type:
-        result = m_font_list.find(f.get_name()) != m_font_list.end();
-        break;
-      case type_field::sample_field_type:
-        result = m_sample_list.find(f.get_name()) != m_sample_list.end();
-        break;
-      }
-  else
-    switch ( f.get_field_type() )
-      {
-      case type_field::integer_field_type:
-        result = m_int.find(f.get_name()) != m_int.end();
-        break;
-      case type_field::u_integer_field_type:
-        result = m_u_int.find(f.get_name()) != m_u_int.end();
-        break;
-      case type_field::real_field_type:
-        result = m_real.find(f.get_name()) != m_real.end();
-        break;
-      case type_field::boolean_field_type:
-        result = m_bool.find(f.get_name()) != m_bool.end();
-        break;
-      case type_field::string_field_type:
-        result = m_string.find(f.get_name()) != m_string.end();
-        break;
-      case type_field::sprite_field_type:
-        result = m_sprite.find(f.get_name()) != m_sprite.end();
-        break;
-      case type_field::animation_field_type:
-        result = m_animation.find(f.get_name()) != m_animation.end();
-        break;
-      case type_field::item_reference_field_type:
-        result = m_item_reference.find(f.get_name()) != m_item_reference.end();
-        break;
-      case type_field::font_field_type:
-        result = m_font.find(f.get_name()) != m_font.end();
-        break;
-      case type_field::sample_field_type:
-        result = m_sample.find(f.get_name()) != m_sample.end();
-        break;
-      }
-
-  return result;
+  call_by_field_type<field_has_value, bool> call;
+  return call(f, *this, f.get_name());
 } // item_instance::has_value()
 
 /*----------------------------------------------------------------------------*/
@@ -249,78 +314,8 @@ bool bf::item_instance::has_value( const type_field& f ) const
  */
 void bf::item_instance::delete_value( const type_field& f )
 {
-  if ( has_value(f) )
-    {
-      if ( f.is_list() )
-        switch ( f.get_field_type() )
-          {
-          case type_field::integer_field_type:
-            m_int_list.erase( f.get_name() );
-            break;
-          case type_field::u_integer_field_type:
-            m_u_int_list.erase( f.get_name() );
-            break;
-          case type_field::real_field_type:
-            m_real_list.erase( f.get_name() );
-            break;
-          case type_field::boolean_field_type:
-            m_bool_list.erase( f.get_name() );
-            break;
-          case type_field::string_field_type:
-            m_string_list.erase( f.get_name() );
-            break;
-          case type_field::sprite_field_type:
-            m_sprite_list.erase( f.get_name() );
-            break;
-          case type_field::animation_field_type:
-            m_animation_list.erase( f.get_name() );
-            break;
-          case type_field::item_reference_field_type:
-            m_item_reference_list.erase( f.get_name() );
-            break;
-	  case type_field::font_field_type:
-            m_font_list.erase( f.get_name() );
-            break;
-	  case type_field::sample_field_type:
-            m_sample_list.erase( f.get_name() );
-            break;
-          }
-      else
-        switch ( f.get_field_type() )
-          {
-          case type_field::integer_field_type:
-            m_int.erase( f.get_name() );
-            break;
-          case type_field::u_integer_field_type:
-            m_u_int.erase( f.get_name() );
-            break;
-          case type_field::real_field_type:
-            m_real.erase( f.get_name() );
-            break;
-          case type_field::boolean_field_type:
-            m_bool.erase( f.get_name() );
-            break;
-          case type_field::string_field_type:
-            m_string.erase( f.get_name() );
-            break;
-          case type_field::sprite_field_type:
-            m_sprite.erase( f.get_name() );
-            break;
-          case type_field::animation_field_type:
-            m_animation.erase( f.get_name() );
-            break;
-          case type_field::item_reference_field_type:
-            m_item_reference.erase( f.get_name() );
-            break;
-	  case type_field::font_field_type:
-            m_font.erase( f.get_name() );
-            break;
-	  case type_field::sample_field_type:
-            m_sample.erase( f.get_name() );
-            break;
-          }
-    }
-
+  call_by_field_type<erase_field_value, void> call;
+  call(f, *this, f.get_name());
   m_rendering_parameters.field_changed( f.get_name() );
 } // item_instance::delete_value()
 
@@ -366,21 +361,21 @@ void bf::item_instance::compile
  * \brief Test if the item has a good initialization.
  * Return "true" if there are no problem.
  * Test item references and required fields.
- * \param error_msg The string of error message.
  * \param map_id The map containing item identifiants.
  */
-bool bf::item_instance::check
-( std::string& error_msg, const std::map<std::string,item_instance*>& map_id)
+bool
+bf::item_instance::check( const std::map<std::string,item_instance*>& map_id)
 {
+  std::string error_msg;
   bool result = check_required_fields(error_msg);
-  
-  if ( result ) 
+
+  if ( result )
     result = check_mass_for_fixed_item(error_msg);
-  
-  if ( result ) 
+
+  if ( result )
     result = check_id_required(error_msg, map_id);
 
-  if ( result ) 
+  if ( result )
     {
       m_rendering_parameters.clear_error();
       m_rendering_parameters.set_is_valid(true);
@@ -389,7 +384,7 @@ bool bf::item_instance::check
     {
       m_rendering_parameters.set_error(error_msg);
       m_rendering_parameters.set_is_valid(false);
-    }  
+    }
 
   return result;
 } // item_instance::check()
@@ -404,16 +399,16 @@ bool bf::item_instance::check_mass_for_fixed_item(std::string& error_msg) const
 {
   bool result = true;
 
-  if ( m_fixed ) 
+  if ( m_fixed )
     {
       type_field field("base_item.mass",type_field::real_field_type);
       if ( has_value(field) )
-	{
-	  result = false;
-	  error_msg = "Error item " + m_class->get_class_name() +
-	    " : for fixed item, the field 'base_item.mass' must be" +
-	    " equal to infinity";
-	}
+        {
+          result = false;
+          error_msg = "Error in item of class '" + m_class->get_class_name() +
+            "': for fixed item, the field 'base_item.mass' must be" +
+            " equal to infinity";
+        }
     }
 
   return result;
@@ -427,7 +422,7 @@ bool bf::item_instance::check_mass_for_fixed_item(std::string& error_msg) const
  * \param map_id The map containing item identifiants.
  */
 bool bf::item_instance::check_id_required
-(std::string& error_msg, 
+(std::string& error_msg,
  const std::map<std::string,item_instance*>& map_id) const
 {
   bool result = true;
@@ -442,40 +437,42 @@ bool bf::item_instance::check_id_required
       const type_field& f = m_class->get_field(*it);
       if( f.get_field_type() == type_field::item_reference_field_type )
         {
-	  if( has_value(f) )
-	    {
-	      if( f.is_list() )
-		{
-		  std::list<item_reference_type>::const_iterator bit =
-		    m_item_reference_list.find(f.get_name())->second.begin();
-		  const std::list<item_reference_type>::const_iterator eit =
-		    m_item_reference_list.find(f.get_name())->second.end();
-		  
-		  for ( ; bit!=eit; ++bit)
-		    if ( map_id.find(bit->get_value()) == map_id.end() )
-		      {
-			result = false;
-			error_msg = "Error item " + m_class->get_class_name() +
-			  " : field '" + *it + 
-			  "'. The identifier '" + bit->get_value() 
-			  + "' does not exist";
-		      }	     
-		}
-	      else
-		{
-		  std::string id;
-		  id = m_item_reference.find(f.get_name())->second.get_value();
-		  
-		  if ( map_id.find(id) == map_id.end() ) 
-		    {
-		      result = false;
-		      error_msg = "Error item " + m_class->get_class_name() +
-			" : field '" + *it + 
-			"'. The identifier '" + id + "' does not exist";
-		    }
-		}
-	    }
-	}
+          if( has_value(f) )
+            {
+              if( f.is_list() )
+                {
+                  std::list<item_reference_type>::const_iterator bit =
+                    m_item_reference_list.find(f.get_name())->second.begin();
+                  const std::list<item_reference_type>::const_iterator eit =
+                    m_item_reference_list.find(f.get_name())->second.end();
+
+                  for ( ; bit!=eit; ++bit)
+                    if ( map_id.find(bit->get_value()) == map_id.end() )
+                      {
+                        result = false;
+                        error_msg = "Error item of class '"
+                          + m_class->get_class_name() +
+                          "': field '" + *it +
+                          "'. The identifier '" + bit->get_value()
+                          + "' does not exist";
+                      }
+                }
+              else
+                {
+                  std::string id;
+                  id = m_item_reference.find(f.get_name())->second.get_value();
+
+                  if ( map_id.find(id) == map_id.end() )
+                    {
+                      result = false;
+                      error_msg = "Error in item of class '"
+                        + m_class->get_class_name() +
+                        "': field '" + *it +
+                        "'. The identifier '" + id + "' does not exist";
+                    }
+                }
+            }
+        }
     }
 
   return result;
@@ -503,8 +500,9 @@ bool bf::item_instance::check_required_fields(std::string& error_msg) const
         if( !has_value(f) )
           {
             result = false;
-            error_msg = "Error item " + m_class->get_class_name() +
-              " : field '" + *it + "' is required";
+            error_msg = "Error in item of class '"
+              + m_class->get_class_name() +
+              "': field '" + *it + "' is required";
           }
     }
 
@@ -752,11 +750,11 @@ bf::item_instance::get_code_value( const type_field& field ) const
       result = bear::level_code_value::field_string;
       break;
     case type_field::sample_field_type:
-      result = bear::level_code_value::field_string;
+      result = bear::level_code_value::field_sample;
       break;
     default:
       {
-        CLAW_ASSERT( false, "Invalid field type." );
+        CLAW_FAIL( "Invalid field type." );
       }
     }
 
@@ -835,3 +833,83 @@ void bf::item_instance::remove_invalid_values()
   remove_invalid_values
     ( m_sample, m_sample_list, type_field::sample_field_type );
 } // item_instance::set_class()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Extract the names of defined fields.
+ * \param m The map in which we take the fields names.
+ * \param fields The set in which the fields are saved.
+ */
+template<typename MapType>
+void bf::item_instance::copy_field_names
+( const MapType& m, std::set<std::string>& fields ) const
+{
+  typename MapType::const_iterator it;
+
+  for (it=m.begin(); it!=m.end(); ++it)
+    fields.insert(it->first);
+} // item_instance::copy_field_names()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Compile a list of values.
+ * \param f The file in which we compile.
+ * \param v The values to save.
+ */
+template<typename T>
+void bf::item_instance::compile_list
+( compiled_file& f, const std::list<T>& v ) const
+{
+  f << v.size();
+
+  typename std::list<T>::const_iterator it = v.begin();
+  const typename std::list<T>::const_iterator eit = v.end();
+
+  for ( ; it!=eit; ++it)
+    it->compile(f);
+} // item_instance::compile_list()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Remove the field values that are not available for the class of this
+ *        item.
+ * \param m The map of values.
+ * \param ml The map of list of values.
+ * \param t The type of the fields explored.
+ */
+template<typename T>
+void bf::item_instance::remove_invalid_values
+( std::map<std::string, T>& m, std::map< std::string, std::list<T> >& ml,
+  type_field::field_type t ) const
+{
+  std::list<std::string> to_remove;
+  typename std::map<std::string, T>::const_iterator it;
+
+  for (it=m.begin(); it!=m.end(); ++it)
+    if ( !m_class->has_field( it->first, t ) )
+      to_remove.push_front(it->first);
+    else
+      {
+        const type_field& f = m_class->get_field(it->first);
+        if ( f.is_list() )
+          to_remove.push_front(it->first);
+      }
+
+  for ( ; !to_remove.empty(); to_remove.pop_front() )
+    m.erase( to_remove.front() );
+
+  typename std::map< std::string, std::list<T> >::const_iterator itl;
+
+  for (itl=ml.begin(); itl!=ml.end(); ++itl)
+    if ( !m_class->has_field( itl->first, t ) )
+      to_remove.push_front(itl->first);
+    else
+      {
+        const type_field& f = m_class->get_field(itl->first);
+        if ( !f.is_list() )
+          to_remove.push_front(itl->first);
+      }
+
+  for ( ; !to_remove.empty(); to_remove.pop_front() )
+    ml.erase( to_remove.front() );
+} // item_instance::remove_invalid_values()
