@@ -1,7 +1,7 @@
 /*
   Bear Engine - Editor library
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -28,11 +28,9 @@
  */
 #include "bf/item_rendering_parameters.hpp"
 
-#include "bf/animation_file_xml_reader.hpp"
+#include "bf/item_class.hpp"
 #include "bf/item_instance.hpp"
-#include "bf/path_configuration.hpp"
 #include "bf/stream_conv.hpp"
-#include "bf/wx_facilities.hpp"
 
 #include <claw/assert.hpp>
 #include <sstream>
@@ -111,7 +109,7 @@ bf::item_rendering_parameters::field_changed( const std::string& field_name )
     m_width = get_field_real( field_name, 0 );
   else if (field_name == s_field_name_height)
     m_height = get_field_real( field_name, 0 );
-  else 
+  else
     {
       const item_class& c(m_item.get_class());
 
@@ -155,6 +153,15 @@ double bf::item_rendering_parameters::get_left() const
   return m_left;
 } // item_rendering_parameters::get_left()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the x-position of the right side of the item, if any.
+ */
+double bf::item_rendering_parameters::get_right() const
+{
+  return get_left() + get_width();
+} // item_rendering_parameters::get_right()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the y-position of the item, if any.
@@ -164,6 +171,33 @@ double bf::item_rendering_parameters::get_bottom() const
   return m_bottom;
 } // item_rendering_parameters::get_bottom()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the y-position of the top of the item, if any.
+ */
+double bf::item_rendering_parameters::get_top() const
+{
+  return get_bottom() + get_height();
+} // item_rendering_parameters::get_top()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the x-position of the middle of the item.
+ */
+double bf::item_rendering_parameters::get_horizontal_middle() const
+{
+  return get_left() + get_width() / 2;
+} // item_rendering_parameters::get_horizontal_middle()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the y-position of the middle of the item.
+ */
+double bf::item_rendering_parameters::get_vertical_middle() const
+{
+  return get_bottom() + get_height() / 2;
+} // item_rendering_parameters::get_vertical_middle()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get the z-position of the item, if any.
@@ -375,10 +409,25 @@ bf::sprite bf::item_rendering_parameters::get_sprite_from_sprites() const
     if ( m_item.has_value( my_class.get_field(default_name) ) )
       m_item.get_value( default_name, result );
 
-  item_class::field_iterator it;
+  if ( result.get_image_name().empty() )
+    result = search_sprite_in_class(my_class);
 
-  for (it=my_class.field_begin();
-       result.get_image_name().empty() && (it!=my_class.field_end()); ++it)
+  return result;
+} // item_rendering_parameters::get_sprite_from_sprites()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get a sprite in the sprites of the item.
+ * \param c The class in which we search the sprites.
+ */
+bf::sprite bf::item_rendering_parameters::search_sprite_in_class
+( const item_class& c ) const
+{
+  item_class::field_iterator it;
+  sprite result;
+
+  for (it=c.field_begin();
+       result.get_image_name().empty() && (it!=c.field_end()); ++it)
     if ( it->get_field_type() == type_field::sprite_field_type )
       if ( m_item.has_value(*it) )
         {
@@ -394,8 +443,18 @@ bf::sprite bf::item_rendering_parameters::get_sprite_from_sprites() const
             m_item.get_value( it->get_name(), result );
         }
 
+  if ( result.get_image_name().empty() )
+    {
+      item_class::const_super_class_iterator itc;
+
+      for ( itc=c.super_class_begin();
+            result.get_image_name().empty() && (itc!=c.super_class_end());
+            ++itc)
+        result = search_sprite_in_class( *itc );
+    }
+
   return result;
-} // item_rendering_parameters::get_sprite_from_sprites()
+} // item_rendering_parameters::search_sprite_in_class()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -406,66 +465,66 @@ bf::sprite bf::item_rendering_parameters::get_sprite_from_animations() const
   const item_class& my_class(m_item.get_class());
   const std::string default_name("item_with_decoration.animation");
   animation result;
-  animation_file_type f;
+  any_animation f;
 
   if ( my_class.has_field( default_name, type_field::animation_field_type ) )
     if ( m_item.has_value( my_class.get_field(default_name) ) )
       {
         m_item.get_value( default_name, f );
-        result = load_animation( f.get_value() );
+        result = f.get_current_animation();
       }
 
-  item_class::field_iterator it;
+  if ( result.empty() )
+    result = search_animation_in_class( my_class );
 
-  for (it=my_class.field_begin();
-       result.empty() && (it!=my_class.field_end()); ++it)
+  if ( result.empty() )
+    return sprite();
+  else
+    return result.get_sprite(0);
+} // item_rendering_parameters::get_sprite_from_animations()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get a sprite in the animations of the item.
+ * \param c The class in which we search the animations.
+ */
+bf::animation bf::item_rendering_parameters::search_animation_in_class
+( const item_class& c ) const
+{
+  item_class::field_iterator it;
+  animation result;
+
+  for (it=c.field_begin(); result.empty() && (it!=c.field_end()); ++it)
     if ( it->get_field_type() == type_field::animation_field_type )
       if ( m_item.has_value(*it) )
         {
           if ( it->is_list() )
             {
-              std::list<animation_file_type> v;
+              std::list<any_animation> v;
               m_item.get_value( it->get_name(), v );
               for( ; !v.empty() && result.empty(); v.pop_front() )
-                result = load_animation( v.front().get_value() );
+                result = v.front().get_current_animation();
             }
           else
             {
+              any_animation f;
               m_item.get_value( it->get_name(), f );
-              result = load_animation( f.get_value() );
+              result = f.get_current_animation();
             }
         }
 
   if ( result.empty() )
-    return sprite();
-  else
-    return result.get_frame(0).get_sprite();
-} // item_rendering_parameters::get_sprite_from_animations()
-
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Get an animation from a file.
- * \param path Path to the compiled animation.
- */
-bf::animation
-bf::item_rendering_parameters::load_animation( const std::string& path ) const
-{
-  std::string::size_type pos = path.rfind(".canim");
-  animation result;
-
-  if ( pos != std::string::npos )
     {
-      std::string p( path.substr(0, pos) + ".anim" );
+      item_class::const_super_class_iterator itc;
 
-      if ( path_configuration::get_instance().expand_file_name(p, 1) )
-        {
-          animation_file_xml_reader reader;
-          result = reader.load( std_to_wx_string(p) );
-        }
+      for ( itc=c.super_class_begin();
+            result.empty() && (itc!=c.super_class_end());
+            ++itc )
+        result = search_animation_in_class( *itc );
     }
 
   return result;
-} // item_rendering_parameters::load_animation()
+} // item_rendering_parameters::search_animation_in_class()
 
 /*----------------------------------------------------------------------------*/
 /**
