@@ -1,7 +1,7 @@
 /*
   Bear Engine - Model editor
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -29,10 +29,11 @@
 #include "bf/slider_ctrl.hpp"
 
 #include <wx/dcbuffer.h>
+#include <wx/settings.h>
 
 const unsigned int bf::slider_ctrl::s_slider_border = 8;
 
-const wxEventType bf::slider_event::value_change_event_type = 
+const wxEventType bf::slider_event::value_change_event_type =
   wxNewEventType();
 
 const wxEventType bf::tick_event::move_event_type = wxNewEventType();
@@ -112,13 +113,15 @@ bf::tick_event::tick_event( wxEventType t, wxWindowID id )
  * \brief Constructor.
  * \param initial_value The initial value of the tick.
  * \param new_value The new value of the tick.
+ * \param copy Tell if the user asks to copy the data associated with the
+ *        previous position (?).
  * \param t The type of the event.
  * \param id The id of the window that generates the event.
  */
 bf::tick_event::tick_event
-( double initial_value, double new_value, bool copy, 
+( double initial_value, double new_value, bool copy,
   wxEventType t, wxWindowID id )
-  : wxNotifyEvent(t, id), 
+  : wxNotifyEvent(t, id),
     m_initial_value(initial_value), m_new_value(new_value), m_copy(copy)
 {
 
@@ -132,7 +135,7 @@ bf::tick_event::tick_event
 bf::tick_event::tick_event( const tick_event& that )
   : wxNotifyEvent(that), m_initial_value(that.m_initial_value),
     m_new_value(that.m_new_value), m_copy(that.m_copy)
-{ 
+{
 } // tick_event::tick_event()
 
 /*----------------------------------------------------------------------------*/
@@ -204,7 +207,7 @@ wxSize bf::slider_ctrl::drag_info::delta() const
  * \param max_value The maximum value of the slider.
  */
 bf::slider_ctrl::slider_ctrl
-( wxWindow* parent, wxWindowID id, double value, double min_value, 
+( wxWindow* parent, wxWindowID id, double value, double min_value,
   double max_value )
   : super( parent, id ), m_drag_info(NULL),
     m_value(value), m_min_value(min_value), m_max_value(max_value),
@@ -262,9 +265,9 @@ double bf::slider_ctrl::get_value() const
  */
 void bf::slider_ctrl::set_value( double value )
 {
-  if ( m_value != value) 
+  if ( m_value != value)
     {
-      m_value = value;      
+      m_value = value;
       render();
     }
 } // slider_ctrl::set_value()
@@ -281,7 +284,7 @@ void bf::slider_ctrl::set_max_value( double max )
   if ( m_value > m_max_value )
     m_value = m_max_value;
 
-  render(); 
+  render();
 } // slider_ctrl::set_value()
 
 /*----------------------------------------------------------------------------*/
@@ -292,12 +295,12 @@ void bf::slider_ctrl::set_max_value( double max )
 bool bf::slider_ctrl::has_tick( double value ) const
 {
   bool result = false;
-  
+
   ticks::const_iterator it;
-  for ( it = m_ticks->begin(); ( it != m_ticks->end() ) && ! result; ++it) 
-    if ( *it == value ) 
+  for ( it = m_ticks->begin(); ( it != m_ticks->end() ) && !result; ++it)
+    if ( *it == value )
       result = true;
-  
+
   return result;
 } // slider_ctrl::has_tick()
 
@@ -305,7 +308,7 @@ bool bf::slider_ctrl::has_tick( double value ) const
 /**
  * \brief Send the event "value_change".
  */
-void bf::slider_ctrl::send_event_change_value() 
+void bf::slider_ctrl::send_event_change_value()
 {
   slider_event event
     ( m_value, slider_event::value_change_event_type, GetId() );
@@ -321,14 +324,14 @@ void bf::slider_ctrl::send_event_change_value()
  * \param copy Indicates if the move is a copy.
  */
 void bf::slider_ctrl::send_event_move_tick
-(double initial_value, double new_value, bool copy) 
+(double initial_value, double new_value, bool copy)
 {
   tick_event event
     ( initial_value, new_value, copy, tick_event::move_event_type, GetId() );
   event.SetEventObject(this);
   ProcessEvent(event);
 
-  if ( ! event.IsAllowed() ) 
+  if ( !event.IsAllowed() )
     {
       set_value(m_drag_info->initial_value);
       send_event_change_value();
@@ -359,7 +362,7 @@ void bf::slider_ctrl::render_slider( wxDC& dc ) const
   dc.DrawLine( s_slider_border, GetSize().y / 2 - 2,
                GetSize().x - s_slider_border, GetSize().y / 2 - 2 );
 
-  dc.SetPen(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNSHADOW));  
+  dc.SetPen(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNSHADOW));
   dc.SetBrush(*wxTRANSPARENT_BRUSH);
   dc.DrawRoundedRectangle( 1, 1, GetSize().x - 2, GetSize().y - 2, 3 );
 } // slider_ctrl::render_slider()
@@ -372,7 +375,7 @@ void bf::slider_ctrl::render_slider( wxDC& dc ) const
 void bf::slider_ctrl::render_value( wxDC& dc ) const
 {
   wxPoint pos(get_position(m_value),GetSize().y / 2);
-    
+
   dc.SetPen(wxSystemSettings::GetColour(wxSYS_COLOUR_3DSHADOW));
   dc.SetBrush(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
   wxPoint p3[5];
@@ -387,7 +390,7 @@ void bf::slider_ctrl::render_value( wxDC& dc ) const
   wxPoint p[4];
   p[0] = wxPoint( pos.x, pos.y + 3 );
   p[1] = wxPoint( pos.x - 3, pos.y );
-  p[2] = wxPoint( pos.x - 3, pos.y - 8 );  
+  p[2] = wxPoint( pos.x - 3, pos.y - 8 );
   p[3] = wxPoint( pos.x + 3, pos.y - 8 );
   dc.DrawLines(4, p);
 } // slider_ctrl::render_value()
@@ -403,8 +406,8 @@ void bf::slider_ctrl::render_drag_info( wxDC& dc ) const
     if ( m_drag_info->drag_mode == drag_info::drag_mode_tick_move )
       {
         double v = get_value(m_drag_info->mouse_position.x);
-        
-        if ( has_tick(v) ) 
+
+        if ( has_tick(v) )
           {
             render_tick( dc, get_position(m_drag_info->initial_value),
                          *wxRED, *wxTRANSPARENT_BRUSH);
@@ -429,7 +432,7 @@ void bf::slider_ctrl::render_ticks( wxDC& dc ) const
   if ( m_ticks )
     {
       ticks::const_iterator it;
-      
+
       for (it = m_ticks->begin(); it != m_ticks->end(); ++it )
         render_tick(dc,*it);
     }
@@ -443,7 +446,7 @@ void bf::slider_ctrl::render_ticks( wxDC& dc ) const
  */
 void bf::slider_ctrl::render_tick( wxDC& dc, tick t ) const
 {
-  if ( t == m_value ) 
+  if ( t == m_value )
     {
       wxBrush brush(*wxBLACK);
       render_tick( dc, get_position(t), *wxBLACK, brush);
@@ -461,7 +464,7 @@ void bf::slider_ctrl::render_tick( wxDC& dc, tick t ) const
  * \param brush The brush.
  */
 void bf::slider_ctrl::render_tick
-( wxDC& dc, unsigned int position, const wxColour& colour, 
+( wxDC& dc, unsigned int position, const wxColour& colour,
   const wxBrush& brush ) const
 {
   wxPen pen(colour);
@@ -470,7 +473,7 @@ void bf::slider_ctrl::render_tick
 
   wxPoint p[3];
   wxPoint pos(position,GetSize().y / 2);
-  
+
   p[0] = wxPoint( pos.x, pos.y+5 );
   p[1] = wxPoint( pos.x+4, pos.y +9 );
   p[2] = wxPoint( pos.x-4, pos.y +9 );
@@ -487,27 +490,27 @@ double bf::slider_ctrl::get_value( int pos ) const
 {
   double result;
 
-  if ( pos < (int)(s_slider_border)) 
+  if ( pos < (int)(s_slider_border))
     result = 0;
-  else if ( pos > (int)(s_slider_border + get_slider_width()) ) 
+  else if ( pos > (int)(s_slider_border + get_slider_width()) )
     result = m_max_value;
   else
     {
-      pos = pos - s_slider_border;
-      
-      result = 
-        ( (double)pos * ( m_max_value - m_min_value ) ) / 
+      pos -= s_slider_border;
+
+      result =
+        ( (double)pos * ( m_max_value - m_min_value ) ) /
         (double)get_slider_width() + m_min_value;
-      
+
       if ( result < m_min_value )
         result = m_min_value;
-      
+
       if ( result > m_max_value )
         result = m_max_value;
     }
 
   result = (double)((int)((result+0.0005) * 1000)) / 1000;
-  
+
   return result;
 } // slider_ctrl::get_value()
 
@@ -519,7 +522,7 @@ double bf::slider_ctrl::get_value( int pos ) const
 unsigned int bf::slider_ctrl::get_position( double value ) const
 {
   return  s_slider_border + (unsigned int)
-    ( ( ( value - m_min_value ) * (double)(get_slider_width() ) ) / 
+    ( ( ( value - m_min_value ) * (double)(get_slider_width() ) ) /
       ( m_max_value - m_min_value ) );
 } // slider_ctrl::get_position()
 
@@ -533,18 +536,18 @@ double bf::slider_ctrl::nearest_tick( double v ) const
   double result = v;
   double min_dist = m_max_value + 1;
 
-  if ( m_ticks ) 
+  if ( m_ticks )
     {
       slider_ctrl::ticks::const_iterator it;
-      
+
       for (it = m_ticks->begin(); it != m_ticks->end(); ++it )
         {
           double dist;
-          if ( *it > v ) 
+          if ( *it > v )
             dist = *it - v;
           else
             dist = v - *it;
-          
+
           if ( dist < min_dist )
             {
               result = *it;
@@ -552,7 +555,7 @@ double bf::slider_ctrl::nearest_tick( double v ) const
             }
         }
     }
-  
+
   return result;
 } // slider_ctrl::nearest_tick()
 
@@ -600,7 +603,7 @@ void bf::slider_ctrl::apply_drag_mode_tick_move( bool control, bool alt )
  * \brief Event sent to a resized window.
  * \param event The event.
  */
-void bf::slider_ctrl::on_size(wxSizeEvent& event)
+void bf::slider_ctrl::on_size( wxSizeEvent& WXUNUSED(event) )
 {
   Refresh();
 } // slider_ctrl::on_size()
@@ -610,7 +613,7 @@ void bf::slider_ctrl::on_size(wxSizeEvent& event)
  * \brief Draw the content of the window.
  * \param event The paint event.
  */
-void bf::slider_ctrl::on_paint(wxPaintEvent& event)
+void bf::slider_ctrl::on_paint( wxPaintEvent& WXUNUSED(event) )
 {
   render();
 } // slider_ctrl::on_paint()
@@ -620,19 +623,19 @@ void bf::slider_ctrl::on_paint(wxPaintEvent& event)
  * \brief The user start a click in the frame.
  * \param event The mouse event that occured.
  */
-void bf::slider_ctrl::on_mouse_left_down(wxMouseEvent& event)
+void bf::slider_ctrl::on_mouse_left_down( wxMouseEvent& event )
 {
   wxPoint point( event.GetX(), event.GetY() );
 
-  if ( event.ControlDown() ) 
+  if ( event.ControlDown() )
     {
       double v = get_value(point.x);
       v = nearest_tick(v);
-      
+
       if ( has_tick(v) )
         {
           m_drag_info = new drag_info();
-          m_drag_info->drag_mode = drag_info::drag_mode_tick_move;          
+          m_drag_info->drag_mode = drag_info::drag_mode_tick_move;
           m_drag_info->initial_value = v;
           m_drag_info->mouse_origin = point;
           m_drag_info->mouse_position = m_drag_info->mouse_origin;
@@ -643,7 +646,7 @@ void bf::slider_ctrl::on_mouse_left_down(wxMouseEvent& event)
       m_drag_info = new drag_info();
       m_drag_info->drag_mode = drag_info::drag_mode_move;
       m_drag_info->mouse_origin = point;
-      m_drag_info->mouse_position = m_drag_info->mouse_origin;      
+      m_drag_info->mouse_position = m_drag_info->mouse_origin;
     }
 } // slider_ctrl::on_mouse_left_down()
 
@@ -652,22 +655,22 @@ void bf::slider_ctrl::on_mouse_left_down(wxMouseEvent& event)
  * \brief The user clicked in the frame.
  * \param event The mouse event that occured.
  */
-void bf::slider_ctrl::on_mouse_move(wxMouseEvent& event)
+void bf::slider_ctrl::on_mouse_move( wxMouseEvent& event )
 {
   const wxPoint point( event.GetX(),event.GetY() );
-  
-  if ( event.LeftIsDown() )    
+
+  if ( event.LeftIsDown() )
     if ( m_drag_info != NULL )
       {
         m_drag_info->mouse_position = point;
         double v = get_value(point.x);
-        
-        if ( m_drag_info->drag_mode == drag_info::drag_mode_move ) 
+
+        if ( m_drag_info->drag_mode == drag_info::drag_mode_move )
           {
             if ( event.ShiftDown() )
               v = nearest_tick(v);
           }
-        
+
         if ( v != m_value )
           {
             set_value(v);
@@ -681,26 +684,26 @@ void bf::slider_ctrl::on_mouse_move(wxMouseEvent& event)
  * \brief The user click in the frame.
  * \param event The mouse event that occured.
  */
-void bf::slider_ctrl::on_mouse_left_up(wxMouseEvent& event)
+void bf::slider_ctrl::on_mouse_left_up( wxMouseEvent& event )
 {
   const wxPoint point( event.GetX(),event.GetY() );
-  
+
   if ( m_drag_info == NULL )
     event.Skip();
   else
     {
       m_drag_info->mouse_position = point;
-      if ( m_drag_info->drag_mode == drag_info::drag_mode_tick_move ) 
+      if ( m_drag_info->drag_mode == drag_info::drag_mode_tick_move )
         {
           apply_drag_mode_tick_move
-	    ( event.ControlDown(), event.AltDown() );
+            ( event.ControlDown(), event.AltDown() );
           delete m_drag_info;
           m_drag_info = NULL;
           render();
         }
-      else if ( m_drag_info->drag_mode == drag_info::drag_mode_move ) 
+      else if ( m_drag_info->drag_mode == drag_info::drag_mode_move )
         apply_drag_mode_move( event.ShiftDown() );
-      
+
       if ( m_drag_info != NULL )
         {
           m_drag_info = NULL;
@@ -714,7 +717,7 @@ void bf::slider_ctrl::on_mouse_left_up(wxMouseEvent& event)
  * \brief The user double click in the frame.
  * \param event The mouse event that occured.
  */
-void bf::slider_ctrl::on_mouse_left_dclick(wxMouseEvent& event)
+void bf::slider_ctrl::on_mouse_left_dclick( wxMouseEvent& event )
 {
   double v = get_value(event.GetX());
 
