@@ -1,7 +1,7 @@
 /*
   Bear Engine - Level editor
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -34,8 +34,10 @@
 #include "bf/item_class_pool.hpp"
 #include "bf/item_choice_frame.hpp"
 #include "bf/properties_frame.hpp"
+#include "bf/layer.hpp"
 #include "bf/layer_list_frame.hpp"
 #include "bf/level_file_xml_writer.hpp"
+#include "bf/windows_layout.hpp"
 #include "bf/wx_facilities.hpp"
 
 #include "bf/history/action_add_item.hpp"
@@ -50,6 +52,8 @@
 #include <wx/dcbuffer.h>
 #include <wx/clipbrd.h>
 
+#include <limits>
+
 /*----------------------------------------------------------------------------*/
 const wxCoord bf::ingame_view::s_point_size = 10;
 const wxCoord bf::ingame_view::s_grip_size = 10;
@@ -240,13 +244,13 @@ void bf::ingame_view::set_zoom(unsigned int z)
  */
 void bf::ingame_view::set_zoom(unsigned int z, wxPoint mouse_position)
 {
-  // Go in the center of mouse position and current position 
+  // Go in the center of mouse position and current position
   wxPoint c = unzoom(m_view + GetSize() / 2);
   c.x = ( c.x + mouse_position.x ) / 2;
   c.y = ( c.y + mouse_position.y ) / 2;
 
   m_zoom = z;
-  set_view_position( zoom(c.x) - GetSize().x/2, 
+  set_view_position( zoom(c.x) - GetSize().x/2,
                      zoom(c.y) - GetSize().y/2);
 } // ingame_view::set_zoom()
 
@@ -483,13 +487,18 @@ void bf::ingame_view::set_grid_on_selection()
 void bf::ingame_view::do_action( level_action* action )
 {
   CLAW_PRECOND( action != NULL );
+  apply_action(action);
+} // ingame_view::do_action()
 
-  if ( m_history.do_action(action) )
-    {
-      m_parent.set_changed( true );
-      m_parent.set_layer_info();
-      update_layout();
-    }
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Do the action of moving the current seletion.
+ * \param action The action to do.
+ */
+void bf::ingame_view::do_action( action_move_selection* action )
+{
+  CLAW_PRECOND( action != NULL );
+  apply_action(action);
 } // ingame_view::do_action()
 
 /*----------------------------------------------------------------------------*/
@@ -590,12 +599,16 @@ const bf::level_history& bf::ingame_view::get_history() const
 void bf::ingame_view::update_layout()
 {
   m_layout.get_layer_list_frame().set_level_view(this);
+  m_layout.get_properties_frame().clear();
 
   if ( has_selection() )
-    m_layout.get_properties_frame().set_item
-      ( get_level().get_main_selection() );
-  else
-    m_layout.get_properties_frame().set_item( NULL );
+    {
+      item_selection::const_iterator it;
+
+      for ( it=get_level().get_selection().begin();
+            it!=get_level().get_selection().end(); ++it )
+        m_layout.get_properties_frame().add_item(*it);
+    }
 
   Refresh();
 } // ingame_view::update_layout()
@@ -619,7 +632,8 @@ bool bf::ingame_view::has_selection() const
 void bf::ingame_view::clear_selection()
 {
   get_level().clear_selection();
-  update_layout();
+  m_layout.get_properties_frame().clear();
+  Refresh();
 } // ingame_view::clear_selection()
 
 /*----------------------------------------------------------------------------*/
@@ -637,7 +651,7 @@ void bf::ingame_view::select_all()
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Select an item, its layer and center view on the item.
- * \param items The item to select.
+ * \param item The item to select.
  * \param index The index of layer.
  */
 void bf::ingame_view::select_item_and_layer
@@ -654,7 +668,7 @@ void bf::ingame_view::select_item_and_layer
   wxCoord x = zoom((wxCoord)item->get_rendering_parameters().get_left() +
                    (wxCoord)item->get_rendering_parameters().get_width()/2)
     - GetSize().x/2;
-  
+
   wxCoord y = zoom((wxCoord)item->get_rendering_parameters().get_bottom()+
                    (wxCoord)item->get_rendering_parameters().get_height()/2)
     - GetSize().y/2;
@@ -713,7 +727,7 @@ void bf::ingame_view::paste_from_clipboard()
 {
   do_action
     ( new action_paste_from_clipboard
-      ( get_level(), s_clipboard, unzoom(m_view.x + GetSize().x / 2), 
+      ( get_level(), s_clipboard, unzoom(m_view.x + GetSize().x / 2),
         unzoom(m_view.y + GetSize().y / 2) ) );
 } // ingame_view::paste_from_clipboard()
 
@@ -869,9 +883,9 @@ void bf::ingame_view::render_items
       render_item_wireframe(dc, *wireframe_forced.front(), i );
 
   for (it=z_order.begin(); it!=z_order.end(); ++it)
-    if ( ! it->second->get_rendering_parameters().is_valid() )
+    if ( !it->second->get_rendering_parameters().is_valid() )
       render_non_valid_item(dc, *it->second);
- 
+
   if ( m_display_id )
     for (it=z_order.begin(); it!=z_order.end(); ++it)
       render_item_id(dc, *it->second);
@@ -896,7 +910,7 @@ void bf::ingame_view::render_relationship
   item_class::field_iterator itf;
 
   item.get_class().find_hierarchy(h);
-  
+
   for ( ; !h.empty(); h.pop_front() )
     for ( itf=h.front()->field_begin(); itf!=h.front()->field_end(); ++itf )
       if ( itf->get_field_type() == type_field::item_reference_field_type )
@@ -1053,12 +1067,15 @@ bf::ingame_view::render_item_id( wxDC& dc, const item_instance& item ) const
 void bf::ingame_view::render_item_as_sprite
 ( wxDC& dc, const item_instance& item, const wxPoint& pos ) const
 {
+  wxPoint p(pos);
+
   // the y coordinate of the ingame_view and the level are opposed
-  wxPoint gap
-    ( zoom(item.get_rendering_parameters().get_gap_x()),
-      -zoom(item.get_rendering_parameters().get_gap_y()) );
-  
-  render_sprite(dc, item, pos + gap);
+  p.x += zoom( wxCoord(item.get_rendering_parameters().get_gap_x()) );
+  p.y += zoom(item.get_rendering_parameters().get_height())
+    - zoom(item.get_rendering_parameters().get_sprite().height())
+    - zoom(item.get_rendering_parameters().get_gap_y());
+
+  render_sprite(dc, item, p);
 } // ingame_view::render_item_as_sprite()
 
 /*----------------------------------------------------------------------------*/
@@ -1092,7 +1109,6 @@ void bf::ingame_view::render_item_as_wireframe
  * \param dc The device context for the drawings.
  * \param item The item to render.
  * \param pos The position of the item on the screen.
- * \param active Tell if the item is in the active layer.
  */
 void bf::ingame_view::render_item_id
 ( wxDC& dc, const item_instance& item, const wxPoint& pos ) const
@@ -1144,7 +1160,7 @@ void bf::ingame_view::render_item_as_point
 bool bf::ingame_view::render_sprite
 ( wxDC& dc, const item_instance& item, const wxPoint& pos ) const
 {
-  std::pair<wxBitmap, wxPoint> spr( get_item_visual(item, pos) );
+  std::pair<wxBitmap, wxPoint> spr( get_item_visual(item) );
 
   if ( spr.first.IsOk() )
     dc.DrawBitmap
@@ -1189,16 +1205,16 @@ void bf::ingame_view::render_non_valid_item
 ( wxDC& dc, const item_instance& item ) const
 {
   wxPen pen( *wxRED );
-  
+
   dc.SetPen( pen );
   dc.SetBrush(*wxTRANSPARENT_BRUSH);
-  
+
   const wxPoint pos
     ( zoom((wxCoord)item.get_rendering_parameters().get_left()) - m_view.x,
       m_view.y + GetSize().y
       - zoom((wxCoord)item.get_rendering_parameters().get_bottom())
       - zoom((wxCoord)item.get_rendering_parameters().get_height()) );
-  
+
   wxSize size
     ( zoom((wxCoord)item.get_rendering_parameters().get_width()),
       zoom((wxCoord)item.get_rendering_parameters().get_height()) );
@@ -1221,12 +1237,12 @@ void bf::ingame_view::render_non_valid_item_as_point
 ( wxDC& dc, const item_instance& item, const wxPoint& pos ) const
 {
   wxPoint p[2];
-  
+
   p[0] = wxPoint( pos.x - 2 * s_point_size, pos.y - 2 * s_point_size );
   p[1] = wxPoint( pos.x + 2 * s_point_size, pos.y + 2 * s_point_size );
 
   dc.DrawPolygon(2, p);
-  
+
   p[0] = wxPoint( pos.x - 2 * s_point_size, pos.y + 2 * s_point_size );
   p[1] = wxPoint( pos.x + 2 * s_point_size, pos.y - 2 * s_point_size);
 
@@ -1239,9 +1255,10 @@ void bf::ingame_view::render_non_valid_item_as_point
  * \param dc The device context for the drawings.
  * \param item The item to render.
  * \param pos The position of the box.
+ * \param size The size of the box.
  */
 void bf::ingame_view::render_non_valid_item_box
-( wxDC& dc, const item_instance& item, 
+( wxDC& dc, const item_instance& item,
   const wxPoint& pos, const wxSize& size ) const
 {
   wxPoint p[2];
@@ -1261,8 +1278,6 @@ void bf::ingame_view::render_non_valid_item_box
 /**
  * \brief Render the grip of the selected item.
  * \param dc The device context for the drawings.
- * \param pos The position of the item.
- * \param size The size of the item.
  */
 void bf::ingame_view::render_grip( wxDC& dc ) const
 {
@@ -1375,7 +1390,7 @@ void bf::ingame_view::render_drag_mode_move( wxDC& dc ) const
           m_view.y + GetSize().y - zoom(m_drag_info->delta().y)
           - zoom((wxCoord)(*it)->get_rendering_parameters().get_bottom())
           - zoom((wxCoord)(*it)->get_rendering_parameters().get_height()) );
-      
+
       render_item(dc, **it, pos, get_active_index());
     }
 } // ingame_view::render_drag_mode_move()
@@ -1415,39 +1430,69 @@ void bf::ingame_view::render_grid( wxDC& dc ) const
 {
   if ( !empty() && m_display_grid )
     {
-      wxColour color(200,200,200);
+      wxColour color(200, 200, 200);
       dc.SetPen( wxPen( color, 1, wxSHORT_DASH ) );
       dc.SetBrush(*wxTRANSPARENT_BRUSH);
 
-      int width = get_active_layer().get_width();
-      int height = get_active_layer().get_height();
-      wxSize offset = m_grid.get_offset();
-      wxSize step = m_grid.get_step();
-
-      for ( int column = unzoom(m_view.x) / step.x;
-            (column * step.x) + offset.x < width + unzoom(m_view.x) ; ++column )
-        if ( (column * step.x) + offset.x > unzoom(m_view.x) )
-          {
-            wxPoint p[2];
-            p[0] = wxPoint( zoom((column * step.x)+offset.x) - m_view.x, 0 );
-            p[1] = wxPoint( p[0].x, zoom(height) );
-            dc.DrawPolygon(2, p);
-          }
-
-      for ( int line = unzoom(m_view.y) / step.y;
-            (line * step.y) + offset.y < height + unzoom(m_view.y) ; ++line )
-        if ( (line * step.y) + offset.y > unzoom(m_view.y) )
-          {
-            wxPoint p[2];
-            p[0] = wxPoint
-              ( 0,
-                zoom(GetSize().y - (line * step.y) - offset.y - 1)
-                + m_view.y );
-            p[1] = wxPoint( zoom(width), p[0].y );
-            dc.DrawPolygon(2, p);
-          }
+      render_grid_vertical_lines(dc);
+      render_grid_horizontal_lines(dc);
     }
-} // ingame_view::render_grid()
+} // render_grid()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Render the vertical lines of the grid on the screen.
+ * \param dc The device context for the drawings.
+ */
+void bf::ingame_view::render_grid_vertical_lines( wxDC& dc ) const
+{
+  const int width = get_active_layer().get_width();
+  const int height = get_active_layer().get_height();
+  const wxCoord offset = m_grid.get_offset().GetX();
+  const wxCoord step = m_grid.get_step().GetX();
+  const wxCoord bound =
+    unzoom( m_view.x + std::min(zoom(width), GetSize().x) );
+
+  for ( int column = unzoom(m_view.x) / step;
+        (column * step) + offset < bound; ++column )
+    if ( (column * step) + offset > unzoom(m_view.x) )
+      {
+        wxPoint p[2];
+        p[0] = wxPoint
+          ( zoom((column * step) + offset) - m_view.x, GetSize().y );
+        p[1] = wxPoint
+          ( p[0].x,
+            GetSize().y - std::min(zoom(height) + m_view.y, GetSize().y) );
+        dc.DrawPolygon(2, p);
+      }
+} // render_grid_vertical_lines()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Render the horizontal lines of the grid on the screen.
+ * \param dc The device context for the drawings.
+ */
+void bf::ingame_view::render_grid_horizontal_lines( wxDC& dc ) const
+{
+  const int width = get_active_layer().get_width();
+  const int height = get_active_layer().get_height();
+  const wxCoord offset = m_grid.get_offset().GetY();
+  const wxCoord step = m_grid.get_step().GetY();
+  const wxCoord bound =
+    unzoom( m_view.y + std::min(zoom(height), GetSize().y) );
+
+  for ( int line = unzoom(m_view.y) / step;
+        (line * step) + offset < bound; ++line )
+    if ( (line * step) + offset > unzoom(m_view.y) )
+      {
+        wxPoint p[2];
+        p[0] = wxPoint
+          ( 0, GetSize().y - zoom( (line * step) + offset ) + m_view.y );
+        p[1] = wxPoint
+          ( std::min( GetSize().x, zoom(width) - m_view.x), p[0].y );
+        dc.DrawPolygon(2, p);
+      }
+} // ingame_view::render_grid_horizontal_lines()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -1457,7 +1502,7 @@ void bf::ingame_view::render_grid( wxDC& dc ) const
  * \return The sprite and the position where it has to be rendered.
  */
 std::pair<wxBitmap, wxPoint> bf::ingame_view::get_item_visual
-( const item_instance& item, const wxPoint& pos ) const
+( const item_instance& item ) const
 {
   CLAW_PRECOND( item.get_rendering_parameters().has_sprite() );
 
@@ -1602,18 +1647,30 @@ bf::item_instance* bf::ingame_view::pick_first_item( const wxPoint& pos )
   item_instance* result=NULL;
   std::list<item_instance*> instances;
   layer::item_iterator it;
+  wxCoord min_max_dist = std::numeric_limits<wxCoord>::max();
 
   if ( !empty() )
     {
       layer& the_layer( m_history.get_level().get_active_layer() );
 
       for (it=the_layer.item_begin();
-           (result==NULL) && it!=the_layer.item_end(); ++it)
+           it!=the_layer.item_end(); ++it)
         {
           wxRect box = get_bounding_box( *it );
 
           if ( box.Contains(pos) )
-            result = &(*it);
+            {
+              const wxCoord dist =
+                std::max
+                ( std::max( pos.x - box.GetLeft(), box.GetRight() - pos.x),
+                  std::max( pos.y - box.GetTop(), box.GetBottom() - pos.y) );
+
+              if ( dist < min_max_dist )
+                {
+                  result = &(*it);
+                  min_max_dist = dist;
+                }
+            }
         }
     }
 
@@ -1624,7 +1681,7 @@ bf::item_instance* bf::ingame_view::pick_first_item( const wxPoint& pos )
 /**
  * \brief Pick items found near a given point.
  * \param pos The position of the point.
- * \param item The selected items.
+ * \param items The selected items.
  */
 void bf::ingame_view::pick_item
 ( const wxPoint& pos, std::set<bf::item_instance*>& items )
@@ -1656,7 +1713,7 @@ void bf::ingame_view::pick_item
 
 /*----------------------------------------------------------------------------*/
 /**
- * \brief Pick all items in the last box dragged.
+ * \brief Pick all items intersecting a box
  * \param item (out) The items selected.
  * \param box The box where to pick the items.
  */
@@ -1690,11 +1747,17 @@ void bf::ingame_view::toggle_selection( item_instance* item )
   if ( item != NULL )
     {
       if ( get_level().item_is_selected(item) )
-        get_level().remove_from_selection(item);
+        {
+          get_level().remove_from_selection(item);
+          m_layout.get_properties_frame().remove_item(item);
+        }
       else
-        get_level().add_to_selection(item, true);
+        {
+          get_level().add_to_selection(item, true);
+          m_layout.get_properties_frame().add_item(item);
+        }
 
-      update_layout();
+      Refresh();
     }
 } // ingame_view::toggle_selection()
 
@@ -1708,7 +1771,8 @@ void bf::ingame_view::add_selection( item_instance* item )
   if ( item != NULL )
     {
       get_level().add_to_selection(item, true);
-      update_layout();
+      m_layout.get_properties_frame().add_item(item);
+      Refresh();
     }
 } // ingame_view::add_selection()
 
@@ -1724,10 +1788,13 @@ void bf::ingame_view::add_selection( const std::list<item_instance*>& item )
       std::list<item_instance*>::const_iterator it;
 
       for (it=item.begin(); it!=item.end(); ++it)
-        get_level().add_to_selection(*it);
+        {
+          get_level().add_to_selection(*it);
+          m_layout.get_properties_frame().add_item(*it);
+        }
 
       get_level().add_to_selection(item.front(), true);
-      update_layout();
+      Refresh();
     }
 } // ingame_view::add_selection()
 
@@ -1759,12 +1826,15 @@ void bf::ingame_view::set_selection
                 item.end() );
 
   if ( !add )
-    get_level().clear_selection();
+    {
+      get_level().clear_selection();
+      m_layout.get_properties_frame().clear();
+    }
 
   add_selection( item );
   get_level().add_to_selection(selected, true);
 
-  update_layout();
+  Refresh();
 } // ingame_view::set_selection()
 
 /*----------------------------------------------------------------------------*/
@@ -1788,7 +1858,6 @@ void bf::ingame_view::set_selection( item_instance* item )
 /**
  * \brief Get the focus box of an item.
  * \param item The item to bound.
- * \param box (out) The bounding box of the item.
  */
 wxRect bf::ingame_view::get_bounding_box( const item_instance& item ) const
 {
@@ -1862,6 +1931,7 @@ void bf::ingame_view::move_selection()
 /**
  * \brief Calculate the gap between the grid and the selected item.
  * \param item_position The coordinate of the selected item.
+ * \param size_item The size of the selected item.
  * \param offset_grid The oofset of the grid.
  * \param step_grid The step of the grid.
  */
@@ -1904,7 +1974,7 @@ int bf::ingame_view::update_coordinate_magnetism
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Update the position of the mouse according to the constraints.
- * \param mouse_position The position of the mouse in the level.
+ * \param position The position of the mouse in the level.
  */
 void bf::ingame_view::update_mouse_position( const wxPoint& position )
 {
@@ -1923,12 +1993,12 @@ void bf::ingame_view::update_mouse_position( const wxPoint& position )
 
   pos.x +=
     update_coordinate_magnetism
-    ( item_position.x, 
+    ( item_position.x,
       (unsigned int)main_selection->get_rendering_parameters().get_width(),
       m_grid.get_offset().x, m_grid.get_step().x );
   pos.y +=
     update_coordinate_magnetism
-    ( item_position.y, 
+    ( item_position.y,
       (unsigned int)main_selection->get_rendering_parameters().get_height(),
       m_grid.get_offset().y, m_grid.get_step().y );
 
@@ -1939,27 +2009,6 @@ void bf::ingame_view::update_mouse_position( const wxPoint& position )
     m_drag_info->mouse_position.y = pos.y;
 } // ingame_view::update_mouse_position()
 
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Move the selection of one pixel.
- * \param  keycode The keycode pressed.
- */
-void bf::ingame_view::move_pixel_selection(int keycode)
-{
-  int d_x = 0;
-  int d_y = 0;
-
-  switch( keycode )
-    {
-    case WXK_LEFT: d_x = -1; break;
-    case WXK_UP: d_y = 1; break;
-    case WXK_RIGHT: d_x = 1; break;
-    case WXK_DOWN: d_y = -1; break;
-    }
-
-  do_action( new action_move_selection(get_level(), d_x, d_y) );
-} // ingame_view::move_pixel_selection()
-
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Event Move the grid of one pixel.
@@ -2060,51 +2109,43 @@ void bf::ingame_view::write_mouse_position(const wxPoint& point)
         ( wxString::Format( _("y=%d"), point.y), 2 );
 
       item_instance* item = pick_first_item(point);
-      wxString class_str, id_str;
+      wxString class_str, id_str, dist_str;
 
       if ( item != NULL )
         {
           class_str = std_to_wx_string(item->get_class().get_class_name());
 
           if ( !item->get_id().empty() )
-            id_str = _("id='") + std_to_wx_string(item->get_id()) + wxT("'");
+            id_str =
+              wxString::Format
+              ( _("id = '%s'"), std_to_wx_string(item->get_id()).c_str() );
 
           if ( !item->get_rendering_parameters().is_valid() )
-            class_str += _("  ") + 
+            class_str += wxT("  ") +
               std_to_wx_string(item->get_rendering_parameters().get_error());
+
+          if ( m_history.get_level().has_selection() )
+            {
+              const item_rendering_parameters& r1
+                ( m_history.get_level().get_main_selection()->
+                  get_rendering_parameters() );
+              const item_rendering_parameters& r2
+                ( item->get_rendering_parameters() );
+
+              dist_str =
+                wxString::Format
+                ( _("(%.2f, %.2f)"),
+                  r1.get_horizontal_middle() - r2.get_horizontal_middle(),
+                  r1.get_vertical_middle() - r2.get_vertical_middle() );
+            }
         }
 
       m_parent.GetStatusBar()->SetStatusText( class_str, 0 );
       m_parent.GetStatusBar()->SetStatusText( id_str, 3 );
-      m_parent.GetStatusBar()->SetStatusText( wxT(""), 4 );
+      m_parent.GetStatusBar()->SetStatusText( dist_str, 4 );
     }
 } // ingame_view::write_mouse_position()
 
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Choose the next layer as the current layer .
- */
-void bf::ingame_view::next_layer()
-{
-  if ( get_level().layers_count() > 1 )
-    set_active_index( (get_active_index() + 1) % get_level().layers_count() );
-} // ingame_view::next_layer()
-
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Choose the previous layer as the current layer .
- */
-void bf::ingame_view::previous_layer()
-{
-  if ( get_level().layers_count() > 1 )
-    {
-      if ( get_active_index() == 0 )
-        set_active_index( get_level().layers_count() - 1 );
-      else
-        set_active_index( get_active_index() - 1 );
-    }
-} // ingame_view::previous_layer()
-
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Check if the user tries to resize an item and set the drag mode if it
@@ -2399,12 +2440,42 @@ wxSize bf::ingame_view::unzoom( wxSize v ) const
   return wxSize( unzoom(v.x), unzoom(v.y) );
 } // ingame_view::unzoom()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Apply an action to the level.
+ * \param action The action to apply.
+ */
+template<typename Action>
+void bf::ingame_view::apply_action( Action* action )
+{
+  CLAW_PRECOND( action != NULL );
+  item_selection old_sel;
+
+  if ( m_history.get_level().has_selection() )
+    old_sel = m_history.get_level().get_selection();
+
+  if ( m_history.do_action(action) )
+    {
+      m_parent.set_changed( true );
+      m_parent.set_layer_info();
+
+      if ( !m_history.get_level().empty()
+           && !old_sel.same_group_than(m_history.get_level().get_selection()) )
+        update_layout();
+      else
+        {
+          m_layout.get_properties_frame().refresh();
+          Refresh();
+        }
+    }
+} // ingame_view::apply_action()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Event sent to a resized window.
  * \param event The event.
  */
-void bf::ingame_view::on_size(wxSizeEvent& event)
+void bf::ingame_view::on_size( wxSizeEvent& WXUNUSED(event) )
 {
   Refresh();
 } // ingame_view::on_size()
@@ -2414,7 +2485,7 @@ void bf::ingame_view::on_size(wxSizeEvent& event)
  * \brief Draw the content of the window.
  * \param event The paint event.
  */
-void bf::ingame_view::on_paint(wxPaintEvent& event)
+void bf::ingame_view::on_paint( wxPaintEvent& WXUNUSED(event) )
 {
   render();
 } // ingame_view::on_paint()
@@ -2439,12 +2510,11 @@ void bf::ingame_view::on_mouse_left_down(wxMouseEvent& event)
 
   if ( !grip )
     {
-      bool item_exist = false;
       std::set<item_instance*> items;
 
       if ( event.AltDown() )
         pick_item( point, items );
-      else if ( !exist_selected_item(point) )
+      else if ( !exist_selected_item(point) || event.ControlDown() )
         {
           item_instance* item = pick_first_item( point );
 
@@ -2452,12 +2522,9 @@ void bf::ingame_view::on_mouse_left_down(wxMouseEvent& event)
             items.insert(item);
         }
       else
-        {
-          items.insert(first_selected_item(point));
-          item_exist = true;
-        }
+        items.insert(first_selected_item(point));
 
-      if ( items.empty() && !item_exist )
+      if ( items.empty() )
         m_drag_info->drag_mode = drag_info::drag_mode_selection;
       else
         {
@@ -2599,9 +2666,9 @@ void bf::ingame_view::on_mouse_wheel_rotation(wxMouseEvent& event)
       {
         if (get_zoom() > 5)
           {
-	    set_zoom( get_zoom() - 5, point );
-	    m_parent.update_zoom();
-	  }
+            set_zoom( get_zoom() - 5, point );
+            m_parent.update_zoom();
+          }
       }
     else
       set_view_position
@@ -2612,8 +2679,8 @@ void bf::ingame_view::on_mouse_wheel_rotation(wxMouseEvent& event)
         ( m_view.x - m_parent.get_h_scrollbar().GetPageSize() / 4, m_view.y );
     else if ( event.ControlDown() )
       {
-	set_zoom( get_zoom() + 5,point );
-	m_parent.update_zoom();
+        set_zoom( get_zoom() + 5,point );
+        m_parent.update_zoom();
       }
     else
       set_view_position
@@ -2636,7 +2703,7 @@ void bf::ingame_view::on_key_down(wxKeyEvent& event)
     case WXK_RIGHT:
     case WXK_DOWN:
       if ( event.ControlDown() )
-        move_pixel_selection(event.GetKeyCode());
+        event.Skip();
       else if ( event.ShiftDown() )
         move_grid(event.GetKeyCode());
       else
@@ -2652,25 +2719,6 @@ void bf::ingame_view::on_key_down(wxKeyEvent& event)
     }
 } // ingame_view::on_key_down()
 
-/*----------------------------------------------------------------------------*/
-/**
- * \brief The user pressed a keybord key.
- * \param event The keyboard event that occured.
- */
-void bf::ingame_view::on_key_up(wxKeyEvent& event)
-{
-  if ( !event.ControlDown() && !event.ShiftDown() && !event.AltDown() )
-    switch( event.GetKeyCode() )
-      {
-      case WXK_PAGEUP: previous_layer(); break;
-      case WXK_PAGEDOWN: next_layer(); break;
-      default:
-        event.Skip();
-      }
-  else
-    event.Skip();
-} // ingame_view::on_key_up()
-
 /*----------------------------------------------------------------------------*/
 BEGIN_EVENT_TABLE(bf::ingame_view, wxWindow)
   EVT_SIZE( bf::ingame_view::on_size )
@@ -2680,6 +2728,5 @@ BEGIN_EVENT_TABLE(bf::ingame_view, wxWindow)
   EVT_MIDDLE_UP( bf::ingame_view::on_mouse_middle_up )
   EVT_MOUSEWHEEL( bf::ingame_view::on_mouse_wheel_rotation )
   EVT_MOTION( bf::ingame_view::on_mouse_move )
-  EVT_KEY_UP( bf::ingame_view::on_key_up )
   EVT_KEY_DOWN( bf::ingame_view::on_key_down )
 END_EVENT_TABLE()
