@@ -1,7 +1,7 @@
 /*
     Bear Engine - Level editor
 
-    Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+    Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
     This program is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by the
@@ -37,15 +37,23 @@
 #include "bf/item_instance.hpp"
 #include "bf/layer_list_frame.hpp"
 #include "bf/layer_properties_frame.hpp"
+#include "bf/level_editor.hpp"
 #include "bf/level_file_xml_reader.hpp"
 #include "bf/level_properties_frame.hpp"
 #include "bf/path_configuration.hpp"
 #include "bf/properties_frame.hpp"
+#include "bf/windows_layout.hpp"
 #include "bf/wx_facilities.hpp"
 
 #include <wx/notebook.h>
+#include <wx/filename.h>
+#include <wx/toolbar.h>
+#include <wx/artprov.h>
+
 #include <claw/logger.hpp>
 
+DECLARE_APP(bf::level_editor)
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Default constructor.
@@ -54,22 +62,7 @@ bf::main_frame::main_frame()
   : wxFrame(NULL, wxID_ANY, wxT("Bear Factory - Level editor")),
     m_windows_layout(NULL)
 {
-  m_config.load();
-
-  if ( path_configuration::get_instance().item_class_path.empty()
-       || path_configuration::get_instance().data_path.empty() )
-    configure_paths();
-
-  m_class_pool.scan_directory
-    ( path_configuration::get_instance().item_class_path );
-
-  std::list<std::string>::const_iterator it;
-
-  for ( it=path_configuration::get_instance().data_path.begin();
-        it!=path_configuration::get_instance().data_path.end(); ++it )
-    image_pool::get_instance().scan_directory(*it);
-
-  SetSize( m_config.main_rect );
+  SetSize( wxGetApp().get_config().main_rect );
   Show();
 
   SetSizer( new wxBoxSizer(wxVERTICAL) );
@@ -78,18 +71,21 @@ bf::main_frame::main_frame()
     new wxNotebook(this, -1, wxDefaultPosition, wxDefaultSize, wxNB_TOP );
 
   layer_list_frame* lay = new layer_list_frame( tabs );
-  lay->Show( m_config.layer_list_visible );
+  lay->Show( wxGetApp().get_config().layer_list_visible );
   tabs->AddPage( lay, _("Layers") );
 
-  m_class_tree = new class_tree_ctrl( m_class_pool, tabs );
-  m_class_tree->Show( m_config.item_class_pool_visible );
+  m_class_tree =
+    new class_tree_ctrl
+    ( wxGetApp().get_item_class_pool(), tabs, IDC_CLASS_LIST );
+  m_class_tree->Show( wxGetApp().get_config().item_class_pool_visible );
   tabs->AddPage( m_class_tree, _("Class") );
 
   properties_frame* prop = new properties_frame( tabs );
-  prop->Show( m_config.properties_visible );
+  prop->Show( wxGetApp().get_config().properties_visible );
   tabs->AddPage( prop, _("Properties") );
 
-  m_windows_layout = new windows_layout( m_class_pool, *this, *prop, *lay );
+  m_windows_layout =
+    new windows_layout( wxGetApp().get_item_class_pool(), *this, *prop, *lay );
 
   create_menu();
   create_controls();
@@ -123,7 +119,8 @@ void bf::main_frame::new_level( const wxString& path )
   if ( dlg.ShowModal() == wxID_OK )
     {
       gui_level* lvl =
-        new gui_level( dlg.get_width(), dlg.get_height(), dlg.get_music() );
+        new gui_level
+        ( dlg.get_name(), dlg.get_width(), dlg.get_height(), dlg.get_music() );
 
       add_level_view( new ingame_view_frame(*m_windows_layout, lvl, path) );
     }
@@ -136,10 +133,10 @@ void bf::main_frame::new_level( const wxString& path )
  */
 void bf::main_frame::load_level( const wxString& path )
 {
+  wxLogNull no_log;
+
   level_file_xml_reader reader;
-
-  gui_level* lvl = reader.load(m_class_pool, path);
-
+  gui_level* lvl = reader.load(wxGetApp().get_item_class_pool(), path);
   add_level_view( new ingame_view_frame(*m_windows_layout, lvl, path) );
 } // main_frame::load_level()
 
@@ -156,15 +153,16 @@ void bf::main_frame::set_active_level( ingame_view_frame* view )
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Enable/disable the controls relative to the levels.
- * \param b Tell if the controls are enabled.
+ * \param enable Tell if the controls are enabled.
  */
-void bf::main_frame::turn_level_entries( bool b )
+void bf::main_frame::turn_level_entries( bool enable )
 {
-  GetMenuBar()->Enable( wxID_SAVE, b );
-  GetMenuBar()->Enable( wxID_SAVEAS, b );
-  GetMenuBar()->Enable( IDM_SAVE_ALL_LEVELS, b );
+  GetMenuBar()->Enable( wxID_SAVE, enable );
+  GetMenuBar()->Enable( wxID_SAVEAS, enable );
+  GetMenuBar()->Enable( IDM_SAVE_ALL_LEVELS, enable );
+  GetMenuBar()->Enable( IDM_UPDATE_IMAGE_POOL, enable );
 
-  FindWindowById( wxID_SAVE, this )->Enable( b );
+  GetToolBar()->EnableTool( wxID_SAVE, enable );
 } // main_frame::turn_level_entries()
 
 /*----------------------------------------------------------------------------*/
@@ -188,20 +186,18 @@ void bf::main_frame::create_menu()
  */
 void bf::main_frame::create_controls()
 {
-  wxBoxSizer* sizer = new wxBoxSizer(wxHORIZONTAL);
-  const wxSize btn_size(25, 25);
+  wxToolBar* bar = CreateToolBar();
 
-  sizer->Add
-    ( new wxButton(this, wxID_NEW, wxT("N"), wxDefaultPosition, btn_size), 0,
-      wxALL, 3 );
-  sizer->Add
-    ( new wxButton(this, wxID_OPEN, wxT("O"), wxDefaultPosition, btn_size), 0,
-      wxALL, 3 );
-  sizer->Add
-    ( new wxButton(this, wxID_SAVE, wxT("S"), wxDefaultPosition, btn_size), 0,
-      wxALL, 3 );
+  bar->AddTool
+    ( wxID_NEW, _("&New"), wxArtProvider::GetBitmap(wxART_NEW), _("New") );
+  bar->AddTool
+    ( wxID_OPEN, _("&Open"), wxArtProvider::GetBitmap(wxART_FILE_OPEN),
+      _("Open") );
+  bar->AddTool
+    ( wxID_SAVE, _("&Save"), wxArtProvider::GetBitmap(wxART_FILE_SAVE),
+      _("Save") );
 
-  GetSizer()->Add(sizer, 0, wxALL, 0);
+  bar->Realize();
 } // main_frame::create_controls()
 
 /*----------------------------------------------------------------------------*/
@@ -274,19 +270,19 @@ wxMenu* bf::main_frame::create_help_menu() const
  */
 void bf::main_frame::save_config()
 {
-  m_config.main_rect = GetScreenRect();
+  wxGetApp().get_config().main_rect = GetScreenRect();
 
-  m_config.properties_visible =
+  wxGetApp().get_config().properties_visible =
     m_windows_layout->get_properties_frame().IsShown();
-  m_config.layer_list_visible =
+  wxGetApp().get_config().layer_list_visible =
     m_windows_layout->get_layer_list_frame().IsShown();
-  m_config.item_class_pool_visible = m_class_tree->IsShown();
+  wxGetApp().get_config().item_class_pool_visible = m_class_tree->IsShown();
 
   if ( m_windows_layout->get_current_level_view() != NULL )
-    m_config.default_level_window_size =
+    wxGetApp().get_config().default_level_window_size =
       m_windows_layout->get_current_level_view()->GetSize();
 
-  m_config.save();
+  wxGetApp().get_config().save();
 } // main_frame::save_config()
 
 /*----------------------------------------------------------------------------*/
@@ -301,45 +297,20 @@ void bf::main_frame::add_level_view( ingame_view_frame* view )
   if ( m_windows_layout->get_current_level_view() != NULL )
     s = m_windows_layout->get_current_level_view()->GetSize();
   else
-    s = m_config.default_level_window_size;
+    s = wxGetApp().get_config().default_level_window_size;
 
   view->SetSize(s);
   view->Show();
 } // main_frame::add_level_view()
 
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Show the path configuration dialog.
- */
-void bf::main_frame::configure_paths()
-{
-  config_frame dlg(this);
-
-  if ( dlg.ShowModal() == wxID_OK )
-    update_image_pool();
-} // main_frame::configure_paths()
-
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Add the paths of the configuration in the image pool.
- */
-void bf::main_frame::update_image_pool()
-{
-  std::list<std::string>::const_iterator it;
-
-  for ( it=path_configuration::get_instance().data_path.begin();
-        it!=path_configuration::get_instance().data_path.end(); ++it )
-    image_pool::get_instance().scan_directory(*it);
-} // main_frame::update_image_pool()
-
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Answer to a click on "Configuration" menu.
  * \param event This event occured.
  */
-void bf::main_frame::on_configuration_menu(wxCommandEvent& event)
+void bf::main_frame::on_configuration_menu( wxCommandEvent& WXUNUSED(event) )
 {
-  configure_paths();
+  wxGetApp().configure();
 } // main_frame::on_configuration_menu()
 
 /*----------------------------------------------------------------------------*/
@@ -347,9 +318,10 @@ void bf::main_frame::on_configuration_menu(wxCommandEvent& event)
  * \brief Answer to a click on "Update image pool" menu.
  * \param event This event occured.
  */
-void bf::main_frame::on_update_image_pool_menu(wxCommandEvent& event)
+void bf::main_frame::on_update_image_pool_menu
+( wxCommandEvent& WXUNUSED(event) )
 {
-  update_image_pool();
+  wxGetApp().update_image_pool();
 } // main_frame::on_update_image_pool_menu()
 
 /*----------------------------------------------------------------------------*/
@@ -357,7 +329,7 @@ void bf::main_frame::on_update_image_pool_menu(wxCommandEvent& event)
  * \brief Answer to a click on "Item properties" in the window menu.
  * \param event This event occured.
  */
-void bf::main_frame::on_item_properties_menu(wxCommandEvent& event)
+void bf::main_frame::on_item_properties_menu( wxCommandEvent& event )
 {
   m_windows_layout->get_properties_frame().Show( event.IsChecked() );
 } // main_frame::on_item_properties_menu()
@@ -367,7 +339,7 @@ void bf::main_frame::on_item_properties_menu(wxCommandEvent& event)
  * \brief Answer to a click on "Layer list" in the window menu.
  * \param event This event occured.
  */
-void bf::main_frame::on_layer_list_menu(wxCommandEvent& event)
+void bf::main_frame::on_layer_list_menu( wxCommandEvent& event )
 {
   m_windows_layout->get_layer_list_frame().Show( event.IsChecked() );
 } // main_frame::on_layer_list_menu()
@@ -377,7 +349,7 @@ void bf::main_frame::on_layer_list_menu(wxCommandEvent& event)
  * \brief Answer to a click on "Item classes" in the window menu.
  * \param event This event occured.
  */
-void bf::main_frame::on_item_class_pool_menu(wxCommandEvent& event)
+void bf::main_frame::on_item_class_pool_menu( wxCommandEvent& event )
 {
   m_class_tree->Show( event.IsChecked() );
 } // main_frame::on_item_class_pool_menu()
@@ -387,7 +359,7 @@ void bf::main_frame::on_item_class_pool_menu(wxCommandEvent& event)
  * \brief Answer to a click on "New level".
  * \param event This event occured.
  */
-void bf::main_frame::on_new_level(wxCommandEvent& event)
+void bf::main_frame::on_new_level( wxCommandEvent& WXUNUSED(event) )
 {
   new_level( wxEmptyString );
 } // main_frame::on_new_level()
@@ -397,9 +369,20 @@ void bf::main_frame::on_new_level(wxCommandEvent& event)
  * \brief Answer to a click on "Open level".
  * \param event This event occured.
  */
-void bf::main_frame::on_open_level(wxCommandEvent& event)
+void bf::main_frame::on_open_level( wxCommandEvent& WXUNUSED(event) )
 {
-  wxFileDialog dlg( this, _("Choose a level"), wxT(""), wxT(""),
+  wxFileName path;
+
+  if ( m_windows_layout->get_current_level_view() != NULL )
+    {
+      path = m_windows_layout->get_current_level_view()->get_level_file();
+
+      for ( windows_layout::iterator it=m_windows_layout->begin();
+            path.GetPath().empty() && (it!=m_windows_layout->end()); ++it )
+        path = it->get_level_file();
+    }
+
+  wxFileDialog dlg( this, _("Choose a level"), path.GetPath(), wxEmptyString,
                     _("Level files (*.lvl)|*.lvl"),
                     wxFD_OPEN | wxFD_FILE_MUST_EXIST );
 
@@ -423,7 +406,7 @@ void bf::main_frame::on_open_level(wxCommandEvent& event)
  * \brief Answer to a click on "Save as".
  * \param event This event occured.
  */
-void bf::main_frame::on_save_as(wxCommandEvent& event)
+void bf::main_frame::on_save_as( wxCommandEvent& WXUNUSED(event) )
 {
   m_windows_layout->get_current_level_view()->save_as();
 } // main_frame::on_save_as()
@@ -433,7 +416,7 @@ void bf::main_frame::on_save_as(wxCommandEvent& event)
  * \brief Answer to a click on "Save".
  * \param event This event occured.
  */
-void bf::main_frame::on_save(wxCommandEvent& event)
+void bf::main_frame::on_save( wxCommandEvent& WXUNUSED(event) )
 {
   m_windows_layout->get_current_level_view()->save();
 } // main_frame::on_save()
@@ -443,7 +426,7 @@ void bf::main_frame::on_save(wxCommandEvent& event)
  * \brief Answer to a click on "Save all".
  * \param event This event occured.
  */
-void bf::main_frame::on_save_all(wxCommandEvent& event)
+void bf::main_frame::on_save_all( wxCommandEvent& WXUNUSED(event) )
 {
   windows_layout::iterator it;
 
@@ -456,7 +439,7 @@ void bf::main_frame::on_save_all(wxCommandEvent& event)
  * \brief Answer to a click on "Exit".
  * \param event This event occured.
  */
-void bf::main_frame::on_exit(wxCommandEvent& event)
+void bf::main_frame::on_exit( wxCommandEvent& WXUNUSED(event) )
 {
   Close();
 } // main_frame::on_exit()
@@ -466,7 +449,7 @@ void bf::main_frame::on_exit(wxCommandEvent& event)
  * \brief Answer to the event that occurs when opening menus.
  * \param event This event occured.
  */
-void bf::main_frame::on_menu_open(wxMenuEvent& event)
+void bf::main_frame::on_menu_open( wxMenuEvent& WXUNUSED(event) )
 {
   GetMenuBar()->Check
     ( IDM_ITEM_PROPERTIES, m_windows_layout->get_properties_frame().IsShown() );
@@ -482,7 +465,7 @@ void bf::main_frame::on_menu_open(wxMenuEvent& event)
  * \brief A menu entry is highlighted.
  * \param event This event occured.
  */
-void bf::main_frame::on_menu_highlight(wxMenuEvent& event)
+void bf::main_frame::on_menu_highlight( wxMenuEvent& event )
 {
   ingame_view_frame* current_view = m_windows_layout->get_current_level_view();
 
@@ -496,7 +479,7 @@ void bf::main_frame::on_menu_highlight(wxMenuEvent& event)
  * \brief Answer to an activation of the "online doc" menu.
  * \param event This event occured.
  */
-void bf::main_frame::on_online_doc(wxCommandEvent& event)
+void bf::main_frame::on_online_doc( wxCommandEvent& WXUNUSED(event) )
 {
   wxLaunchDefaultBrowser
     ( wxT("http://plee-the-bear.sourceforge.net/" )
@@ -508,7 +491,7 @@ void bf::main_frame::on_online_doc(wxCommandEvent& event)
  * \brief Answer to an activation of the "about" menu.
  * \param event This event occured.
  */
-void bf::main_frame::on_about(wxCommandEvent& event)
+void bf::main_frame::on_about( wxCommandEvent& WXUNUSED(event) )
 {
   about_dialog dlg(*this);
 
@@ -520,7 +503,7 @@ void bf::main_frame::on_about(wxCommandEvent& event)
  * \brief Procedure called when closing the window.
  * \param event This event occured.
  */
-void bf::main_frame::on_close(wxCloseEvent& event)
+void bf::main_frame::on_close( wxCloseEvent& event )
 {
   save_config();
 
@@ -562,6 +545,19 @@ void bf::main_frame::on_close(wxCloseEvent& event)
     event.Veto();
 } // main_frame::on_close()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Procedure called when the user picks a class in the class list.
+ * \param event This event occured.
+ */
+void bf::main_frame::on_class_selected( class_selected_event& event )
+{
+  ingame_view_frame* view = m_windows_layout->get_current_level_view();
+
+  if ( view != NULL )
+    view->add_item(event.get_class_name());
+} // main_frame::on_class_selected()
+
 /*----------------------------------------------------------------------------*/
 BEGIN_EVENT_TABLE(bf::main_frame, wxFrame)
   EVT_BUTTON( wxID_NEW, bf::main_frame::on_new_level )
@@ -592,4 +588,7 @@ BEGIN_EVENT_TABLE(bf::main_frame, wxFrame)
   EVT_MENU( wxID_ABOUT, bf::main_frame::on_about )
 
   EVT_CLOSE( bf::main_frame::on_close )
+
+  EVT_CLASS_SELECTED( bf::main_frame::IDC_CLASS_LIST,
+                      bf::main_frame::on_class_selected )
 END_EVENT_TABLE()
