@@ -1,7 +1,7 @@
 /*
     Bear Engine - Model editor
 
-    Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+    Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
     This program is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by the
@@ -39,19 +39,13 @@
  * \param action_name The name of the action.
  * \param duration The duration of the action.
  * \param sound The sound file.
- * \param width The width of the action.
- * \param height The height of the action.
  * \param auto_next The new next action.
- * \param x_alignment The type of alignment on x-coordinate.
- * \param x_value The value of alignment on x-coordinate.
- * \param y_alignment The type of alignment on y-coordinate.
- * \param y_value The value of alignment on y-coordinate.
  */
 bf::action::action
-(const std::string& action_name, double duration, 
- const sound_description& sound, 
+(const std::string& action_name, double duration,
+ const sound_description& sound,
  const std::string& auto_next)
-  : m_name(action_name), m_duration(duration), 
+  : m_name(action_name), m_duration(duration),
     m_sound(sound),
     m_auto_next(auto_next)
 {
@@ -63,8 +57,6 @@ bf::action::action
  * \brief Constructor.
  * \param action_name The name of the action.
  * \param duration The duration of the action.
- * \param width The width of the action.
- * \param height The height of the action.
  */
 bf::action::action
 (const std::string& action_name, double duration)
@@ -79,16 +71,16 @@ bf::action::action
 bf::action::~action()
 {
   mark_list::const_iterator it;
-  
+
   for ( it = m_marks.begin(); it != m_marks.end(); ++it)
-    delete *it; 
-  
+    delete *it;
+
   m_marks.clear();
 
   snapshot_list::const_iterator it_2;
   for ( it_2 = m_snapshots.begin(); it_2 != m_snapshots.end(); ++it_2)
-    delete *it_2; 
-  
+    delete *it_2;
+
   m_snapshots.clear();
 } // action::~action()
 
@@ -106,13 +98,13 @@ void bf::action::add_snapshot( snapshot* s )
   // Add the marks in the action
   for ( mark_list::const_iterator mit=m_marks.begin();
         mit!=m_marks.end(); ++mit )
-    if ( ! s->has_mark(*mit) ) 
+    if ( !s->has_mark(*mit) )
       s->add_mark(*mit);
-  
+
   if ( it==m_snapshots.end() )
     // add the new snapshot
     m_snapshots.push_back(s);
-  else if ( (*it)->get_date() != s->get_date() ) 
+  else if ( (*it)->get_date() != s->get_date() )
     // create a snapshot with a copy of the marks
     it = m_snapshots.insert( it, s );
 } // action::add_snapshot()
@@ -131,7 +123,7 @@ void bf::action::remove_snapshot( snapshot* s )
       ok = true;
     else
       ++it;
-  
+
   if (ok)
     m_snapshots.erase(it);
 } // action::remove_snapshot()
@@ -173,15 +165,15 @@ bool bf::action::has_snapshot_date( double date ) const
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell if there exists a greater date.
- * \param date The given date. 
+ * \param date The given date.
 */
 bool bf::action::has_greater_date( double date ) const
 {
   bool result = false;
 
-  if ( ! m_snapshots.empty() ) 
+  if ( !m_snapshots.empty() )
     result = ( date < (*m_snapshots.rbegin())->get_date() );
-  
+
   return result;
 } // action::has_greater_date()
 
@@ -212,18 +204,28 @@ const bf::snapshot* bf::action::get_snapshot_before_date( double date ) const
 const bf::snapshot*
 bf::action::get_snapshot_before_or_at_date( double date ) const
 {
-  const snapshot* result = NULL;
+  snapshot_list::const_iterator it( get_const_iterator_less_equal(date) );
 
-  snapshot_list::const_iterator it;
-  bool stop(false);
+  if ( it == m_snapshots.end() )
+    return NULL;
+  else
+    return *it;
+} // action::get_snapshot_before_or_at_date()
 
-  for ( it=m_snapshots.begin(); !stop && (it!=m_snapshots.end()); ++it )
-    if ( (*it)->get_date() > date )
-      stop = true;
-    else
-      result = *it;
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the snapshot at a given date or before if there is no such
+ *        snapshot.
+ * \param date The snapshot date considered.
+ */
+bf::snapshot* bf::action::get_snapshot_before_or_at_date( double date )
+{
+  snapshot_list::iterator it( get_iterator_less_equal(date) );
 
-  return result;
+  if ( it == m_snapshots.end() )
+    return NULL;
+  else
+    return *it;
 } // action::get_snapshot_before_or_at_date()
 
 /*----------------------------------------------------------------------------*/
@@ -249,44 +251,45 @@ const bf::snapshot* bf::action::get_snapshot_after_date( double date ) const
     return NULL;
 } // action::get_snapshot_after_date()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get a snapshot.
+ * \param date The date of the snapshot to return.
+ */
+bf::snapshot* bf::action::get_snapshot_after_or_at_date( double date )
+{
+  CLAW_PRECOND( has_snapshot_date(date) );
+
+  return *get_iterator_greater_equal(date);
+} // action::get_snapshot_after_or_at_date()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get a snapshot.
+ * \param date The date of the snapshot to return.
+ */
+const bf::snapshot*
+bf::action::get_snapshot_after_or_at_date( double date ) const
+{
+  CLAW_PRECOND( has_snapshot_date(date) );
+
+  return *get_const_iterator_greater_equal(date);
+} // action::get_snapshot_after_or_at_date()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Get snapshot after a given date.
  * \param date The snapshot date considered.
  * \param snapshots The list of snapshots in return.
  */
-void bf::action::get_snapshot_after_date
+void bf::action::get_all_snapshots_after_or_at_date
 ( double date, snapshot_list& snapshots ) const
 {
   snapshot_list::const_iterator it=get_const_iterator_greater_equal(date);
 
-  for ( ; it != m_snapshots.end(); ++it ) 
+  for ( ; it != m_snapshots.end(); ++it )
     snapshots.push_back(*it);
-} // action::get_snapshot_after_date()
-
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Get a snapshot.
- * \param date The date of the snapshot to return.
- */
-bf::snapshot* bf::action::get_snapshot( double date )
-{
-  CLAW_PRECOND( has_snapshot_date(date) );
-  
-  return *get_iterator_greater_equal(date);
-} // action::get_snapshot()
-
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Get a snapshot.
- * \param date The date of the snapshot to return.
- */
-const bf::snapshot* bf::action::get_snapshot( double date ) const
-{
-  CLAW_PRECOND( has_snapshot_date(date) );
-  
-  return *get_const_iterator_greater_equal(date);
-} // action::get_snapshot()
+} // action::get_all_snapshots_after_or_at_date()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -325,7 +328,7 @@ void bf::action::add_mark(mark* m)
 /**
  * \brief Remove a mark from the action.
  * \param m The mark to remove.
- * 
+ *
  * The mark is removed from all snapshots.
  */
 void bf::action::remove_mark( mark* m )
@@ -388,7 +391,7 @@ bool bf::action::has_mark_label(const std::string& mark_label) const
 bf::mark* bf::action::get_mark( const std::string& label )
 {
   CLAW_PRECOND( has_mark_label(label) );
-  
+
   mark_list::iterator it_m;
   bool ok = false;
 
@@ -408,7 +411,7 @@ bf::mark* bf::action::get_mark( const std::string& label )
 bf::action::const_mark_iterator bf::action::mark_begin() const
 {
   return m_marks.begin();
-} // mark::mark_begin()
+} // action::mark_begin()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -417,7 +420,7 @@ bf::action::const_mark_iterator bf::action::mark_begin() const
 bf::action::const_mark_iterator bf::action::mark_end() const
 {
   return m_marks.end();
-} // mark::mark_end()
+} // action::mark_end()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -426,7 +429,7 @@ bf::action::const_mark_iterator bf::action::mark_end() const
 bf::action::const_mark_ptr_iterator bf::action::mark_ptr_begin() const
 {
   return m_marks.begin();
-} // mark::mark_ptr_begin()
+} // action::mark_ptr_begin()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -435,7 +438,56 @@ bf::action::const_mark_ptr_iterator bf::action::mark_ptr_begin() const
 bf::action::const_mark_ptr_iterator bf::action::mark_ptr_end() const
 {
   return m_marks.end();
-} // mark::mark_ptr_end()
+} // action::mark_ptr_end()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the cumulated visibility of a mark in a time interval.
+ * \param m The mark for which we want the visibility.
+ * \param from The beginning of the time interval.
+ * \param to The end of the time interval.
+ */
+double bf::action::accumulated_mark_visibility
+( const mark* m, double from, double to ) const
+{
+  CLAW_PRECOND( m!=NULL );
+  CLAW_PRECOND( from <= to );
+
+  snapshot_list::const_iterator it( get_const_iterator_less_equal(from) );
+  const snapshot_list::const_iterator to_s( get_const_iterator_less_equal(to) );
+
+  CLAW_PRECOND( it != m_snapshots.end() );
+  CLAW_PRECOND( to_s != m_snapshots.end() );
+
+  double result;
+
+  // The first iterator is before \a from. If the mark is visible, we
+  // remove the time between the snapshot and \a from.
+  if ( (*it)->get_placement(m).is_visible() )
+    result = -(from - (*it)->get_date());
+  else
+    result = 0;
+
+  // Then we sum the duration of the snapshots where the mark is
+  // visible until the last snapshot.
+  while ( it!=to_s )
+    {
+      snapshot_list::const_iterator n(it);
+      ++n;
+
+      if ( (*it)->get_placement(m).is_visible() )
+        result += (*n)->get_date() - (*it)->get_date();
+
+      it = n;
+    }
+
+  // Finally, we add the time during which the mark is visible during
+  // the last snapshot
+  if ( (*to_s)->get_placement(m).is_visible() )
+    result += to - (*to_s)->get_date();
+
+  return result;
+} // action::accumulated_mark_visibility()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -462,7 +514,7 @@ void bf::action::get_bounds
 
   it=snapshot_begin();
 
-  if ( it != eit ) 
+  if ( it != eit )
     {
       it->get_bounds
         ( snapshot_min_x, snapshot_min_y, snapshot_max_x, snapshot_max_y );
@@ -477,7 +529,7 @@ void bf::action::get_bounds
       min_y = 0;
       max_x = 100;
       max_y = 100;
-    } 
+    }
 
   for (; it!=eit; ++it)
     {
@@ -509,7 +561,7 @@ const std::string&  bf::action::get_name() const
 {
   return m_name;
 } // action::get_name()
-    
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Set the name of the action to start after this one.
@@ -528,7 +580,7 @@ const std::string&  bf::action::get_auto_next() const
 {
   return m_auto_next;
 } // action::get_auto_next()
-    
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Set the duration of the action.
@@ -546,7 +598,7 @@ void bf::action::set_duration(double duration)
 double bf::action::get_duration() const
 {
   return m_duration;
-} // action::get_duration() 
+} // action::get_duration()
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -580,8 +632,11 @@ const bf::sound_description& bf::action::get_sound() const
 /**
  * \brief Compile the action.
  * \param f The file in which we compile.
+ * \param anim_ref The references to the animations common to several marks.
  */
-void bf::action::compile( compiled_file& f ) const
+void bf::action::compile
+( compiled_file& f,
+  const std::map<any_animation, std::size_t>& anim_ref ) const
 {
   f << m_name << m_duration << m_auto_next;
 
@@ -597,7 +652,7 @@ void bf::action::compile( compiled_file& f ) const
   for (itm=m_marks.begin(); itm!=m_marks.end(); ++itm, ++i)
     {
       label_to_int[itm->get_label()] = i;
-      itm->compile(f);
+      itm->compile(f, anim_ref);
     }
 
   f << m_snapshots.size();
@@ -660,3 +715,54 @@ bf::action::get_const_iterator_greater_equal( double d ) const
 
   return it;
 } // action::get_const_iterator_greater_equal()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get an iterator on the first snapshot with a date less or equal to a
+ *        given date.
+ * \param d The maximum date of the snapshot searched.
+ */
+bf::action::snapshot_list::iterator
+bf::action::get_iterator_less_equal( double d )
+{
+  snapshot_list::iterator it( m_snapshots.begin() );
+  snapshot_list::iterator result( m_snapshots.end() );
+  bool stop(false);
+
+  while ( !stop && (it!=m_snapshots.end()) )
+    if ( (*it)->get_date() > d )
+      stop = true;
+    else
+      {
+        result = it;
+        ++it;
+      }
+
+  return result;
+} // action::get_iterator_less_equal()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get a constant iterator on the first snapshot with a date less or
+ *        equal to a given date.
+ * \param d The maximum date of the snapshot searched.
+ */
+bf::action::snapshot_list::const_iterator
+bf::action::get_const_iterator_less_equal( double d ) const
+{
+  snapshot_list::const_iterator it( m_snapshots.begin() );
+  snapshot_list::const_iterator result( m_snapshots.end() );
+  bool stop(false);
+
+  while ( !stop && (it!=m_snapshots.end()) )
+    if ( (*it)->get_date() > d )
+      stop = true;
+    else
+      {
+        result = it;
+        ++it;
+      }
+
+  return result;
+} // action::get_const_iterator_less_equal()
+
