@@ -1,7 +1,7 @@
 /*
     Bear Engine - Model editor
 
-    Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+    Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
     This program is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by the
@@ -29,6 +29,7 @@
 #include "bf/model_frame.hpp"
 
 #include "bf/compiled_file.hpp"
+#include "bf/gui_model.hpp"
 #include "bf/model_view_ctrl.hpp"
 #include "bf/model_view.hpp"
 #include "bf/mark.hpp"
@@ -47,7 +48,9 @@
 
 #include "bf/icon/compile.xpm"
 #include "bf/icon/player_play.xpm"
+#include "bf/icon/player_start.xpm"
 #include "bf/icon/player_stop.xpm"
+#include "bf/icon/wireframe.xpm"
 #include "bf/icon/zoom_fit.xpm"
 #include "bf/icon/zoom_in.xpm"
 #include "bf/icon/zoom_out.xpm"
@@ -80,7 +83,7 @@ const unsigned int bf::model_frame::s_button_width = 25;
 bf::model_frame::model_frame
 ( windows_layout& layout, gui_model* mdl, const wxString& model_file )
   : wxFrame( layout.get_root_window(), wxID_ANY, _("New model") ),
-    m_layout(layout), m_history(mdl),  m_model_file(model_file), 
+    m_layout(layout), m_history(mdl),  m_model_file(model_file),
     m_changed(false), m_slider(NULL), m_timer(this, ID_TIMER)
 {
   CLAW_PRECOND(mdl != NULL);
@@ -93,6 +96,15 @@ bf::model_frame::model_frame
   set_changed(false);
 } // model_frame::model_frame()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Get the path of the file in which the model is saved.
+ */
+const wxString& bf::model_frame::get_model_file() const
+{
+  return m_model_file;
+} // model_frame::get_model_file()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Tell if the model is changed or not.
@@ -150,9 +162,9 @@ void bf::model_frame::set_active_action( const std::string& action_name )
   m_history.get_model().set_active_action(action_name);
   m_history.get_model().clear_mark_selection();
   set_date(0);
-  
+
   update_layout();
-  
+
   make_title();
 } // model_frame::set_active_action()
 
@@ -165,7 +177,7 @@ void bf::model_frame::reset_active_action()
   m_history.get_model().reset_active_action();
   m_history.get_model().clear_mark_selection();
   set_date(0);
-  
+
   update_layout();
 
   make_title();
@@ -230,7 +242,7 @@ bool bf::model_frame::save()
 bool bf::model_frame::save_as()
 {
   bool result = false;
-  wxFileDialog dlg( this, _("Choose a file"), wxT(""), m_model_file,
+  wxFileDialog dlg( this, _("Choose a file"), wxEmptyString, m_model_file,
                     _("Model files (*.mdl)|*.mdl"),
                     wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
 
@@ -302,12 +314,12 @@ void bf::model_frame::make_title()
 
   if ( m_history.get_model().has_active_action() )
     SetTitle
-      ( GetTitle() + _(" (") +
+      ( GetTitle() + wxT(" (") +
         wxFileName
         ( std_to_wx_string
           ( m_history.get_model().get_active_action().get_name())).GetName()
-        + _(")") );
-  
+        + wxT(")") );
+
   if ( m_changed )
     SetTitle( GetTitle() + wxT("*") );
 } // model_frame::make_title()
@@ -372,11 +384,11 @@ void bf::model_frame::create_member_controls()
   m_slider = new slider_with_ticks(this, ID_SLIDER,0,0,100);
 
   m_new_button = new wxBitmapButton
-    ( this, ID_NEW_SNAPSHOT, 
+    ( this, ID_NEW_SNAPSHOT,
       wxArtProvider::GetBitmap(wxART_NEW, wxART_MENU, wxSize(16,16)) );
-  
+
   m_delete_button = new wxBitmapButton
-    ( this, ID_DELETE_SNAPSHOT, 
+    ( this, ID_DELETE_SNAPSHOT,
       wxArtProvider::GetBitmap(wxART_DELETE, wxART_MENU, wxSize(16,16)) );
 
   m_play_button = new wxBitmapButton
@@ -396,6 +408,9 @@ void bf::model_frame::create_sizer_controls()
   wxBoxSizer* v_sizer = new wxBoxSizer(wxVERTICAL);
 
   h_sizer->Add( m_play_button );
+  h_sizer->Add
+    ( new wxBitmapButton(this, ID_START, wxBitmap(player_start_xpm)) );
+
   h_sizer->Add( m_slider, 1, wxEXPAND);
   h_sizer->Add( m_new_button );
   h_sizer->Add( m_delete_button );
@@ -445,9 +460,9 @@ wxMenu* bf::model_frame::create_model_menu() const
   result->AppendSeparator();
   result->Append
     ( ID_CHECK_MODEL, _("Check the &validty of the model\tF8"),
-      _("Check validity of the level."));
+      _("Check the validity of the model."));
   result->Append
-    ( ID_COMPILE_MODEL, _("C&ompile the level\tF9"), _("Compile the model."));
+    ( ID_COMPILE_MODEL, _("C&ompile the model\tF9"), _("Compile the model."));
   result->AppendSeparator();
   result->Append( wxID_CLOSE, _("&Close"), _("Close the model.") );
 
@@ -495,6 +510,26 @@ wxMenu* bf::model_frame::create_view_menu() const
 {
   wxMenu* result = new wxMenu();
 
+  result->AppendCheckItem
+    ( ID_WIREFRAME, _("&Wireframe\tW"),
+      _("Display the bounding box and the marks.") );
+
+  result->AppendSeparator();
+
+  wxMenu* sub_menu = new wxMenu;
+
+  wx_menu_append_item
+    ( sub_menu, wxID_ZOOM_IN, _("Zoom more\tCtrl+Wheel up"),
+      _("Increase the zoom ratio."), wxBitmap(zoom_in_xpm) );
+  wx_menu_append_item
+    ( sub_menu, wxID_ZOOM_OUT, _("Zoom less\tCtrl+Wheel down"),
+      _("Decrease the zoom ratio."), wxBitmap(zoom_out_xpm) );
+  wx_menu_append_item
+    ( sub_menu,  wxID_ZOOM_100, _("No zoom"), _("Remove the zoom ratio."),
+      wxBitmap(zoom_original_xpm) );
+
+  result->Append(wxID_ANY, _("&Zoom"), sub_menu, _("Adjust the zoom ratio.") );
+
   return result;
 } // model_frame::create_view_menu()
 
@@ -507,27 +542,43 @@ void bf::model_frame::create_toolbar()
   wxToolBar* bar = CreateToolBar();
 
   bar->AddTool
-    ( wxID_SAVE, _("&Save"), wxArtProvider::GetBitmap(wxART_FILE_SAVE) );
-  bar->AddTool( ID_COMPILE_MODEL, _("&Compile"), wxBitmap(compile_xpm) );
-  bar->AddSeparator();
-  bar->AddTool( wxID_CUT, _("Cu&t"), wxArtProvider::GetBitmap(wxART_CUT) );
-  bar->AddTool( wxID_COPY, _("&Copy"), wxArtProvider::GetBitmap(wxART_COPY) );
+    ( wxID_SAVE, _("&Save"), wxArtProvider::GetBitmap(wxART_FILE_SAVE),
+      _("Save") );
   bar->AddTool
-    ( wxID_PASTE, _("&Paste"), wxArtProvider::GetBitmap(wxART_PASTE) );
+    ( ID_COMPILE_MODEL, _("&Compile"), wxBitmap(compile_xpm), _("Compile") );
   bar->AddSeparator();
-  bar->AddTool( wxID_UNDO, _("&Undo"), wxArtProvider::GetBitmap(wxART_UNDO) );
-  bar->AddTool( wxID_REDO, _("&Redo"), wxArtProvider::GetBitmap(wxART_REDO) );
+  bar->AddTool
+    ( wxID_CUT, _("Cu&t"), wxArtProvider::GetBitmap(wxART_CUT), _("Cut") );
+  bar->AddTool
+    ( wxID_COPY, _("&Copy"), wxArtProvider::GetBitmap(wxART_COPY), _("Copy") );
+  bar->AddTool
+    ( wxID_PASTE, _("&Paste"), wxArtProvider::GetBitmap(wxART_PASTE),
+      _("Paste") );
   bar->AddSeparator();
-  
+  bar->AddTool
+    ( wxID_UNDO, _("&Undo"), wxArtProvider::GetBitmap(wxART_UNDO), _("Undo") );
+  bar->AddTool
+    ( wxID_REDO, _("&Redo"), wxArtProvider::GetBitmap(wxART_REDO), _("Redo") );
+  bar->AddSeparator();
+
+  bar->AddCheckTool
+    ( ID_WIREFRAME, _("&Wireframe"), wxBitmap(wireframe_xpm),
+      wxBitmap(wireframe_xpm), _("Wireframe") );
+  bar->AddSeparator();
+
   m_zoom_spin = new wxSpinCtrl( bar, ID_ZOOM );
   m_zoom_spin->SetRange(10, std::numeric_limits<int>::max());
   m_zoom_spin->SetValue(100);
-  
-  bar->AddTool( wxID_ZOOM_IN, _("Zoom &in"), wxBitmap(zoom_in_xpm) );
-  bar->AddTool( wxID_ZOOM_OUT, _("Zoom &out"), wxBitmap(zoom_out_xpm) );
-  bar->AddTool( wxID_ZOOM_100, _("Zoom &100"), wxBitmap(zoom_original_xpm) );
-  bar->AddTool( wxID_ZOOM_FIT, _("Zoom &fit"),  wxBitmap(zoom_fit_xpm) );
-  bar->AddControl(m_zoom_spin); 
+
+  bar->AddTool
+    ( wxID_ZOOM_IN, _("Zoom &in"), wxBitmap(zoom_in_xpm), _("Zoom in") );
+  bar->AddTool
+    ( wxID_ZOOM_OUT, _("Zoom &out"), wxBitmap(zoom_out_xpm), _("Zoom out") );
+  bar->AddTool
+    ( wxID_ZOOM_100, _("No zoom"), wxBitmap(zoom_original_xpm), _("No zoom") );
+  bar->AddTool
+    ( wxID_ZOOM_FIT, _("Zoom &fit"),  wxBitmap(zoom_fit_xpm), _("Zoom fit") );
+  bar->AddControl(m_zoom_spin);
 
   bar->Realize();
 } // model_frame::create_toolbar()
@@ -556,6 +607,15 @@ void bf::model_frame::create_accelerators()
   SetAcceleratorTable(accel);
 } // model_frame::create_accelerators()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Toggle the wireframe mode.
+ */
+void bf::model_frame::toggle_wireframe_mode()
+{
+  m_model_view->toggle_wireframe_mode();
+} // model_frame::toggle_wireframe_mode()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Check the validity of current model.
@@ -566,27 +626,27 @@ bool bf::model_frame::check_model()
   bool result = true;
   std::string error_msg;
   std::list< const action* > actions;
-  
+
   model::const_action_iterator it;
 
-  for ( it = get_model().action_begin(); 
+  for ( it = get_model().action_begin();
         it != get_model().action_end(); ++it )
-    if ( !it->get_auto_next().empty() ) 
+    if ( !it->get_auto_next().empty() )
       if ( !get_model().has_action( it->get_auto_next() )  )
         actions.push_back(&(*it));
 
-  if ( ! actions.empty() )
+  if ( !actions.empty() )
     {
       error_check_model_dialog dlg(*this,actions);
-      
+
       dlg.ShowModal();
-      
+
       unsigned int index = dlg.get_index_selected();
       unsigned int i = 0;
       std::list< const action* >::const_iterator it;
-      
-      for ( it = actions.begin(); 
-            (i != index) && (it != actions.end()); ++i, ++it) 
+
+      for ( it = actions.begin();
+            (i != index) && (it != actions.end()); ++i, ++it)
         ;
 
       set_active_action((*it)->get_name());
@@ -623,7 +683,7 @@ void bf::model_frame::compile_model()
 
       if ( m_model_file.IsEmpty() )
         do_it = save_as();
-      
+
       if ( do_it )
         compile_model_no_check();
     }
@@ -681,16 +741,16 @@ void bf::model_frame::update_layout()
 void bf::model_frame::update_slider()
 {
   m_slider->clear_ticks();
-  
+
   if ( m_history.get_model().has_active_action() )
     {
       m_slider->set_max_value
         (m_history.get_model().get_active_action().get_duration());
-      
+
       action::const_snapshot_iterator it;
       for ( it = m_history.get_model().get_active_action().snapshot_begin();
             it != m_history.get_model().get_active_action().snapshot_end();
-            ++it ) 
+            ++it )
         m_slider->add_tick(it->get_date());
     }
 } // model_frame::update_slider()
@@ -711,6 +771,14 @@ void bf::model_frame::update_button()
  */
 void bf::model_frame::update_toolbar()
 {
+  wxToolBar* bar = GetToolBar();
+
+  bar->ToggleTool(ID_WIREFRAME, m_model_view->get_wireframe_drawing());
+
+  if( m_model_view->get_zoom() == 10 )
+    bar->EnableTool(wxID_ZOOM_OUT, false);
+  else
+    bar->EnableTool(wxID_ZOOM_OUT, true);
   m_zoom_spin->SetValue(m_model_view->get_zoom());
 } // model_frame::update_toolbar()
 
@@ -726,7 +794,7 @@ void bf::model_frame::fill()
   m_model_view->reload_action();
   m_model_view->set_date(m_slider->get_value());
 } // model_frame::fill()
- 
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Indicates if we can delete a snapshot.
@@ -735,15 +803,15 @@ bool bf::model_frame::can_delete_snapshot() const
 {
   bool result = false;
 
-  if ( m_history.get_model().has_active_action() ) 
+  if ( m_history.get_model().has_active_action() )
     {
       double date = m_slider->get_value();
-      
-      result = 
+
+      result =
         m_history.get_model().get_active_action().has_snapshot_date(date) &&
         (m_slider->get_value() != 0) ;
     }
-  
+
   return result;
 } // model_frame::can_delete_snapshot()
 
@@ -755,14 +823,14 @@ bool bf::model_frame::can_create_snapshot() const
 {
   bool result = false;
 
-  if ( m_history.get_model().has_active_action() ) 
+  if ( m_history.get_model().has_active_action() )
     {
       double date = m_slider->get_value();
-      
-      result = 
-	! m_history.get_model().get_active_action().has_snapshot_date(date);
+
+      result =
+        !m_history.get_model().get_active_action().has_snapshot_date(date);
     }
-  
+
   return result;
 } // model_frame::can_create_snapshot()
 
@@ -777,13 +845,13 @@ void bf::model_frame::delete_snapshot()
       double date = m_slider->get_value();
       action& a = m_history.get_model().get_active_action();
 
-      if ( a.has_snapshot_date(date) ) 
+      if ( a.has_snapshot_date(date) )
         {
           m_slider->remove_tick(date);
-          
+
           do_action( new action_remove_snapshot
-               ( a.get_name(), a.get_snapshot(date)));
-	           
+               ( a.get_name(), a.get_snapshot_before_or_at_date(date)));
+
           fill();
         }
     }
@@ -798,14 +866,15 @@ void bf::model_frame::create_snapshot()
   if (m_history.get_model().has_active_action())
     {
       const action& a = m_history.get_model().get_active_action();
-      
-      const snapshot* ref = a.get_snapshot_before_date(m_slider->get_value());
-      if ( ref == NULL ) 
-        ref = a.get_snapshot(0);
+
+      const snapshot* ref =
+        a.get_snapshot_before_or_at_date(m_slider->get_value());
+      if ( ref == NULL )
+        ref = a.get_snapshot_before_or_at_date(0);
 
       snapshot* s = new snapshot(* ref);
       s->set_date(m_slider->get_value());
-      
+
       do_action( new action_add_snapshot( a.get_name(), s) );
       fill();
     }
@@ -849,12 +918,48 @@ void bf::model_frame::stop()
   m_timer.Stop();
 } // model_frame::stop()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Update the content of a the menu bar or the popup menu. Entries are
+ *        enabled/disabled, checkmarks are set or not.
+ * \param m The menu to update
+ */
+template<typename MenuType>
+void bf::model_frame::update_menu( MenuType& m ) const
+{
+  m.Enable(wxID_SAVE, is_changed());
+
+  if ( m_history.can_undo() )
+    m.SetLabel
+      ( wxID_UNDO,
+        wxString::Format
+        ( _("Undo \"%s\"\tCTRL+Z"),
+          m_history.get_undo_description().c_str() ) );
+  else
+    m.SetLabel(wxID_UNDO, _("Nothing to undo\tCTRL+Z"));
+
+  if ( m_history.can_redo() )
+    m.SetLabel
+      ( wxID_REDO,
+        wxString::Format
+        ( _("Redo \"%s\"\tCTRL+Y"),
+          m_history.get_undo_description().c_str() ) );
+  else
+    m.SetLabel(wxID_REDO, _("Nothing to redo\tCTRL+Y"));
+
+  m.Enable(wxID_UNDO, m_history.can_undo());
+  m.Enable(wxID_REDO, m_history.can_redo());
+
+  m.Enable(ID_NEW_SNAPSHOT, can_create_snapshot());
+  m.Enable(ID_DELETE_SNAPSHOT, can_delete_snapshot());
+} // model_frame::update_menu()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Fill the windows with the properties of this model.
  * \param event The paint event.
  */
-void bf::model_frame::on_activate(wxActivateEvent& event)
+void bf::model_frame::on_activate(wxActivateEvent& WXUNUSED(event))
 {
   m_layout.set_current_model_frame(*this);
 } // model_frame::on_activate()
@@ -914,7 +1019,7 @@ void bf::model_frame::on_context_menu(wxContextMenuEvent& event)
  * \brief Answer to a click on the menu "Save".
  * \param event This event occured.
  */
-void bf::model_frame::on_save(wxCommandEvent& event)
+void bf::model_frame::on_save(wxCommandEvent& WXUNUSED(event))
 {
   save();
 } // model_frame::on_save()
@@ -924,7 +1029,7 @@ void bf::model_frame::on_save(wxCommandEvent& event)
  * \brief Answer to a click on the menu "Save as".
  * \param event This event occured.
  */
-void bf::model_frame::on_save_as(wxCommandEvent& event)
+void bf::model_frame::on_save_as(wxCommandEvent& WXUNUSED(event))
 {
   save_as();
 } // model_frame::on_save_as()
@@ -934,7 +1039,7 @@ void bf::model_frame::on_save_as(wxCommandEvent& event)
  * \brief Answer to a click on the menu "Undo".
  * \param event This event occured.
  */
-void bf::model_frame::on_undo(wxCommandEvent& event)
+void bf::model_frame::on_undo(wxCommandEvent& WXUNUSED(event))
 {
   undo();
 } // model_frame::on_undo()
@@ -944,7 +1049,7 @@ void bf::model_frame::on_undo(wxCommandEvent& event)
  * \brief Answer to a click on the menu "Redo".
  * \param event This event occured.
  */
-void bf::model_frame::on_redo(wxCommandEvent& event)
+void bf::model_frame::on_redo(wxCommandEvent& WXUNUSED(event))
 {
   redo();
 } // model_frame::on_redo()
@@ -954,7 +1059,7 @@ void bf::model_frame::on_redo(wxCommandEvent& event)
  * \brief Answer to a click on the menu "Cut".
  * \param event This event occured.
  */
-void bf::model_frame::on_cut(wxCommandEvent& event)
+void bf::model_frame::on_cut(wxCommandEvent& WXUNUSED(event))
 {
 
 } // model_frame::on_cut()
@@ -964,7 +1069,7 @@ void bf::model_frame::on_cut(wxCommandEvent& event)
  * \brief Answer to a click on the menu "Copy".
  * \param event This event occured.
  */
-void bf::model_frame::on_copy(wxCommandEvent& event)
+void bf::model_frame::on_copy(wxCommandEvent& WXUNUSED(event))
 {
 
 } // model_frame::on_copy()
@@ -974,7 +1079,7 @@ void bf::model_frame::on_copy(wxCommandEvent& event)
  * \brief Answer to a click on the menu "Paste".
  * \param event This event occured.
  */
-void bf::model_frame::on_paste(wxCommandEvent& event)
+void bf::model_frame::on_paste(wxCommandEvent& WXUNUSED(event))
 {
 
 } // model_frame::on_paste()
@@ -984,7 +1089,7 @@ void bf::model_frame::on_paste(wxCommandEvent& event)
  * \brief Answer to a click on the menu "Delete".
  * \param event This event occured.
  */
-void bf::model_frame::on_delete(wxCommandEvent& event)
+void bf::model_frame::on_delete(wxCommandEvent& WXUNUSED(event))
 {
 
 } // model_frame::on_delete()
@@ -994,7 +1099,7 @@ void bf::model_frame::on_delete(wxCommandEvent& event)
  * \brief Answer to a click on the menu "Check model".
  * \param event This event occured.
  */
-void bf::model_frame::on_check_model(wxCommandEvent& event)
+void bf::model_frame::on_check_model(wxCommandEvent& WXUNUSED(event))
 {
   check_model_verbose();
 } // model_frame::on_check_model()
@@ -1004,7 +1109,7 @@ void bf::model_frame::on_check_model(wxCommandEvent& event)
  * \brief Answer to a click on the menu "Compile model".
  * \param event This event occured.
  */
-void bf::model_frame::on_compile_model(wxCommandEvent& event)
+void bf::model_frame::on_compile_model(wxCommandEvent& WXUNUSED(event))
 {
   compile_model();
 } // model_frame::on_compile_model()
@@ -1014,7 +1119,7 @@ void bf::model_frame::on_compile_model(wxCommandEvent& event)
  * \brief Answer to a click on the menu "Close".
  * \param event This event occured.
  */
-void bf::model_frame::on_menu_close(wxCommandEvent& event)
+void bf::model_frame::on_menu_close(wxCommandEvent& WXUNUSED(event))
 {
   Close();
 } // model_frame::on_menu_close()
@@ -1035,18 +1140,29 @@ void bf::model_frame::on_menu_highlight(wxMenuEvent& event)
  * \brief A menu is opened.
  * \param event The event.
  */
-void bf::model_frame::on_open_menu(wxMenuEvent& event)
+void bf::model_frame::on_open_menu(wxMenuEvent& WXUNUSED(event))
 {
   update_menu( m_popup_menu );
   update_menu( *GetMenuBar() );
 } // model_frame::on_menu_open()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Answer to a click on the menu "Wireframe".
+ * \param event This event occured.
+ */
+void bf::model_frame::on_wireframe(wxCommandEvent& WXUNUSED(event))
+{
+  toggle_wireframe_mode();
+  GetToolBar()->ToggleTool(ID_WIREFRAME, m_model_view->get_wireframe_drawing());
+} // model_frame::on_wireframe()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Answer to a click on the menu "zoom_100".
  * \param event This event occured.
  */
-void bf::model_frame::on_zoom_100(wxCommandEvent& event)
+void bf::model_frame::on_zoom_100(wxCommandEvent& WXUNUSED(event))
 {
   m_model_view->set_zoom(100);
   m_model_view->Refresh();
@@ -1058,7 +1174,7 @@ void bf::model_frame::on_zoom_100(wxCommandEvent& event)
  * \brief Answer to a click on the menu "zoom_in".
  * \param event This event occured.
  */
-void bf::model_frame::on_zoom_in(wxCommandEvent& event)
+void bf::model_frame::on_zoom_in(wxCommandEvent& WXUNUSED(event))
 {
   m_model_view->set_zoom( m_model_view->get_zoom() + 10);
   m_model_view->Refresh();
@@ -1070,11 +1186,11 @@ void bf::model_frame::on_zoom_in(wxCommandEvent& event)
  * \brief Answer to a click on the menu "zoom_out".
  * \param event This event occured.
  */
-void bf::model_frame::on_zoom_out(wxCommandEvent& event)
+void bf::model_frame::on_zoom_out(wxCommandEvent& WXUNUSED(event))
 {
-  if ( m_model_view->get_zoom() > 10) 
+  if ( m_model_view->get_zoom() > 10)
     {
-      if ( m_model_view->get_zoom() < 20 ) 
+      if ( m_model_view->get_zoom() < 20 )
         m_model_view->set_zoom( 10);
       else
         m_model_view->set_zoom( m_model_view->get_zoom() - 10);
@@ -1089,7 +1205,7 @@ void bf::model_frame::on_zoom_out(wxCommandEvent& event)
  * \brief Answer to a click on the menu "zoom_fit".
  * \param event This event occured.
  */
-void bf::model_frame::on_zoom_fit(wxCommandEvent& event)
+void bf::model_frame::on_zoom_fit(wxCommandEvent& WXUNUSED(event))
 {
   m_model_view->auto_zoom();
   m_model_view->Refresh();
@@ -1112,9 +1228,9 @@ void bf::model_frame::on_zoom_change( wxSpinEvent& event )
  * \brief Answer to a click on a new snapshot.
  * \param event This event occured.
  */
-void bf::model_frame::on_new_snapshot(wxCommandEvent& event)
+void bf::model_frame::on_new_snapshot(wxCommandEvent& WXUNUSED(event))
 {
-  if ( can_create_snapshot() ) 
+  if ( can_create_snapshot() )
     create_snapshot();
 } // model_frame::on_new_snapshot()
 
@@ -1123,9 +1239,9 @@ void bf::model_frame::on_new_snapshot(wxCommandEvent& event)
  * \brief Answer to a click on delete snapshot.
  * \param event This event occured.
  */
-void bf::model_frame::on_delete_snapshot(wxCommandEvent& event)
+void bf::model_frame::on_delete_snapshot(wxCommandEvent& WXUNUSED(event))
 {
-  if ( can_delete_snapshot() ) 
+  if ( can_delete_snapshot() )
     delete_snapshot();
 } // model_frame::on_delete_snapshot()
 
@@ -1134,7 +1250,7 @@ void bf::model_frame::on_delete_snapshot(wxCommandEvent& event)
  * \brief The date of the slider changed.
  * \param event This event occured.
  */
-void bf::model_frame::on_slider_change(slider_event& event)
+void bf::model_frame::on_slider_change(slider_event& WXUNUSED(event))
 {
   fill();
   get_model().set_date(m_slider->get_value());
@@ -1152,53 +1268,60 @@ void bf::model_frame::on_tick_move(tick_event& event)
   double new_date = event.get_new_value();
   bool ok = true;
 
-  if ( new_date != initial_date ) 
+  if ( new_date != initial_date )
     {
       action& act = get_model().get_active_action();
-      
+
       if ( act.has_snapshot_date( new_date ) )
         {
           wxMessageDialog dlg
             ( this, _("This date is already used."
                       " The snapshot at this date is going to be removed."),
-              _("Snapshot at this date"), wxCANCEL | wxOK );      
+              _("Snapshot at this date"), wxCANCEL | wxOK );
           ok = (dlg.ShowModal() == wxID_OK );
         }
-      
-      if ( ok ) 
+
+      if ( ok )
         {
           action_group* group = new action_group();
-          
+
           if ( event.get_copy() )
             group->set_description(_("Add snapshot"));
           else
             group->set_description(_("Move snapshot"));
-          
+
           if ( act.has_snapshot_date( new_date ) )
-            group->add_action( new action_remove_snapshot
-                               ( act.get_name(), act.get_snapshot(new_date)));
-          
+            group->add_action
+              ( new action_remove_snapshot
+                ( act.get_name(),
+                  act.get_snapshot_before_or_at_date(new_date) ) );
+
           if ( event.get_copy() )
             {
-              snapshot* s = new snapshot(*act.get_snapshot(initial_date));
+              snapshot* s =
+                new snapshot(*act.get_snapshot_before_or_at_date(initial_date));
               s->set_date(new_date);
               group->add_action( new action_add_snapshot(act.get_name(),s));
             }
           else
-            group->add_action( new action_move_snapshot
-                               (act.get_snapshot(initial_date), new_date) );
-          
+            group->add_action
+              ( new action_move_snapshot
+                ( act.get_snapshot_before_or_at_date(initial_date),
+                  new_date ) );
+
           if ( !event.get_copy() && ( initial_date == 0 ) )
             {
-              snapshot* s = new snapshot(*act.get_snapshot(initial_date));
+              snapshot* s =
+                new snapshot
+                ( *act.get_snapshot_before_or_at_date(initial_date) );
               s->set_date(0);
               group->add_action( new action_add_snapshot(act.get_name(),s));
             }
-          
-          do_action(group);      
-          set_date(new_date);      
+
+          do_action(group);
+          set_date(new_date);
           m_layout.update_snapshot();
-        }    
+        }
       else
         event.Veto();
     }
@@ -1209,7 +1332,7 @@ void bf::model_frame::on_tick_move(tick_event& event)
  * \brief Answer to a click on the "play" or "stop" button.
  * \param event The event that occured.
  */
-void bf::model_frame::on_play_stop( wxCommandEvent& event )
+void bf::model_frame::on_play_stop( wxCommandEvent& WXUNUSED(event) )
 {
   if ( m_timer.IsRunning() )
     stop();
@@ -1217,14 +1340,24 @@ void bf::model_frame::on_play_stop( wxCommandEvent& event )
     play();
 } // model_frame::on_play_stop()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Answer to a click on the "start" button.
+ * \param event The event that occured.
+ */
+void bf::model_frame::on_start( wxCommandEvent& WXUNUSED(event) )
+{
+  m_slider->set_value(0);
+} // model_frame::on_start()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief The timer has expired.
  * \param event The event that occured.
  */
-void bf::model_frame::on_timer( wxTimerEvent& event )
+void bf::model_frame::on_timer( wxTimerEvent& WXUNUSED(event) )
 {
-  play_next();  
+  play_next();
 } // model_frame::on_timer()
 
 /*----------------------------------------------------------------------------*/
@@ -1247,14 +1380,14 @@ void bf::model_frame::on_mark_move(placement_event& event)
   set_mark_selection(event.get_label());
 
   mark* m( get_model().get_main_mark_selection() );
-  const mark_placement& p = 
+  const mark_placement& p =
     get_model().get_selected_snapshot()->get_placement(m);
 
   do_action
     ( new action_modify_placement
       (get_model().get_selected_snapshot(), m, event.get_left(),
        event.get_bottom(), p.get_depth_position(),
-       p.get_angle(), p.is_visible()) );  
+       p.get_angle(), p.is_visible()) );
 } // model_frame::on_mark_move()
 
 /*----------------------------------------------------------------------------*/
@@ -1265,41 +1398,41 @@ void bf::model_frame::on_mark_move(placement_event& event)
 void bf::model_frame::on_bounding_box_change(bounding_box_event& event)
 {
   snapshot* s = get_model().get_selected_snapshot();
-  
+
   if ( s )
     {
       action_group* a = new action_group();
-      a->set_description(_("Modify bounding box"));
+      a->set_description(_("Modify the bounding box"));
 
       snapshot::const_mark_placement_iterator it;
-      
+
       if ( ( event.get_left() != 0 ) || ( event.get_bottom() != 0) )
-	for ( it = s->mark_placement_begin(); 
-	      it != s->mark_placement_end(); ++it )
-	  {
-	    const std::string label(it->get_mark()->get_label());
+        for ( it = s->mark_placement_begin();
+              it != s->mark_placement_end(); ++it )
+          {
+            const std::string label(it->get_mark()->get_label());
 
-	    mark_placement& p = s->get_placement(label);
-	    mark* m = get_model().get_active_action().get_mark(label);
-	    
-	    double x = p.get_position_x();
-	    double y = p.get_position_y();
-	    
-	    if ( event.get_mark_move_x() )
-	      x -= event.get_left();
-	      
-	    if ( event.get_mark_move_y() ) 
-	      y += event.get_height() - event.get_bottom();
+            mark_placement& p = s->get_placement(label);
+            mark* m = get_model().get_active_action().get_mark(label);
+
+            double x = p.get_position_x();
+            double y = p.get_position_y();
+
+            if ( event.get_mark_move_x() )
+              x -= event.get_left();
+
+            if ( event.get_mark_move_y() )
+              y += event.get_height() - event.get_bottom();
+
+            a->add_action
+              ( new action_modify_placement
+                (s, m, x, y, p.get_depth_position(),
+                 p.get_angle(), p.is_visible()) );
+          }
 
-	    a->add_action
-	      ( new action_modify_placement
-		(s, m, x, y, p.get_depth_position(),
-		 p.get_angle(), p.is_visible()) );
-	  }
-      
       a->add_action( new action_modify_snapshot_size
-		    (s, event.get_width(), event.get_height() ));
-      
+                    (s, event.get_width(), event.get_height() ));
+
       do_action( a );
     }
 } // model_frame::on_bounding_box_change()
@@ -1327,6 +1460,9 @@ BEGIN_EVENT_TABLE(bf::model_frame, wxFrame)
   EVT_MENU_HIGHLIGHT_ALL( bf::model_frame::on_menu_highlight )
   EVT_MENU_OPEN( bf::model_frame::on_open_menu )
 
+  EVT_MENU( bf::model_frame::ID_WIREFRAME,
+            bf::model_frame::on_wireframe )
+
   EVT_MENU( wxID_ZOOM_100,
             bf::model_frame::on_zoom_100)
   EVT_MENU( wxID_ZOOM_IN,
@@ -1334,11 +1470,11 @@ BEGIN_EVENT_TABLE(bf::model_frame, wxFrame)
   EVT_MENU( wxID_ZOOM_OUT,
             bf::model_frame::on_zoom_out)
   EVT_MENU( wxID_ZOOM_FIT,
-	    bf::model_frame::on_zoom_fit)
+            bf::model_frame::on_zoom_fit)
   EVT_MENU( bf::model_frame::ID_NEW_SNAPSHOT,
-	    bf::model_frame::on_new_snapshot)
+            bf::model_frame::on_new_snapshot)
   EVT_MENU( bf::model_frame::ID_DELETE_SNAPSHOT,
-	    bf::model_frame::on_delete_snapshot)
+            bf::model_frame::on_delete_snapshot)
   EVT_BUTTON( bf::model_frame::ID_NEW_SNAPSHOT,
               bf::model_frame::on_new_snapshot )
   EVT_BUTTON( bf::model_frame::ID_DELETE_SNAPSHOT,
@@ -1349,12 +1485,15 @@ BEGIN_EVENT_TABLE(bf::model_frame, wxFrame)
                  bf::model_frame::on_tick_move )
   EVT_SPINCTRL( ID_ZOOM,
                 bf::model_frame::on_zoom_change )
+
   EVT_BUTTON( bf::model_frame::ID_PLAY_STOP, bf::model_frame::on_play_stop )
+  EVT_BUTTON( bf::model_frame::ID_START, bf::model_frame::on_start )
+
   EVT_TIMER( bf::model_frame::ID_TIMER, bf::model_frame::on_timer )
   EVT_MARK_SELECTION_CHANGE( bf::model_frame::ID_MODEL_VIEW,
-			     bf::model_frame::on_mark_change )
+                             bf::model_frame::on_mark_change )
   EVT_PLACEMENT_MOVE( bf::model_frame::ID_MODEL_VIEW,
-		      bf::model_frame::on_mark_move )
+                      bf::model_frame::on_mark_move )
   EVT_BOUNDING_BOX_CHANGE( bf::model_frame::ID_MODEL_VIEW,
-			   bf::model_frame::on_bounding_box_change )
+                           bf::model_frame::on_bounding_box_change )
 END_EVENT_TABLE()
