@@ -1,7 +1,7 @@
 /*
     Bear Engine - Editor library
 
-    Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+    Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
     This program is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by the
@@ -57,7 +57,7 @@ namespace bf
 
 
 
-const wxEventType bf::mark_event::selection_change_event_type = 
+const wxEventType bf::mark_event::selection_change_event_type =
   wxNewEventType();
 
 /*----------------------------------------------------------------------------*/
@@ -154,7 +154,7 @@ bf::placement_event::placement_event
  * \param that The event to copy from.
  */
 bf::placement_event::placement_event( const placement_event& that )
-  : wxNotifyEvent(that), m_label(that.m_label), m_left(that.m_left), 
+  : wxNotifyEvent(that), m_label(that.m_label), m_left(that.m_left),
     m_bottom(that.m_bottom)
 {
 
@@ -241,7 +241,7 @@ bf::bounding_box_event::bounding_box_event
  * \param that The event to copy from.
  */
 bf::bounding_box_event::bounding_box_event( const bounding_box_event& that )
-  : wxNotifyEvent(that), 
+  : wxNotifyEvent(that),
     m_left(that.m_left), m_bottom(that.m_bottom),
     m_width(that.m_width), m_height(that.m_height),
     m_mark_move_x(that.m_mark_move_x), m_mark_move_y(that.m_mark_move_y)
@@ -359,12 +359,13 @@ const wxCoord bf::model_view::s_point_size = 5;
 /**
  * \brief Constructor.
  * \param parent The parent window.
+ * \param id Identifier of the window.
  * \param m The model displayed.
  */
 bf::model_view::model_view( wxWindow& parent, wxWindowID id, gui_model& m )
   : wxWindow(&parent, id), m_background_pattern( 20, 20 ),
     m_model(m), m_delta(0, 0), m_image_cache(new sprite_image_cache),
-    m_zoom(100)
+    m_zoom(100), m_wireframe_drawing(true)
 {
   const wxColour dark_gray(127, 127, 127);
   const wxColour light_gray(192, 192, 192);
@@ -521,6 +522,25 @@ void bf::model_view::reload_action()
   Refresh();
 } // model_view::reload_action()
 
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Toggle the value of the wiriframe drawing.
+ */
+void bf::model_view::toggle_wireframe_mode()
+{
+  m_wireframe_drawing = !m_wireframe_drawing;
+  Refresh();
+} // model_view::toggle_wireframe_mode()
+
+/*----------------------------------------------------------------------------*/
+/**
+ * \brief Tell if the bounding box and the marks are drawn.
+ */
+bool bf::model_view::get_wireframe_drawing() const
+{
+  return m_wireframe_drawing;
+} // model_view::get_wireframe_drawing()
+
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Draw the sprite.
@@ -528,7 +548,7 @@ void bf::model_view::reload_action()
 void bf::model_view::render()
 {
   compute_view_size();
-  
+
   wxBufferedPaintDC dc( this );
 
   if( IsShown() )
@@ -543,10 +563,10 @@ void bf::model_view::render()
 
           if ( s != NULL )
             {
-	      draw_marks(dc, *s);
-	      draw_bounding_box(dc, *s);
-	      draw_resized_bounding_box(dc, *s);
-	    }
+              draw_marks(dc, *s);
+              draw_bounding_box(dc, *s);
+              draw_resized_bounding_box(dc, *s);
+            }
           draw_view_box(dc);
         }
     }
@@ -595,13 +615,13 @@ void bf::model_view::draw_marks( wxDC& dc, const snapshot& s ) const
   for ( std::size_t i=0; i!=m.size(); ++i )
     draw_mark_placement(dc, m[i]);
 
-  if (  m_drag_info.drag_mode == drag_info::drag_mode_move ) 
+  if (  m_drag_info.drag_mode == drag_info::drag_mode_move )
     {
-      
+
       mark_placement p
-	( m_model.get_selected_snapshot
-	  ()->get_placement(m_drag_info.picked_mark->get_label()) );
-      
+        ( m_model.get_selected_snapshot
+          ()->get_placement(m_drag_info.picked_mark->get_label()) );
+
       p.set_position_x(to_model_x_coordinate(m_drag_info.mouse_position.x));
       p.set_position_y(to_model_y_coordinate(m_drag_info.mouse_position.y));
 
@@ -631,6 +651,9 @@ void bf::model_view::draw_mark_placement
 void bf::model_view::draw_mark_placement_cross
 ( wxDC& dc, const mark_placement& m ) const
 {
+  if ( !m_wireframe_drawing )
+    return;
+
   const wxPoint center
     ( to_local_x_coordinate(m.get_position_x()),
       to_local_y_coordinate(m.get_position_y()) );
@@ -696,9 +719,13 @@ void bf::model_view::draw_mark_placement_sprite
 /**
  * \brief Draw the bounding box of the action.
  * \param dc The device context for the drawings.
+ * \param s The snapshot for which we draw the bounding box.
  */
 void bf::model_view::draw_bounding_box( wxDC& dc, const snapshot& s ) const
 {
+  if ( !m_wireframe_drawing )
+    return;
+
   const wxRect r
     ( to_local_rect
       ( 0, 0,  s.get_width(), s.get_height() ) );
@@ -715,21 +742,22 @@ void bf::model_view::draw_bounding_box( wxDC& dc, const snapshot& s ) const
 /**
  * \brief Draw the bounding box of the action during a modification of size.
  * \param dc The device context for the drawings.
+ * \param s The snapshot for which we draw the bounding box.
  */
 void bf::model_view::draw_resized_bounding_box
 ( wxDC& dc, const snapshot& s ) const
 {
-  if ( m_drag_info.drag_mode == drag_info::drag_mode_size ) 
+  if ( m_drag_info.drag_mode == drag_info::drag_mode_size )
     {
       wxRect r = get_drag_mode_size_box();
 
       const wxRect re
-	( to_local_x_coordinate(r.x), to_local_y_coordinate(r.y),
-	  to_local_size(r.width), to_local_size(r.height) );
-      
+        ( to_local_x_coordinate(r.x), to_local_y_coordinate(r.y),
+          to_local_size(r.width), to_local_size(r.height) );
+
       dc.SetBrush( wxBrush(*wxBLACK, wxTRANSPARENT) );
       dc.SetPen( wxPen(*wxGREEN, 1, wxDOT) );
-      
+
       dc.DrawRectangle(re);
     }
 } // model_view::draw_bounding_box()
@@ -738,6 +766,7 @@ void bf::model_view::draw_resized_bounding_box
 /**
  * \brief Draw the grip around the bounding box of the action.
  * \param dc The device context for the drawings.
+ * \param s The snapshot for which we display the grips around the bounding box.
  */
 void bf::model_view::draw_grip( wxDC& dc, const snapshot& s ) const
 {
@@ -749,39 +778,39 @@ void bf::model_view::draw_grip( wxDC& dc, const snapshot& s ) const
 
   dc.DrawRectangle
     ( to_local_rect
-      ( s.get_width(), 
-	s.get_height(), s_grip_size, s_grip_size ) );
+      ( s.get_width(),
+        s.get_height(), s_grip_size, s_grip_size ) );
 
   dc.DrawRectangle
     ( to_local_rect
-      ( -s_grip_size, s.get_height(), 
-	s_grip_size, s_grip_size ) );
+      ( -s_grip_size, s.get_height(),
+        s_grip_size, s_grip_size ) );
 
   dc.DrawRectangle
     ( to_local_rect
-      ( s.get_width(), -s_grip_size, 
-	s_grip_size, s_grip_size ) );
-  
+      ( s.get_width(), -s_grip_size,
+        s_grip_size, s_grip_size ) );
+
   dc.DrawRectangle
     ( to_local_rect
       ( -s_grip_size, (s.get_height()-s_grip_size)/2,
-	s_grip_size, s_grip_size ));
-  
-  dc.DrawRectangle
-    ( to_local_rect
-      ( (s.get_width()-s_grip_size)/2, 
-	s.get_height(), s_grip_size, s_grip_size ) );
+        s_grip_size, s_grip_size ));
 
   dc.DrawRectangle
     ( to_local_rect
-      ( (s.get_width()-s_grip_size)/2, -s_grip_size, 
-	s_grip_size, s_grip_size ) );
+      ( (s.get_width()-s_grip_size)/2,
+        s.get_height(), s_grip_size, s_grip_size ) );
 
   dc.DrawRectangle
     ( to_local_rect
-      ( s.get_width(), 
-	(s.get_height()-s_grip_size)/2,
-	s_grip_size, s_grip_size ) );
+      ( (s.get_width()-s_grip_size)/2, -s_grip_size,
+        s_grip_size, s_grip_size ) );
+
+  dc.DrawRectangle
+    ( to_local_rect
+      ( s.get_width(),
+        (s.get_height()-s_grip_size)/2,
+        s_grip_size, s_grip_size ) );
 } // model_view::draw_grip()
 
 /*----------------------------------------------------------------------------*/
@@ -817,7 +846,7 @@ void bf::model_view::compute_view_size()
   double max_x=0;
   double max_y=0;
 
-  if ( m_model.has_active_action() ) 
+  if ( m_model.has_active_action() )
     m_model.get_active_action().get_bounds(min_x, min_y, max_x, max_y);
 
   m_view_origin.set( min_x-5, min_y-5 );
@@ -862,7 +891,7 @@ double bf::model_view::to_model_x_coordinate( double x ) const
  */
 double bf::model_view::to_model_y_coordinate( double y ) const
 {
-  return m_view_size.y + m_view_origin.y 
+  return m_view_size.y + m_view_origin.y
     - 100 * ( y + m_delta.y ) / (double)m_zoom;
 } // model_view::to_model_y_coordinate()
 
@@ -946,7 +975,7 @@ bool bf::model_view::coordinate_in_bounding_box(const  wxPoint point) const
  * \brief Check if a given position is in a anchor.
  * \param point The considered position.
  */
-bool bf::model_view::coordinate_in_anchors(const wxPoint point) 
+bool bf::model_view::coordinate_in_anchors(const wxPoint point)
 {
   bool grip = true;
 
@@ -955,105 +984,105 @@ bool bf::model_view::coordinate_in_anchors(const wxPoint point)
   const snapshot* snap =
     m_model.get_active_action().get_snapshot_before_or_at_date
     (m_player.get_date());
-  
-  if ( snap ) 
+
+  if ( snap )
     {
       const wxRect box
-	( to_local_rect( 0, 0,  snap->get_width(), snap->get_height() ) );
+        ( to_local_rect( 0, 0,  snap->get_width(), snap->get_height() ) );
       const wxSize s( grip_size, grip_size );
       wxPoint mouse_pos;
-      
+
       const wxRect top_left( box.GetTopLeft() - s, s );
       const wxRect top_right( box.GetTopRight() - wxSize(0, grip_size), s );
       const wxRect bottom_left
-	( box.GetBottomLeft() - wxSize(grip_size, 0), s );
+        ( box.GetBottomLeft() - wxSize(grip_size, 0), s );
       const wxRect bottom_right( box.GetBottomRight(), s );
-      
+
       if ( top_left.Contains(point) )
-	{
-	  m_drag_info.mouse_origin = box.GetBottomRight();
-	  mouse_pos = box.GetTopLeft();
-	  m_drag_info.mark_move_x = true;
-	}
+        {
+          m_drag_info.mouse_origin = box.GetBottomRight();
+          mouse_pos = box.GetTopLeft();
+          m_drag_info.mark_move_x = true;
+        }
       else if ( top_right.Contains(point) )
-	{
-	  m_drag_info.mouse_origin = box.GetBottomLeft();
-	  mouse_pos = box.GetTopRight();
-	}
+        {
+          m_drag_info.mouse_origin = box.GetBottomLeft();
+          mouse_pos = box.GetTopRight();
+        }
       else if ( bottom_left.Contains(point) )
-	{
-	  m_drag_info.mouse_origin = box.GetTopRight();
-	  mouse_pos = box.GetBottomLeft();
-	  m_drag_info.mark_move_x = true;
-	  m_drag_info.mark_move_y = true;
-	}
+        {
+          m_drag_info.mouse_origin = box.GetTopRight();
+          mouse_pos = box.GetBottomLeft();
+          m_drag_info.mark_move_x = true;
+          m_drag_info.mark_move_y = true;
+        }
       else if ( bottom_right.Contains(point) )
-	{
-	  m_drag_info.mouse_origin = box.GetTopLeft();
-	  mouse_pos = box.GetBottomRight();
-	  m_drag_info.mark_move_y = true;
-	}
+        {
+          m_drag_info.mouse_origin = box.GetTopLeft();
+          mouse_pos = box.GetBottomRight();
+          m_drag_info.mark_move_y = true;
+        }
       else
-	{
-	  const wxCoord h((wxCoord)to_local_size(snap->get_height())/ 2);
-	  const wxCoord w((wxCoord)to_local_size(snap->get_width())/ 2);
-	  const wxRect middle_left
-	    ( box.GetLeft() - grip_size,
-	      box.GetTop() + h - grip_size / 2, grip_size, grip_size );
-	  const wxRect middle_right
-	    ( box.GetRight(), box.GetTop() + h - grip_size / 2,
-	      grip_size, grip_size );
-	  const wxRect middle_bottom
-	    ( box.GetLeft() + w - grip_size / 2,
-	      box.GetTop() - grip_size, grip_size, grip_size );
-	  const wxRect middle_top
-	    ( box.GetLeft() + w - grip_size / 2,
-	      box.GetBottom(), grip_size, grip_size );
-	  
-	  if ( middle_left.Contains(point) )
-	    {
-	      m_drag_info.mouse_origin = box.GetTopRight();
-	      mouse_pos = box.GetTopLeft();
-	      m_drag_info.y_active = false;
-	      m_drag_info.mark_move_x = true;
-	    }
-	  else if ( middle_right.Contains(point) )
-	    {
-	      m_drag_info.mouse_origin = box.GetTopLeft();
-	      mouse_pos = box.GetTopRight();
-	      m_drag_info.y_active = false;
-	    }
-	  else if ( middle_bottom.Contains(point) )
-	    {
-	      m_drag_info.mouse_origin = box.GetBottomLeft();
-	      mouse_pos = box.GetTopLeft();
-	      m_drag_info.x_active = false;
-	    }
-	  else if ( middle_top.Contains(point) )
-	    {
-	      m_drag_info.mouse_origin = box.GetTopLeft();
-	      mouse_pos = box.GetBottomRight();
-	      m_drag_info.x_active = false;
-	      m_drag_info.mark_move_y = true;
-	    }
-	  else
-	    grip = false;
-	}
-      
+        {
+          const wxCoord h((wxCoord)to_local_size(snap->get_height())/ 2);
+          const wxCoord w((wxCoord)to_local_size(snap->get_width())/ 2);
+          const wxRect middle_left
+            ( box.GetLeft() - grip_size,
+              box.GetTop() + h - grip_size / 2, grip_size, grip_size );
+          const wxRect middle_right
+            ( box.GetRight(), box.GetTop() + h - grip_size / 2,
+              grip_size, grip_size );
+          const wxRect middle_bottom
+            ( box.GetLeft() + w - grip_size / 2,
+              box.GetTop() - grip_size, grip_size, grip_size );
+          const wxRect middle_top
+            ( box.GetLeft() + w - grip_size / 2,
+              box.GetBottom(), grip_size, grip_size );
+
+          if ( middle_left.Contains(point) )
+            {
+              m_drag_info.mouse_origin = box.GetTopRight();
+              mouse_pos = box.GetTopLeft();
+              m_drag_info.y_active = false;
+              m_drag_info.mark_move_x = true;
+            }
+          else if ( middle_right.Contains(point) )
+            {
+              m_drag_info.mouse_origin = box.GetTopLeft();
+              mouse_pos = box.GetTopRight();
+              m_drag_info.y_active = false;
+            }
+          else if ( middle_bottom.Contains(point) )
+            {
+              m_drag_info.mouse_origin = box.GetBottomLeft();
+              mouse_pos = box.GetTopLeft();
+              m_drag_info.x_active = false;
+            }
+          else if ( middle_top.Contains(point) )
+            {
+              m_drag_info.mouse_origin = box.GetTopLeft();
+              mouse_pos = box.GetBottomRight();
+              m_drag_info.x_active = false;
+              m_drag_info.mark_move_y = true;
+            }
+          else
+            grip = false;
+        }
+
       if ( grip )
-	{
-	  m_drag_info.drag_mode = drag_info::drag_mode_size;
-	  m_drag_info.mouse_position = mouse_pos;
-	}
+        {
+          m_drag_info.drag_mode = drag_info::drag_mode_size;
+          m_drag_info.mouse_position = mouse_pos;
+        }
     }
-   
+
   return grip;
 } // model_view::coordinate_in_anchors()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Update the position of the mouse according to the constraints.
- * \param mouse_position The position of the mouse in the level.
+ * \param position The position of the mouse in the model.
  */
 void bf::model_view::update_mouse_position( const wxPoint& position )
 {
@@ -1076,22 +1105,22 @@ const bf::mark* bf::model_view::find_mark_at(const wxPoint point) const
   if ( m_model.has_active_action() )
     {
       const snapshot* s =
-	m_model.get_active_action().get_snapshot_before_or_at_date
-	(m_player.get_date());
+        m_model.get_active_action().get_snapshot_before_or_at_date
+        (m_player.get_date());
 
        snapshot::const_mark_placement_iterator it;
 
-       for ( it = s->mark_placement_begin(); 
-	     ( it!= s->mark_placement_end() ) && ( result == NULL ); ++it )
-	 if ( ( (double)point.x <= to_local_x_coordinate(it->get_position_x())
-		+ (double)s_point_size ) &&
-	      ( (double)point.x >= to_local_x_coordinate(it->get_position_x())
-		- (double)s_point_size ) &&
-	      ( (double)point.y <= to_local_y_coordinate(it->get_position_y())
-		+ (double)s_point_size ) &&
-	      ( (double)point.y >= to_local_y_coordinate(it->get_position_y())
-		- (double)s_point_size ) )
-	   result = it->get_mark();
+       for ( it = s->mark_placement_begin();
+             ( it!= s->mark_placement_end() ) && ( result == NULL ); ++it )
+         if ( ( (double)point.x <= to_local_x_coordinate(it->get_position_x())
+                + (double)s_point_size ) &&
+              ( (double)point.x >= to_local_x_coordinate(it->get_position_x())
+                - (double)s_point_size ) &&
+              ( (double)point.y <= to_local_y_coordinate(it->get_position_y())
+                + (double)s_point_size ) &&
+              ( (double)point.y >= to_local_y_coordinate(it->get_position_y())
+                - (double)s_point_size ) )
+           result = it->get_mark();
     }
 
   return result;
@@ -1107,10 +1136,10 @@ void bf::model_view::move_mark(const wxPoint point)
   if ( m_model.has_active_action() && m_drag_info.picked_mark )
     {
       placement_event event
-	( m_drag_info.picked_mark->get_label(), 
-	  to_model_x_coordinate(point.x),
-	  to_model_y_coordinate(point.y),
-	  placement_event::move_event_type, GetId() );
+        ( m_drag_info.picked_mark->get_label(),
+          to_model_x_coordinate(point.x),
+          to_model_y_coordinate(point.y),
+          placement_event::move_event_type, GetId() );
       event.SetEventObject(this);
       ProcessEvent(event);
     }
@@ -1123,48 +1152,48 @@ void bf::model_view::move_mark(const wxPoint point)
 wxRect bf::model_view::get_drag_mode_size_box() const
 {
   CLAW_PRECOND( m_drag_info.drag_mode == drag_info::drag_mode_size );
-  
+
   wxRect result;
 
   const snapshot* snap =
     m_model.get_active_action().get_snapshot_before_or_at_date
     (m_player.get_date());
-  
-  if ( snap ) 
+
+  if ( snap )
     {
       if ( m_drag_info.x_active )
-	{
-	  result.x =
-	    to_model_x_coordinate
-	    ( std::min( m_drag_info.mouse_position.x, 
-			m_drag_info.mouse_origin.x ));
-	  result.width =
-	    to_model_size
-	    ( std::abs( m_drag_info.mouse_position.x - 
-			m_drag_info.mouse_origin.x ) + 1 );
-	}
+        {
+          result.x =
+            to_model_x_coordinate
+            ( std::min( m_drag_info.mouse_position.x,
+                        m_drag_info.mouse_origin.x ));
+          result.width =
+            to_model_size
+            ( std::abs( m_drag_info.mouse_position.x -
+                        m_drag_info.mouse_origin.x ) + 1 );
+        }
       else
-	{
-	  result.x = to_model_x_coordinate(m_drag_info.mouse_origin.x);
-	  result.width = (int)snap->get_width();
-	}
-      
+        {
+          result.x = to_model_x_coordinate(m_drag_info.mouse_origin.x);
+          result.width = (int)snap->get_width();
+        }
+
       if ( m_drag_info.y_active )
-	{
-	  result.y = 
-	    to_model_y_coordinate
-	    ( std::min( m_drag_info.mouse_position.y, 
-			m_drag_info.mouse_origin.y ) );
-	  result.height = 
-	    to_model_size
-	    ( std::abs( m_drag_info.mouse_position.y - 
-			m_drag_info.mouse_origin.y ) + 1);
-	}
+        {
+          result.y =
+            to_model_y_coordinate
+            ( std::min( m_drag_info.mouse_position.y,
+                        m_drag_info.mouse_origin.y ) );
+          result.height =
+            to_model_size
+            ( std::abs( m_drag_info.mouse_position.y -
+                        m_drag_info.mouse_origin.y ) + 1);
+        }
       else
-	{
-	  result.y = to_model_y_coordinate(m_drag_info.mouse_origin.y);
-	  result.height = snap->get_height();
-	}
+        {
+          result.y = to_model_y_coordinate(m_drag_info.mouse_origin.y);
+          result.height = snap->get_height();
+        }
     }
 
   return result;
@@ -1177,36 +1206,36 @@ wxRect bf::model_view::get_drag_mode_size_box() const
 void bf::model_view::apply_drag_mode_size()
 {
   CLAW_PRECOND( m_drag_info.drag_mode == drag_info::drag_mode_size );
-  
+
   wxRect box = get_drag_mode_size_box();
 
    const snapshot* snap =
     m_model.get_active_action().get_snapshot_before_or_at_date
     (m_player.get_date());
-  
-  if ( snap ) 
+
+  if ( snap )
     {
       if ( (box.x != 0)
-	   || (box.y != 0)
-	   || (box.width != snap->get_width())
-	   || (box.height != snap->get_height()) )
-	{
-	  bounding_box_event event
-	    ( box.x, box.y, box.width, box.height, 
-	      m_drag_info.mark_move_x, m_drag_info.mark_move_y,
-	      bounding_box_event::change_event_type, GetId() );
-	  event.SetEventObject(this);
-	  ProcessEvent(event);
-	}
+           || (box.y != 0)
+           || (box.width != snap->get_width())
+           || (box.height != snap->get_height()) )
+        {
+          bounding_box_event event
+            ( box.x, box.y, box.width, box.height,
+              m_drag_info.mark_move_x, m_drag_info.mark_move_y,
+              bounding_box_event::change_event_type, GetId() );
+          event.SetEventObject(this);
+          ProcessEvent(event);
+        }
     }
 } // model_view::apply_drag_mode_size()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Send the event "selection_change".
- * \param value The name of the new selected mark.
+ * \param label The name of the new selected mark.
  */
-void bf::model_view::send_event_selection_change(const std::string& label) 
+void bf::model_view::send_event_selection_change(const std::string& label)
 {
   mark_event event
     ( label, mark_event::selection_change_event_type, GetId() );
@@ -1219,7 +1248,7 @@ void bf::model_view::send_event_selection_change(const std::string& label)
  * \brief Draw the content of the window.
  * \param event The paint event.
  */
-void bf::model_view::on_paint(wxPaintEvent& event)
+void bf::model_view::on_paint( wxPaintEvent& WXUNUSED(event) )
 {
   render();
 } // model_view::on_paint()
@@ -1235,16 +1264,16 @@ void bf::model_view::on_mouse_move( wxMouseEvent& event )
     {
       if ( event.LeftIsDown() )
         {
-	  update_mouse_position(event.GetPosition());
+          update_mouse_position(event.GetPosition());
 
-	  if ( (m_drag_info.drag_mode == drag_info::drag_mode_pick) &&
-	       ( ( std::abs(event.GetPosition().x - 
-			    m_drag_info.mouse_origin.x) >= 10 )
-		 || ( std::abs(event.GetPosition().y - 
-			       m_drag_info.mouse_origin.y) >= 10 ) ) )
-	       m_drag_info.drag_mode = drag_info::drag_mode_move;
+          if ( (m_drag_info.drag_mode == drag_info::drag_mode_pick) &&
+               ( ( std::abs(event.GetPosition().x -
+                            m_drag_info.mouse_origin.x) >= 10 )
+                 || ( std::abs(event.GetPosition().y -
+                               m_drag_info.mouse_origin.y) >= 10 ) ) )
+               m_drag_info.drag_mode = drag_info::drag_mode_move;
         }
-      else 
+      else
         m_drag_info.clear();
 
       Refresh();
@@ -1258,6 +1287,9 @@ void bf::model_view::on_mouse_move( wxMouseEvent& event )
  */
 void bf::model_view::on_left_down( wxMouseEvent& event )
 {
+  if ( !m_model.has_active_action() )
+    return;
+
   const mark* m = find_mark_at( event.GetPosition() );
   m_drag_info.clear();
   m_drag_info.mouse_origin = event.GetPosition();
@@ -1285,18 +1317,18 @@ void bf::model_view::on_left_up( wxMouseEvent& event )
   switch ( m_drag_info.drag_mode )
     {
     case drag_info::drag_mode_pick:
-      if ( m_drag_info.picked_mark != NULL ) 
-	send_event_selection_change(m_drag_info.picked_mark->get_label());
+      if ( m_drag_info.picked_mark != NULL )
+        send_event_selection_change(m_drag_info.picked_mark->get_label());
       break;
     case drag_info::drag_mode_move:
-      if ( m_drag_info.picked_mark != NULL ) 
-	move_mark(event.GetPosition());
+      if ( m_drag_info.picked_mark != NULL )
+        move_mark(event.GetPosition());
       break;
     case drag_info::drag_mode_size:
       apply_drag_mode_size();
       break;
     default:
-	break;
+        break;
     }
 
   m_drag_info.clear();
