@@ -1,7 +1,7 @@
 /*
   Plee the Bear
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -30,24 +30,3 @@
 #include "engine/export.hpp"
 
 BASE_ITEM_EXPORT( gossipy_item, ptb )
-
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Constructor.
- */
-ptb::gossipy_item::gossipy_item()
-  : m_speaking(false)
-{
-  set_speaker_item(this);
-} // gossipy_item::gossipy_item()
-
-/*----------------------------------------------------------------------------*/
-/**
- * \brief Do one iteraton in the progression of the item.
- * \param elapsed_time Elapsed time sice the last call.
- */
-void ptb::gossipy_item::progress( bear::universe::time_type elapsed_time )
-{
-  if ( !m_speaking )
-    m_speaking = speak(get_level_globals());
-} // gossipy_item::progress()
