@@ -1,7 +1,7 @@
 /*
   Plee the Bear
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -28,10 +28,12 @@
  */
 #include "ptb/item/forest/gorilla.hpp"
 
-#include "ptb/item/plee/plee.hpp"
+#include "ptb/player.hpp"
+#include "ptb/util.hpp"
+
 #include "generic_items/delayed_kill_item.hpp"
 #include "generic_items/camera_shaker.hpp"
-#include "universe/world.hpp"
+#include "engine/world.hpp"
 
 #include "engine/export.hpp"
 
@@ -39,8 +41,8 @@
 
 BASE_ITEM_EXPORT( gorilla, ptb )
 
-const unsigned int ptb::gorilla::s_soul_energy = 20;
-const unsigned int ptb::gorilla::s_listen_distance = 180;
+const unsigned int ptb::gorilla::s_soul_energy = 30;
+const unsigned int ptb::gorilla::s_listen_distance = 200;
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -54,7 +56,7 @@ ptb::gorilla::gorilla()
   set_density(2);
 
   m_monster_type = monster::enemy_monster;
-  m_energy = 16;
+  m_energy = 14;
   m_offensive_phase = true;
   m_offensive_force = s_gorilla_offensive_force;
   m_offensive_coefficients[normal_attack] = 1;
@@ -88,9 +90,9 @@ void ptb::gorilla::pre_cache()
 void ptb::gorilla::build()
 {
   super::build();
-  
+
   set_model_actor( get_level_globals().get_model("model/forest/gorilla.cm") );
-  
+
   start_model_action("idle");
   m_progress = &gorilla::progress_idle;
 } // gorilla::build()
@@ -117,7 +119,7 @@ void ptb::gorilla::inform_no_energy(const monster& attacker)
 {
   m_is_injured = false;
 
-  if ( get_current_action_name() != "dead" ) 
+  if ( get_current_action_name() != "dead" )
     {
       start_dead();
       die(attacker);
@@ -163,21 +165,21 @@ void ptb::gorilla::injure
 {
   if ( ( get_current_action_name() == "idle" ) ||
        ( get_current_action_name() == "scan_left" ) ||
-       ( get_current_action_name() == "scan_right" ) || 
+       ( get_current_action_name() == "scan_right" ) ||
        ( get_current_action_name() == "come_back" ) )
     {
       m_scan_distance= 500;
       choose_angry_action();
-      
+
       const base_item* m = dynamic_cast<const base_item*>(&attacker);
-  
+
       if ( m != NULL )
-	{
-	  if ( m->get_speed().x > 0 )
-	    get_rendering_attributes().mirror(true);
-	  else
-	    get_rendering_attributes().mirror(false);
-	}
+        {
+          if ( m->get_speed().x > 0 )
+            get_rendering_attributes().mirror(true);
+          else
+            get_rendering_attributes().mirror(false);
+        }
     }
 
   super::injure(attacker,info);
@@ -197,40 +199,40 @@ bool ptb::gorilla::get_defensive_power
 {
   bool result = super::get_defensive_power(index,attacker,info);
 
-  if ( ! result )
+  if ( !result )
     {
-      if ( info.get_collision_side() == bear::universe::zone::middle_zone ) 
+      if ( info.get_collision_side() == bear::universe::zone::middle_zone )
         {
-	  if ( get_current_action_name() == "come_back" ) 
-	    {
-	      const base_item* item =dynamic_cast<const base_item*>(&attacker);
-	      if ( ( item != NULL ) && 
-		   get_rendering_attributes().is_mirrored() )
-		{
-		  if ( item->get_left() >= get_left() )
-		    result = 0;
-		}
-	      else if ( item->get_right() <= get_right() )
-		result = 0;;
-	    }
-	  else
-	  result = true;
-	}
-      else 
+          if ( get_current_action_name() == "come_back" )
+            {
+              const base_item* item =dynamic_cast<const base_item*>(&attacker);
+              if ( ( item != NULL ) &&
+                   get_rendering_attributes().is_mirrored() )
+                {
+                  if ( item->get_left() >= get_left() )
+                    result = 0;
+                }
+              else if ( item->get_right() <= get_right() )
+                result = 0;;
+            }
+          else
+          result = true;
+        }
+      else
         {
           if ( get_current_action_name() == "attack" )
             {
-              if ( get_rendering_attributes().is_mirrored() ) 
-                result = ( info.get_collision_side() == 
-                           bear::universe::zone::middle_right_zone ); 
+              if ( get_rendering_attributes().is_mirrored() )
+                result = ( info.get_collision_side() ==
+                           bear::universe::zone::middle_right_zone );
               else
-                result = ( info.get_collision_side() == 
+                result = ( info.get_collision_side() ==
                            bear::universe::zone::middle_left_zone );
             }
-	  else if ( ( get_current_action_name() == "angry_1" ) ||
-		    ( get_current_action_name() == "angry_2" ) || 
-		    ( get_current_action_name() == "angry_3" ) )
-	    result = true;
+          else if ( ( get_current_action_name() == "angry_1" ) ||
+                    ( get_current_action_name() == "angry_2" ) ||
+                    ( get_current_action_name() == "angry_3" ) )
+            result = true;
         }
     }
 
@@ -260,39 +262,42 @@ unsigned int ptb::gorilla::get_offensive_coefficient
   else
     {
       result = super::get_offensive_coefficient(index,other,info);
-      
-      if ( ( index == indefensible_attack ) || ( index == normal_attack ) )
-	{
-	  unsigned int result_attack;
-	  if ( index == indefensible_attack ) 
-	    result_attack = 1;
-	  else
-	    result_attack = 0;
 
-	  if ( info.get_collision_side() == bear::universe::zone::middle_zone )
-	    {
-	      if ( m_is_injured ) 
-		result = 0;
-	      else 
-		result = result_attack;
-	    }
-	}
+      if ( ( index == indefensible_attack ) || ( index == normal_attack ) )
+        {
+          unsigned int result_attack;
+          if ( index == indefensible_attack )
+            result_attack = 1;
+          else
+            result_attack = 0;
+
+          if ( info.get_collision_side() == bear::universe::zone::middle_zone )
+            {
+              if ( m_is_injured )
+                result = 0;
+              else
+                result = result_attack;
+            }
+        }
     }
-  
+
   return result;
 } // gorilla::get_offensive_coefficient()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief The item has attacked.
+ * \param other The monster that is attacked.
  */
-void ptb::gorilla::has_attacked()
+void ptb::gorilla::has_attacked(const monster& other)
 {
-  if ( get_current_action_name() == "attack" )
-    {
-      set_speed(bear::universe::speed_type(0,0));
-      m_want_come_back = true;
-    }
+  if ( ( other.get_monster_type() == monster::player_1_monster ) ||
+       ( other.get_monster_type() == monster::player_2_monster ) )
+    if ( get_current_action_name() == "attack" )
+      {
+        set_speed(bear::universe::speed_type(0,0));
+        m_want_come_back = true;
+      }
 } // gorilla::has_attacked()
 
 /*----------------------------------------------------------------------------*/
@@ -301,8 +306,6 @@ void ptb::gorilla::has_attacked()
  */
 void ptb::gorilla::progress_idle( bear::universe::time_type elapsed_time )
 {
-  super::progress(elapsed_time);
-  
   if ( listen() )
     choose_angry_action();
 } // gorilla::progress_idle()
@@ -313,13 +316,11 @@ void ptb::gorilla::progress_idle( bear::universe::time_type elapsed_time )
  */
 void ptb::gorilla::progress_attack( bear::universe::time_type elapsed_time )
 {
-  super::progress(elapsed_time);
-
   if ( m_want_come_back )
     start_model_action("come_back");
   else if ( !scan(get_rendering_attributes().is_mirrored(), m_scan_distance) )
     m_want_come_back = true;
-  else if ( get_rendering_attributes().is_mirrored() ) 
+  else if ( get_rendering_attributes().is_mirrored() )
     add_internal_force( bear::universe::force_type(-250000, 0) );
   else
     add_internal_force( bear::universe::force_type(250000, 0) );
@@ -331,7 +332,7 @@ void ptb::gorilla::progress_attack( bear::universe::time_type elapsed_time )
  */
 void ptb::gorilla::progress_angry( bear::universe::time_type elapsed_time )
 {
-  super::progress(elapsed_time);
+  // do nothing
 } // gorilla::progress_angry()
 
 /*----------------------------------------------------------------------------*/
@@ -340,8 +341,6 @@ void ptb::gorilla::progress_angry( bear::universe::time_type elapsed_time )
  */
 void ptb::gorilla::progress_dead( bear::universe::time_type elapsed_time )
 {
-  super::progress(elapsed_time);
-
   if (  has_bottom_contact() )
     {
       bear::visual::animation soul_anim
@@ -362,19 +361,17 @@ void ptb::gorilla::progress_dead( bear::universe::time_type elapsed_time )
  */
 void ptb::gorilla::progress_scan_left( bear::universe::time_type elapsed_time )
 {
-  super::progress(elapsed_time);
-  
-  if ( m_want_scan ) 
+  if ( m_want_scan )
     {
       if ( listen() )
-	choose_angry_action();
+        choose_angry_action();
       else if ( scan( !get_rendering_attributes().is_mirrored(),
                       m_scan_distance ) )
-	{
-	  choose_angry_action();
-	  get_rendering_attributes().mirror
-	    (! get_rendering_attributes().is_mirrored());
-	}
+        {
+          choose_angry_action();
+          get_rendering_attributes().mirror
+            (!get_rendering_attributes().is_mirrored());
+        }
     }
 } // gorilla::progress_scan_left()
 
@@ -384,15 +381,13 @@ void ptb::gorilla::progress_scan_left( bear::universe::time_type elapsed_time )
  */
 void ptb::gorilla::progress_scan_right(bear::universe::time_type elapsed_time)
 {
-  super::progress(elapsed_time);
-
-  if ( m_want_scan ) 
+  if ( m_want_scan )
     {
       if ( listen() )
-	choose_angry_action();
+        choose_angry_action();
       else if ( scan( get_rendering_attributes().is_mirrored(),
-		      m_scan_distance) )
-	choose_angry_action();
+                      m_scan_distance) )
+        choose_angry_action();
     }
 } // gorilla::progress_scan_right()
 
@@ -402,8 +397,6 @@ void ptb::gorilla::progress_scan_right(bear::universe::time_type elapsed_time)
  */
 void ptb::gorilla::progress_come_back(bear::universe::time_type elapsed_time)
 {
-  super::progress(elapsed_time);
-
   get_rendering_attributes().mirror(get_left() >= m_origin_position.x );
 
   if ( scan(get_rendering_attributes().is_mirrored(), m_scan_distance) )
@@ -412,7 +405,7 @@ void ptb::gorilla::progress_come_back(bear::universe::time_type elapsed_time)
     start_model_action("idle");
   else if ( has_right_contact() || has_left_contact() )
     start_model_action("idle");
-  else if ( get_rendering_attributes().is_mirrored() ) 
+  else if ( get_rendering_attributes().is_mirrored() )
     add_internal_force( bear::universe::force_type(-70000, 0) );
   else
     add_internal_force( bear::universe::force_type(70000, 0) );
@@ -425,7 +418,7 @@ void ptb::gorilla::progress_come_back(bear::universe::time_type elapsed_time)
 void ptb::gorilla::start_idle()
 {
   m_scan_distance = 320;
-  m_progress = &gorilla::progress_idle;  
+  m_progress = &gorilla::progress_idle;
   m_origin_position = get_bottom_left();
 } // gorilla::start_idle()
 
@@ -445,7 +438,7 @@ void ptb::gorilla::start_attack()
  */
 void ptb::gorilla::start_dead()
 {
-  if ( get_current_action_name() != "dead" ) 
+  if ( get_current_action_name() != "dead" )
     {
       start_model_action("dead");
       m_progress = &gorilla::progress_dead;
@@ -517,29 +510,24 @@ bool ptb::gorilla::listen()
     {
       result = true;
       get_rendering_attributes().mirror(true);
-    }    
-  
+    }
+
   return result;
 } // gorilla::listen()
 
 /*----------------------------------------------------------------------------*/
 /**
  * \brief Scan if there is a player in a given direction.
- * \param left_orientation True if the orientation is toward the left. 
+ * \param left_orientation True if the orientation is toward the left.
  * \param distance The distance of scan.
  */
 bool ptb::gorilla::scan
 ( bool left_orientation, bear::universe::coordinate_type distance ) const
 {
   bool result = false;
-  plee::get_instance_message msg;
-  
-  get_level_globals().send_message(plee::player_name(1), msg);
-  plee* p1 = msg.get_instance();
 
-  msg.clear();
-  get_level_globals().send_message(plee::player_name(2), msg);
-  plee* p2 = msg.get_instance();
+  player* p1 = util::find_player( get_level_globals(), 1 );
+  player* p2 = util::find_player( get_level_globals(), 2 );
 
   if( p1 != NULL )
     result = scan_for_player(*p1, left_orientation, distance);
@@ -554,11 +542,11 @@ bool ptb::gorilla::scan
 /**
  * \brief Check if a player is visible for the gorilla.
  * \param p The player we are looking for.
- * \param left_orientation True if the orientation is toward the left. 
+ * \param left_orientation True if the orientation is toward the left.
  * \param distance The distance of scan.
  */
 bool ptb::gorilla::scan_for_player
-( const plee& p, bool left_orientation,
+( const player& p, bool left_orientation,
   bear::universe::coordinate_type distance ) const
 {
   bool result = false;
@@ -570,11 +558,12 @@ bool ptb::gorilla::scan_for_player
       bear::engine::model_mark_placement m;
       m.set_position( get_center_of_mass() );
       get_mark_placement("eyes", m);
+      bear::universe::position_type pos(p.get_bottom_middle());
+      pos.y += 1;
 
       result = scan_no_wall_in_direction
         ( m.get_position(),
-          bear::universe::vector_type
-          ( p.get_bottom_middle() - m.get_position() ),
+          bear::universe::vector_type( pos - m.get_position() ),
           distance );
     }
 
@@ -585,7 +574,7 @@ bool ptb::gorilla::scan_for_player
 /**
  * \brief Scan if there is a player in a given direction.
  * \param origin The position from which we start watching.
- * \param left_orientation True if the orientation is toward the left. 
+ * \param left_orientation True if the orientation is toward the left.
  * \param distance The distance of scan.
  */
 bool ptb::gorilla::scan_no_wall_in_direction
@@ -596,9 +585,10 @@ bool ptb::gorilla::scan_no_wall_in_direction
   bear::universe::item_picking_filter filter;
   filter.set_can_move_items_value(true);
 
-  if ( dir.length() <= distance )
+  if ( ( dir.length() <= distance ) && get_layer().has_world() )
     return
-      get_owner().pick_item_in_direction(origin, dir, filter) == NULL;
+      get_layer().get_world().pick_item_in_direction
+      (origin, dir, filter) == NULL;
   else
     return false;
 } // gorilla::scan_no_wall_in_direction()
@@ -623,24 +613,26 @@ void ptb::gorilla::shake()
   // create the camera_shaker
   bear::camera_shaker* new_camera_shaker = new bear::camera_shaker;
 
-  CLAW_ASSERT(new_camera_shaker->is_valid(), 
-	      "The camera shaker isn't correctly initialized" );  
-  new_item( *new_camera_shaker );  
+  CLAW_ASSERT(new_camera_shaker->is_valid(),
+              "The camera shaker isn't correctly initialized" );
+  new_item( *new_camera_shaker );
   new_camera_shaker->set_center_of_mass(get_center_of_mass());
+  new_camera_shaker->set_shaker_force(20);
+  new_camera_shaker->toggle_on(NULL);
 
   // create the delayed_kill_item
-  bear::delayed_kill_item* new_delayed_kill_item = 
+  bear::delayed_kill_item* new_delayed_kill_item =
     new bear::delayed_kill_item;
   std::vector<base_item*> list;
   list.push_back(new_camera_shaker);
-  
+
   new_delayed_kill_item->set_item_list_field
     ("delayed_kill_item.killing_items", list);
-  new_delayed_kill_item->set_real_field( "delayed_kill_item.duration", 0.05);
+  new_delayed_kill_item->set_real_field( "delayed_kill_item.duration", 0.2);
 
-  CLAW_ASSERT(new_delayed_kill_item->is_valid(), 
-	      "The camera shaker isn't correctly initialized" );  
-  new_item( *new_delayed_kill_item );  
+  CLAW_ASSERT(new_delayed_kill_item->is_valid(),
+              "The camera shaker isn't correctly initialized" );
+  new_item( *new_delayed_kill_item );
   new_delayed_kill_item->set_center_of_mass(get_center_of_mass());
 } // gorilla::shake()
 
@@ -658,39 +650,39 @@ unsigned int ptb::gorilla::get_offensive_coefficient_attack
 {
   // The gorilla always attack on the front in attack_state.
   unsigned int result = super::get_offensive_coefficient(index,other,info);
-  
+
   if ( ( index == indefensible_attack ) || ( index == normal_attack ) )
      {
        unsigned int result_attack;
-       if ( index == indefensible_attack ) 
-	 result_attack = 1;
+       if ( index == indefensible_attack )
+         result_attack = 1;
        else
-	 result_attack = 0;
+         result_attack = 0;
 
        if ( info.get_collision_side() == bear::universe::zone::middle_zone )
-	 {
-	   if ( m_is_injured ) 
-	     result = 0;
-	   else 
-	     result = result_attack;
-	 }
-       else 
-	 {
-	   if ( get_rendering_attributes().is_mirrored() ) 
+         {
+           if ( m_is_injured )
+             result = 0;
+           else
+             result = result_attack;
+         }
+       else
+         {
+           if ( get_rendering_attributes().is_mirrored() )
                 {
-                  if ( info.get_collision_side() == 
+                  if ( info.get_collision_side() ==
                        bear::universe::zone::middle_left_zone )
                     result = result_attack;
                 }
               else
                 {
-                  if ( info.get_collision_side() == 
+                  if ( info.get_collision_side() ==
                        bear::universe::zone::middle_right_zone )
                     result = result_attack;
                 }
-	 } 
+         }
      }
-  
+
   return result;
 } // gorilla::get_offensive_coefficient_attack()
 
@@ -708,26 +700,26 @@ unsigned int ptb::gorilla::get_offensive_coefficient_angry
 {
   // The gorilla always attack in angry_state.
   unsigned int result = super::get_offensive_coefficient(index,other,info);
-  
+
   if ( ( index == indefensible_attack ) || ( index == normal_attack ) )
      {
        unsigned int result_attack;
-       if ( index == indefensible_attack ) 
-	 result_attack = 1;
+       if ( index == indefensible_attack )
+         result_attack = 1;
        else
-	 result_attack = 0;
+         result_attack = 0;
 
        if ( info.get_collision_side() == bear::universe::zone::middle_zone )
-	 {
-	   if ( m_is_injured ) 
-	     result = 0;
-	   else
-	     result = result_attack;
-	 }
-       else 
-	 result = result_attack;
+         {
+           if ( m_is_injured )
+             result = 0;
+           else
+             result = result_attack;
+         }
+       else
+         result = result_attack;
      }
-  
+
   return result;
 } // gorilla::get_offensive_coefficient_angry()
 /*----------------------------------------------------------------------------*/
@@ -742,48 +734,42 @@ unsigned int ptb::gorilla::get_offensive_coefficient_come_back
   const monster& other,
   const bear::universe::collision_info& info ) const
 {
-  // The gorilla attack front of him in come_back state.
+  // The gorilla attacks forward in come_back state.
   unsigned int result = super::get_offensive_coefficient(index,other,info);
-  
+
   if ( ( index == indefensible_attack ) || ( index == normal_attack ) )
      {
-       unsigned int result_attack;
-       if ( index == indefensible_attack ) 
-	 result_attack = 1;
-       else
-	 result_attack = 0;
-
        if ( info.get_collision_side() == bear::universe::zone::middle_zone )
-	 {
-	   if ( m_is_injured ) 
-	     result = 0;
-	   else 
-	     {
-	       const base_item* item = dynamic_cast<const base_item*>(&other);
-	   if ( ( item != NULL ) && 
-		get_rendering_attributes().is_mirrored() )
-	     {
-	       if ( item->get_left() >= get_left() )
-		 result = 0;
-	     }
-	   else if ( item->get_right() <= get_right() )
-	     result = 0;
-	 }
+         {
+           if ( m_is_injured )
+             result = 0;
+           else
+             {
+               const base_item* item = dynamic_cast<const base_item*>(&other);
+               if ( ( item != NULL ) &&
+                    get_rendering_attributes().is_mirrored() )
+                 {
+                   if ( item->get_left() >= get_left() )
+                     result = 0;
+                 }
+               else if ( item->get_right() <= get_right() )
+                 result = 0;
+         }
      }
        else
-	 {
-	   const base_item* item = dynamic_cast<const base_item*>(&other);
-	   if ( ( item != NULL ) && 
-		get_rendering_attributes().is_mirrored() )
-	     {
-	       if ( item->get_left() >= get_left() )
-		 result = 0;
-	     }
-	   else if ( item->get_right() <= get_right() )
-	     result = 0;
-	 }
+         {
+           const base_item* item = dynamic_cast<const base_item*>(&other);
+           if ( ( item != NULL ) &&
+                get_rendering_attributes().is_mirrored() )
+             {
+               if ( item->get_left() >= get_left() )
+                 result = 0;
+             }
+           else if ( item->get_right() <= get_right() )
+             result = 0;
+         }
      }
-  
+
   return result;
 } // gorilla::get_offensive_coefficient_come_back()
 
