@@ -1,7 +1,7 @@
 /*
   Plee the Bear
 
-  Copyright (C) 2005-2009 Julien Jorge, Sebastien Angibaud
+  Copyright (C) 2005-2010 Julien Jorge, Sebastien Angibaud
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -28,7 +28,9 @@
  */
 #include "ptb/item/forest/woodpecker.hpp"
 
-#include "ptb/item/plee/plee.hpp"
+#include "ptb/player.hpp"
+#include "ptb/util.hpp"
+
 #include "universe/forced_movement/forced_join.hpp"
 #include "universe/forced_movement/forced_goto.hpp"
 #include "universe/forced_movement/forced_sequence.hpp"
@@ -40,7 +42,7 @@
 
 BASE_ITEM_EXPORT( woodpecker, ptb )
 
-const unsigned int ptb::woodpecker::s_soul_energy = 20;
+const unsigned int ptb::woodpecker::s_soul_energy = 30;
 const unsigned int ptb::woodpecker::s_listen_distance = 380;
 const unsigned int ptb::woodpecker::s_scan_distance = 300;
 
@@ -93,12 +95,12 @@ void ptb::woodpecker::pre_cache()
 void ptb::woodpecker::build()
 {
   super::build();
-  
+
   m_origin_item = new bear::reference_item();
   m_origin_item->set_center_of_mass(get_center_of_mass());
   new_item( *m_origin_item );
   m_origin_item->set_artificial(true);
-  
+
   m_attack_item = new bear::reference_item();
   m_attack_item->set_bottom_left(get_bottom_left());
   new_item( *m_attack_item );
@@ -106,7 +108,7 @@ void ptb::woodpecker::build()
 
   set_model_actor( get_level_globals().get_model
                    ("model/forest/woodpecker.cm"));
-  
+
   m_origin_orientation = get_rendering_attributes().is_mirrored();
   start_model_action("peck");
   m_progress = &woodpecker::progress_peck;
@@ -136,8 +138,8 @@ void ptb::woodpecker::progress( bear::universe::time_type elapsed_time )
 void ptb::woodpecker::inform_no_energy(const monster& attacker)
 {
   m_is_injured = false;
-  
-  if ( get_current_action_name() != "dead" ) 
+
+  if ( get_current_action_name() != "dead" )
     {
       start_dead();
       die(attacker);
@@ -177,11 +179,11 @@ void ptb::woodpecker::injure
        ( get_current_action_name() == "scan" ) )
     {
       start_model_action("attack");
-      if ( ( info.get_collision_side() == 
+      if ( ( info.get_collision_side() ==
              bear::universe::zone::middle_left_zone ) ||
-           ( info.get_collision_side() == 
+           ( info.get_collision_side() ==
              bear::universe::zone::top_left_zone ) ||
-           ( info.get_collision_side() == 
+           ( info.get_collision_side() ==
              bear::universe::zone::bottom_left_zone ) )
         get_rendering_attributes().mirror(true);
       else
@@ -205,26 +207,26 @@ bool ptb::woodpecker::get_defensive_power
 {
   bool result = super::get_defensive_power(index,attacker,info);
 
-  if ( ! result )
+  if ( !result )
     {
-      if ( info.get_collision_side() == bear::universe::zone::middle_zone ) 
+      if ( info.get_collision_side() == bear::universe::zone::middle_zone )
         result = true;
-      else 
+      else
         {
           if ( get_current_action_name() == "attack" )
-               
+
             {
-              if ( get_rendering_attributes().is_mirrored() ) 
-                result = 
-                  ( info.get_collision_side() == 
+              if ( get_rendering_attributes().is_mirrored() )
+                result =
+                  ( info.get_collision_side() ==
                     bear::universe::zone::middle_right_zone ) ||
-                  ( info.get_collision_side() == 
-                    bear::universe::zone::bottom_zone ); 
+                  ( info.get_collision_side() ==
+                    bear::universe::zone::bottom_zone );
               else
-                result = 
-                  ( info.get_collision_side() == 
+                result =
+                  ( info.get_collision_side() ==
                     bear::universe::zone::middle_left_zone )  ||
-                  ( info.get_collision_side() == 
+                  ( info.get_collision_side() ==
                     bear::universe::zone::bottom_zone );
             }
         }
@@ -249,30 +251,30 @@ unsigned int ptb::woodpecker::get_offensive_coefficient
       bool ok = false;
       if ( info.get_collision_side() == bear::universe::zone::middle_zone )
         ok = true;
-      else 
+      else
         {
           if ( get_current_action_name() == "attack" )
             {
-              if ( get_rendering_attributes().is_mirrored() ) 
+              if ( get_rendering_attributes().is_mirrored() )
                 {
-                  if ( ( info.get_collision_side() == 
-                         bear::universe::zone::middle_left_zone ) || 
-                       ( info.get_collision_side() == 
+                  if ( ( info.get_collision_side() ==
+                         bear::universe::zone::middle_left_zone ) ||
+                       ( info.get_collision_side() ==
                          bear::universe::zone::bottom_zone ) )
                     ok = true;
                 }
               else
                 {
-                  if ( ( info.get_collision_side() == 
-                         bear::universe::zone::middle_right_zone ) || 
-                       ( info.get_collision_side() == 
+                  if ( ( info.get_collision_side() ==
+                         bear::universe::zone::middle_right_zone ) ||
+                       ( info.get_collision_side() ==
                          bear::universe::zone::bottom_zone ) )
                     ok = true;
                 }
             }
         }
-      
-      if ( ok ) 
+
+      if ( ok )
         {
           if ( index == indefensible_attack )
             result = 1;
@@ -280,7 +282,7 @@ unsigned int ptb::woodpecker::get_offensive_coefficient
             result = 0;
         }
     }
-  
+
   return result;
 } // woodpecker::get_offensive_coefficient()
 
@@ -288,13 +290,14 @@ unsigned int ptb::woodpecker::get_offensive_coefficient
 /*----------------------------------------------------------------------------*/
 /**
  * \brief The item has attacked.
+ * \param other The monster that is attacked.
  */
-void ptb::woodpecker::has_attacked()
+void ptb::woodpecker::has_attacked(const monster& other)
 {
-  if ( get_current_action_name() == "attack" ) 
+  if ( get_current_action_name() == "attack" )
     {
       join_origin();
-      if ( ! get_rendering_attributes().is_flipped() ) 
+      if ( !get_rendering_attributes().is_flipped() )
         get_rendering_attributes().mirror(true);
       get_rendering_attributes().flip(false);
       start_model_action("come_back");
@@ -332,15 +335,15 @@ void ptb::woodpecker::progress_attack( bear::universe::time_type elapsed_time )
 {
   super::progress(elapsed_time);
 
-  if ( get_forced_movement().is_finished() ) 
+  if ( get_forced_movement().is_finished() )
     {
       join_origin();
 
-      if ( ! get_rendering_attributes().is_flipped() ) 
+      if ( !get_rendering_attributes().is_flipped() )
         get_rendering_attributes().mirror(true);
-      
+
       get_rendering_attributes().flip(false);
-      start_model_action("come_back");      
+      start_model_action("come_back");
     }
 } // woodpecker::progress_attack()
 
@@ -375,7 +378,7 @@ void ptb::woodpecker::progress_scan( bear::universe::time_type elapsed_time )
   super::progress(elapsed_time);
   unsigned int index;
   bool left_orientation;
-  
+
   if ( scan(s_scan_distance, index, left_orientation) )
     {
       get_rendering_attributes().mirror(!left_orientation);
@@ -383,8 +386,8 @@ void ptb::woodpecker::progress_scan( bear::universe::time_type elapsed_time )
       create_attack_movement(index);
       start_model_action("attack");
     }
-  else if ( ! listen(left_orientation) )
-    start_model_action("peck"); 
+  else if ( !listen(left_orientation) )
+    start_model_action("peck");
 } // woodpecker::progress_scan()
 
 /*----------------------------------------------------------------------------*/
@@ -396,7 +399,7 @@ void ptb::woodpecker::progress_come_back
 {
   super::progress(elapsed_time);
 
-  if ( get_forced_movement().is_finished() ) 
+  if ( get_forced_movement().is_finished() )
     {
       get_rendering_attributes().mirror(m_origin_orientation);
       start_model_action("peck");
@@ -413,8 +416,8 @@ void ptb::woodpecker::start_peck()
   set_system_angle(0);
   clear_forced_movement();
   set_center_of_mass(m_origin_item->get_center_of_mass());
-  m_progress = &woodpecker::progress_peck;  
-  
+  m_progress = &woodpecker::progress_peck;
+
   bear::universe::forced_join mvt
     (std::numeric_limits<bear::universe::time_type>::infinity());
   mvt.set_reference_item(*m_origin_item);
@@ -441,7 +444,7 @@ void ptb::woodpecker::start_dead()
 {
   get_rendering_attributes().set_angle(0);
   set_system_angle(0);
-  if ( get_current_action_name() != "dead" ) 
+  if ( get_current_action_name() != "dead" )
     {
       clear_forced_movement();
       m_origin_item->kill();
@@ -469,7 +472,7 @@ void ptb::woodpecker::start_scan()
  * \brief Start to come back.
  */
 void ptb::woodpecker::start_come_back()
-{ 
+{
   get_rendering_attributes().set_angle(0);
   set_system_angle(0);
   m_progress = &woodpecker::progress_come_back;
@@ -490,7 +493,7 @@ bool ptb::woodpecker::listen(bool& left_orientation)
 /*---------------------------------------------------------------------------*/
 /**
  * \brief Scan if there is a player in a given direction.
- * \param left_orientation True if the orientation is on the left. 
+ * \param left_orientation True if the orientation is on the left.
  * \param distance The distance of scan.
  * \param index The index of the player found.
  * \param left_orientation Indicates if the player is in the left.
@@ -508,19 +511,19 @@ bool ptb::woodpecker::scan
           result = m_first_player->get_center_of_mass().distance
             ( get_center_of_mass() ) < distance;
           index = 1;
-          left_orientation = 
+          left_orientation =
             m_first_player->get_center_of_mass().x <= get_center_of_mass().x;
         }
     }
-  
-  if ( ( ! result ) && m_second_player.get() )
+
+  if ( ( !result ) && m_second_player.get() )
     {
-      if ( m_second_player->get_bottom() <= get_top() ) 
+      if ( m_second_player->get_bottom() <= get_top() )
         {
           result = m_second_player->get_center_of_mass().distance
             ( get_center_of_mass() ) < distance;
           index = 2;
-          left_orientation = 
+          left_orientation =
             m_second_player->get_center_of_mass().x <= get_center_of_mass().x;
         }
     }
@@ -534,21 +537,11 @@ bool ptb::woodpecker::scan
  */
 void ptb::woodpecker::search_players()
 {
-  plee::get_instance_message msg;
-  
-  if ( m_first_player.get() == NULL )
-    {
-      get_level_globals().send_message(plee::player_name(1),msg);
-      m_first_player = msg.get_instance();
-    }
-  
-  if ( ( game_variables::get_players_count() == 2 ) && 
-       ( m_second_player.get() == NULL ) )
-    {
-       plee::get_instance_message msg2;
-      get_level_globals().send_message(plee::player_name(2),msg2);
-      m_second_player = msg2.get_instance();
-    }
+  if ( m_first_player == (player*)NULL )
+    m_first_player = util::find_player( get_level_globals(), 1 );
+
+  if ( m_second_player == (player*)NULL )
+    m_second_player = util::find_player( get_level_globals(), 2 );
 } // woodpecker::search_players()
 
 /*----------------------------------------------------------------------------*/
@@ -570,13 +563,13 @@ void ptb::woodpecker::create_attack_movement(unsigned int index)
       m_attack_item->set_center_of_mass(m_second_player->get_center_of_mass());
       vect = m_second_player->get_center_of_mass() - get_center_of_mass();
     }
-    
+
   set_system_angle(atan2(vect.y,vect.x));
   get_rendering_attributes().flip(vect.x<0);
-  
+
   bear::universe::forced_join mvt_join(0.7);
   mvt_join.set_reference_item(*m_attack_item);
-     
+
   set_forced_movement(mvt_join);
 } // create_attack_movement()
 
@@ -628,9 +621,9 @@ void ptb::woodpecker::create_wig()
 {
   bear::engine::level_globals& glob = get_level_globals();
   bear::decorative_item* item = new bear::decorative_item;
-  
+
   item->set_sprite
-	  (glob.auto_sprite("gfx/forest/woodpecker/woodpecker-1.png","wig"));
+          (glob.auto_sprite("gfx/forest/woodpecker/woodpecker-1.png","wig"));
   item->set_phantom(false);
   item->set_artificial(false);
   item->set_can_move_items(false);
@@ -646,9 +639,9 @@ void ptb::woodpecker::create_wig()
 
   item->set_center_of_mass(get_center_of_mass());
 
-  new_item( *item );  
+  new_item( *item );
   CLAW_ASSERT( item->is_valid(),
-  	       "The wig of woodpecker isn't correctly initialized" );
+                 "The wig of woodpecker isn't correctly initialized" );
 } // woodpecker::create_wig()
 
 /*----------------------------------------------------------------------------*/
@@ -666,20 +659,21 @@ void ptb::woodpecker::create_feathers()
  * \brief Create a feather when the woodpecker dies.
  */
 void ptb::woodpecker::create_feather()
-{ 
+{
   // create reference
   bear::decorative_item* ref = new bear::decorative_item;
   set_feather_reference(ref);
   new_item( *ref );
   CLAW_ASSERT( ref->is_valid(),
-	       "The reference of feather of woodpecker isn't correctly initialized" );
+               "The reference of feather of woodpecker isn't correctly "
+               "initialized" );
 
   // create feather
   bear::decorative_item* item = new bear::decorative_item;
   set_feather(item);
-  new_item( *item );  
+  new_item( *item );
   CLAW_ASSERT( item->is_valid(),
-  	       "The feather of woodpecker isn't correctly initialized" );
+                 "The feather of woodpecker isn't correctly initialized" );
 
   // create movement
   bear::universe::forced_stay_around mvt;
@@ -701,7 +695,7 @@ void ptb::woodpecker::create_feather()
  */
 void ptb::woodpecker::set_feather_reference
 (bear::decorative_item* ref)
-{ 
+{
   ref->set_phantom(false);
   ref->set_artificial(false);
   ref->set_can_move_items(false);
@@ -726,25 +720,25 @@ void ptb::woodpecker::set_feather_reference
  * param item The item to set.
  */
 void ptb::woodpecker::set_feather(bear::decorative_item* item)
-{ 
+{
   bear::engine::level_globals& glob = get_level_globals();
-  
+
   item->set_friction(0.7);
-  
+
   double r = 3* ((double)rand() / (double)RAND_MAX);
-  if ( r >= 2 ) 
+  if ( r >= 2 )
     item->set_sprite
-      (glob.auto_sprite("gfx/forest/woodpecker/woodpecker-1.png", 
-			"main feather"));
-  else if ( r >= 1 ) 
+      (glob.auto_sprite("gfx/forest/woodpecker/woodpecker-1.png",
+                        "main feather"));
+  else if ( r >= 1 )
     item->set_sprite
-      (glob.auto_sprite("gfx/forest/woodpecker/woodpecker-1.png", 
-			"white feather"));
-  else 
+      (glob.auto_sprite("gfx/forest/woodpecker/woodpecker-1.png",
+                        "white feather"));
+  else
     item->set_sprite
-      (glob.auto_sprite("gfx/forest/woodpecker/woodpecker-1.png", 
-			"red feather"));
- 
+      (glob.auto_sprite("gfx/forest/woodpecker/woodpecker-1.png",
+                        "red feather"));
+
   item->set_phantom(false);
   item->set_artificial(false);
   item->set_can_move_items(false);
